@@ -49,7 +49,7 @@ print "Dataset consists of %d faces" % n_samples
 
 
 ###############################################################################
-def plot_gallery(title, images):
+def plot_gallery(title, images, n_col=n_col, n_row=n_row):
     pl.figure(figsize=(2. * n_col, 2.26 * n_row))
     pl.suptitle(title, size=16)
     for i, comp in enumerate(images):
@@ -68,36 +68,41 @@ def plot_gallery(title, images):
 estimators = [
     ('Eigenfaces - RandomizedPCA',
      decomposition.RandomizedPCA(n_components=n_components, whiten=True),
-     True, False),
+     True),
 
     ('Non-negative components - NMF',
      decomposition.NMF(n_components=n_components, init='nndsvda', beta=5.0,
                        tol=5e-3, sparseness='components'),
-     False, False),
+     False),
 
     ('Independent components - FastICA',
      decomposition.FastICA(n_components=n_components, whiten=True,
                            max_iter=10),
-     True, True),
+     True),
 
     ('Sparse comp. - MiniBatchSparsePCA',
      decomposition.MiniBatchSparsePCA(n_components=n_components, alpha=0.8,
-                                      n_iter=100, chunk_size=3,
+                                      n_iter=100, batch_size=3,
                                       random_state=rng),
-     True, False),
+     True),
 
     ('MiniBatchDictionaryLearning',
-    decomposition.MiniBatchDictionaryLearning(n_atoms=15, alpha=0.1,
-                                              n_iter=50, chunk_size=3,
-                                              random_state=rng),
-     True, False),
+        decomposition.MiniBatchDictionaryLearning(n_components=15, alpha=0.1,
+                                                  n_iter=50, batch_size=3,
+                                                  random_state=rng),
+     True),
 
     ('Cluster centers - MiniBatchKMeans',
-     MiniBatchKMeans(k=n_components, tol=1e-3, batch_size=20, max_iter=50,
-                     random_state=rng),
-     True, False)
+        MiniBatchKMeans(n_clusters=n_components, tol=1e-3, batch_size=20,
+                        max_iter=50, random_state=rng),
+     True),
+
+    ('Factor Analysis components - FA',
+     decomposition.FactorAnalysis(n_components=n_components, max_iter=2),
+     True),
 ]
 
+
 ###############################################################################
 # Plot a sample of the input data
 
@@ -106,14 +111,12 @@ plot_gallery("First centered Olivetti faces", faces_centered[:n_components])
 ###############################################################################
 # Do the estimation and plot it
 
-for name, estimator, center, transpose in estimators:
+for name, estimator, center in estimators:
     print "Extracting the top %d %s..." % (n_components, name)
     t0 = time()
     data = faces
     if center:
         data = faces_centered
-    if transpose:
-        data = data.T
     estimator.fit(data)
     train_time = (time() - t0)
     print "done in %0.3fs" % train_time
@@ -121,8 +124,10 @@ for name, estimator, center, transpose in estimators:
         components_ = estimator.cluster_centers_
     else:
         components_ = estimator.components_
-    if transpose:
-        components_ = components_.T
+    if hasattr(estimator, 'noise_variance_'):
+        plot_gallery("Pixelwise variance",
+                     estimator.noise_variance_.reshape(1, -1), n_col=1,
+                     n_row=1)
     plot_gallery('%s - Train time %.1fs' % (name, train_time),
                  components_[:n_components])
 
