@@ -90,16 +90,16 @@ pipeline = Pipeline([
 ])
 
 parameters = {
-# uncommenting more parameters will give better exploring power but will
-# increase processing time in a combinatorial way
+    # uncommenting more parameters will give better exploring power but will
+    # increase processing time in a combinatorial way
     'vect__max_df': (0.5, 0.75, 1.0),
-#    'vect__max_features': (None, 5000, 10000, 50000),
+    #'vect__max_features': (None, 5000, 10000, 50000),
     'vect__max_n': (1, 2),  # words or bigrams
-#    'tfidf__use_idf': (True, False),
-#    'tfidf__norm': ('l1', 'l2'),
+    #'tfidf__use_idf': (True, False),
+    #'tfidf__norm': ('l1', 'l2'),
     'clf__alpha': (0.00001, 0.000001),
     'clf__penalty': ('l2', 'elasticnet'),
-#    'clf__n_iter': (10, 50, 80),
+    #'clf__n_iter': (10, 50, 80),
 }
 
 if __name__ == "__main__":
@@ -119,8 +119,8 @@ if __name__ == "__main__":
     print "done in %0.3fs" % (time() - t0)
     print
 
-    print "Best score: %0.3f" % grid_search.best_score
+    print "Best score: %0.3f" % grid_search.best_score_
     print "Best parameters set:"
-    best_parameters = grid_search.best_estimator.get_params()
+    best_parameters = grid_search.best_estimator_.get_params()
     for param_name in sorted(parameters.keys()):
         print "\t%s: %r" % (param_name, best_parameters[param_name])
