@@ -8,6 +8,15 @@ with various manifold learning methods.
 
 For a discussion and comparison of these algorithms, see the
 :ref:`manifold module page <manifold>`
+
+For a similiar example, where the methods are applied to a
+sphere dataset, see :ref:`example_manifold_plot_manifold_sphere.py`
+
+Note that the purpose of the MDS is to find a low-dimensional
+representation of the data (here 2D) in which the distances respect well
+the distances in the original high-dimensional space, unlike other
+manifold-learning algorithms, it does not seeks an isotropic
+representation of the data in the low-dimensional space.
 """
 
 # Author: Jake Vanderplas -- <vanderplas@astro.washington.edu>
@@ -26,21 +35,21 @@ from sklearn import manifold, datasets
 Axes3D
 
 n_points = 1000
-X, color = datasets.samples_generator.make_s_curve(n_points)
+X, color = datasets.samples_generator.make_s_curve(n_points, random_state=0)
 n_neighbors = 10
 n_components = 2
 
-fig = pl.figure(figsize=(12, 8))
+fig = pl.figure(figsize=(15, 8))
 pl.suptitle("Manifold Learning with %i points, %i neighbors"
-               % (1000, n_neighbors), fontsize=14)
+            % (1000, n_neighbors), fontsize=14)
 
 try:
     # compatibility matplotlib < 1.0
-    ax = fig.add_subplot(231, projection='3d')
+    ax = fig.add_subplot(241, projection='3d')
     ax.scatter(X[:, 0], X[:, 1], X[:, 2], c=color, cmap=pl.cm.Spectral)
     ax.view_init(4, -72)
 except:
-    ax = fig.add_subplot(231, projection='3d')
+    ax = fig.add_subplot(241, projection='3d')
     pl.scatter(X[:, 0], X[:, 2], c=color, cmap=pl.cm.Spectral)
 
 methods = ['standard', 'ltsa', 'hessian', 'modified']
@@ -54,7 +63,7 @@ for i, method in enumerate(methods):
     t1 = time()
     print "%s: %.2g sec" % (methods[i], t1 - t0)
 
-    ax = fig.add_subplot(232 + i)
+    ax = fig.add_subplot(242 + i)
     pl.scatter(Y[:, 0], Y[:, 1], c=color, cmap=pl.cm.Spectral)
     pl.title("%s (%.2g sec)" % (labels[i], t1 - t0))
     ax.xaxis.set_major_formatter(NullFormatter())
@@ -65,11 +74,38 @@ t0 = time()
 Y = manifold.Isomap(n_neighbors, n_components).fit_transform(X)
 t1 = time()
 print "Isomap: %.2g sec" % (t1 - t0)
-ax = fig.add_subplot(236)
+ax = fig.add_subplot(246)
 pl.scatter(Y[:, 0], Y[:, 1], c=color, cmap=pl.cm.Spectral)
 pl.title("Isomap (%.2g sec)" % (t1 - t0))
 ax.xaxis.set_major_formatter(NullFormatter())
 ax.yaxis.set_major_formatter(NullFormatter())
 pl.axis('tight')
 
+
+t0 = time()
+mds = manifold.MDS(n_components, max_iter=100, n_init=1)
+Y = mds.fit_transform(X)
+t1 = time()
+print "MDS: %.2g sec" % (t1 - t0)
+ax = fig.add_subplot(247)
+pl.scatter(Y[:, 0], Y[:, 1], c=color, cmap=pl.cm.Spectral)
+pl.title("MDS (%.2g sec)" % (t1 - t0))
+ax.xaxis.set_major_formatter(NullFormatter())
+ax.yaxis.set_major_formatter(NullFormatter())
+pl.axis('tight')
+
+
+t0 = time()
+se = manifold.SpectralEmbedding(n_components=n_components,
+                                n_neighbors=n_neighbors)
+Y = se.fit_transform(X)
+t1 = time()
+print "SpectralEmbedding: %.2g sec" % (t1 - t0)
+ax = fig.add_subplot(248)
+pl.scatter(Y[:, 0], Y[:, 1], c=color, cmap=pl.cm.Spectral)
+pl.title("SpectralEmbedding (%.2g sec)" % (t1 - t0))
+ax.xaxis.set_major_formatter(NullFormatter())
+ax.yaxis.set_major_formatter(NullFormatter())
+pl.axis('tight')
+
 pl.show()
