@@ -13,8 +13,9 @@ selects the informative features and that these have larger SVM weights.
 
 In the total set of features, only the 4 first ones are significant. We
 can see that they have the highest score with univariate feature
-selection. The SVM attributes small weights to these features, but these
-weight are non zero. Applying univariate feature selection before the SVM
+selection. The SVM assigns a large weight to one of these features, but also
+Selects many of the non-informative features.
+Applying univariate feature selection before the SVM
 increases the SVM weight attributed to the significant features, and will
 thus improve classification.
 """
@@ -29,42 +30,51 @@ from sklearn.feature_selection import SelectPercentile, f_classif
 ###############################################################################
 # import some data to play with
 
-# The IRIS dataset
+# The iris dataset
 iris = datasets.load_iris()
 
 # Some noisy data not correlated
-E = np.random.normal(size=(len(iris.data), 35))
+E = np.random.uniform(0, 0.1, size=(len(iris.data), 20))
 
 # Add the noisy data to the informative features
-x = np.hstack((iris.data, E))
+X = np.hstack((iris.data, E))
 y = iris.target
 
 ###############################################################################
 pl.figure(1)
 pl.clf()
 
-x_indices = np.arange(x.shape[-1])
+X_indices = np.arange(X.shape[-1])
 
 ###############################################################################
 # Univariate feature selection with F-test for feature scoring
 # We use the default selection function: the 10% most significant features
 selector = SelectPercentile(f_classif, percentile=10)
-selector.fit(x, y)
-scores = -np.log10(selector.scores_)
+selector.fit(X, y)
+scores = -np.log10(selector.pvalues_)
 scores /= scores.max()
-pl.bar(x_indices - .45, scores, width=.3,
-        label=r'Univariate score ($-Log(p_{value})$)',
-        color='g')
+pl.bar(X_indices - .45, scores, width=.2,
+       label=r'Univariate score ($-Log(p_{value})$)', color='g')
 
 ###############################################################################
 # Compare to the weights of an SVM
 clf = svm.SVC(kernel='linear')
-clf.fit(x, y)
+clf.fit(X, y)
 
 svm_weights = (clf.coef_ ** 2).sum(axis=0)
 svm_weights /= svm_weights.max()
-pl.bar(x_indices - .15, svm_weights, width=.3, label='SVM weight',
-        color='r')
+
+pl.bar(X_indices - .25, svm_weights, width=.2, label='SVM weight', color='r')
+
+clf_selected = svm.SVC(kernel='linear')
+clf_selected.fit(selector.transform(X), y)
+
+svm_weights_selected = (clf_selected.coef_ ** 2).sum(axis=0)
+svm_weights_selected /= svm_weights_selected.max()
+
+pl.bar(X_indices[selector.get_support()] - .05, svm_weights_selected, width=.2,
+       label='SVM weights after selection', color='b')
+
 
 pl.title("Comparing feature selection")
 pl.xlabel('Feature number')
