@@ -1,20 +1,24 @@
 """Testing for K-means"""
+from cStringIO import StringIO
+import sys
+import warnings
 
 import numpy as np
-import warnings
 from scipy import sparse as sp
-from numpy.testing import assert_equal
-from numpy.testing import assert_array_equal
-from numpy.testing import assert_array_almost_equal
-from nose import SkipTest
-from nose.tools import assert_almost_equal
-from nose.tools import assert_raises
-from nose.tools import assert_true
+
+from sklearn.utils.testing import assert_equal
+from sklearn.utils.testing import assert_array_equal
+from sklearn.utils.testing import assert_array_almost_equal
+from sklearn.utils.testing import SkipTest
+from sklearn.utils.testing import assert_almost_equal
+from sklearn.utils.testing import assert_raises
+from sklearn.utils.testing import assert_true
 
 from sklearn.utils.testing import assert_greater
 from sklearn.utils.testing import assert_less
+from sklearn.utils.fixes import unique
 from sklearn.metrics.cluster import v_measure_score
-from sklearn.cluster import KMeans
+from sklearn.cluster import KMeans, k_means
 from sklearn.cluster import MiniBatchKMeans
 from sklearn.cluster.k_means_ import _labels_inertia
 from sklearn.cluster.k_means_ import _mini_batch_step
@@ -166,8 +170,38 @@ def _check_fitted_model(km):
 
 
 def test_k_means_plus_plus_init():
-    k_means = KMeans(init="k-means++", k=n_clusters, random_state=42).fit(X)
-    _check_fitted_model(k_means)
+    km = KMeans(init="k-means++", n_clusters=n_clusters,
+                     random_state=42).fit(X)
+    _check_fitted_model(km)
+
+
+def test_k_means_check_fitted():
+    km = KMeans(n_clusters=n_clusters, random_state=42)
+    assert_raises(AttributeError, km._check_fitted)
+
+
+def test_k_means_new_centers():
+    # Explore the part of the code where a new center is reassigned
+    X = np.array([[0, 0, 1, 1],
+                  [0, 0, 0, 0],
+                  [0, 1, 0, 0],
+                  [0, 0, 0, 0],
+                  [0, 0, 0, 0],
+                  [0, 1, 0, 0]])
+    labels = [0, 1, 2, 1, 1, 2]
+    bad_centers = np.array([[+0,  1,  0,  0],
+                            [.2,  0, .2, .2],
+                            [+0,  0,  0,  0]])
+
+    km = KMeans(n_clusters=3, init=bad_centers, n_init=1, max_iter=10,
+                random_state=1)
+    for this_X in (X, sp.coo_matrix(X)):
+        km.fit(this_X)
+        this_labels = km.labels_
+        # Reorder the labels so that the first instance is in cluster 0,
+        # the second in cluster 1, ...
+        this_labels = unique(this_labels, return_index=True)[1][this_labels]
+        np.testing.assert_array_equal(this_labels, labels)
 
 
 def _get_mac_os_version():
@@ -179,64 +213,77 @@ def _get_mac_os_version():
 
 
 def test_k_means_plus_plus_init_2_jobs():
-    if _get_mac_os_version() == '10.7':
+    if _get_mac_os_version() >= '10.7':
         raise SkipTest('Multi-process bug in Mac OS X Lion (see issue #636)')
-    k_means = KMeans(init="k-means++", k=n_clusters, n_jobs=2,
+    km = KMeans(init="k-means++", n_clusters=n_clusters, n_jobs=2,
                      random_state=42).fit(X)
-    _check_fitted_model(k_means)
+    _check_fitted_model(km)
 
 
 def test_k_means_plus_plus_init_sparse():
-    k_means = KMeans(init="k-means++", k=n_clusters, random_state=42)
-    k_means.fit(X_csr)
-    _check_fitted_model(k_means)
+    km = KMeans(init="k-means++", n_clusters=n_clusters, random_state=42)
+    km.fit(X_csr)
+    _check_fitted_model(km)
 
 
 def test_k_means_random_init():
-    k_means = KMeans(init="random", k=n_clusters, random_state=42).fit(X)
-    _check_fitted_model(k_means)
+    km = KMeans(init="random", n_clusters=n_clusters, random_state=42)
+    km.fit(X)
+    _check_fitted_model(km)
 
 
 def test_k_means_random_init_sparse():
-    k_means = KMeans(init="random", k=n_clusters, random_state=42).fit(X_csr)
-    _check_fitted_model(k_means)
+    km = KMeans(init="random", n_clusters=n_clusters, random_state=42)
+    km.fit(X_csr)
+    _check_fitted_model(km)
 
 
 def test_k_means_plus_plus_init_not_precomputed():
-    k_means = KMeans(init="k-means++", k=n_clusters, random_state=42,
+    km = KMeans(init="k-means++", n_clusters=n_clusters, random_state=42,
                      precompute_distances=False).fit(X)
-    _check_fitted_model(k_means)
+    _check_fitted_model(km)
 
 
 def test_k_means_random_init_not_precomputed():
-    k_means = KMeans(init="random", k=n_clusters, random_state=42,
+    km = KMeans(init="random", n_clusters=n_clusters, random_state=42,
                      precompute_distances=False).fit(X)
-    _check_fitted_model(k_means)
+    _check_fitted_model(km)
 
 
 def test_k_means_perfect_init():
-    k_means = KMeans(init=centers.copy(), k=n_clusters, random_state=42,
-                     n_init=1)
-    k_means.fit(X)
-    _check_fitted_model(k_means)
+    km = KMeans(init=centers.copy(), n_clusters=n_clusters, random_state=42,
+                n_init=1)
+    km.fit(X)
+    _check_fitted_model(km)
 
 
 def test_mb_k_means_plus_plus_init_dense_array():
-    mb_k_means = MiniBatchKMeans(init="k-means++", k=n_clusters,
+    mb_k_means = MiniBatchKMeans(init="k-means++", n_clusters=n_clusters,
                                  random_state=42)
     mb_k_means.fit(X)
     _check_fitted_model(mb_k_means)
 
 
+def test_mb_kmeans_verbose():
+    mb_k_means = MiniBatchKMeans(init="k-means++", n_clusters=n_clusters,
+                                 random_state=42, verbose=1)
+    old_stdout = sys.stdout
+    sys.stdout = StringIO()
+    try:
+        mb_k_means.fit(X)
+    finally:
+        sys.stdout = old_stdout
+
+
 def test_mb_k_means_plus_plus_init_sparse_matrix():
-    mb_k_means = MiniBatchKMeans(init="k-means++", k=n_clusters,
+    mb_k_means = MiniBatchKMeans(init="k-means++", n_clusters=n_clusters,
                                  random_state=42)
     mb_k_means.fit(X_csr)
     _check_fitted_model(mb_k_means)
 
 
 def test_minibatch_init_with_large_k():
-    mb_k_means = MiniBatchKMeans(init='k-means++', init_size=10, k=20)
+    mb_k_means = MiniBatchKMeans(init='k-means++', init_size=10, n_clusters=20)
     # Check that a warning is raised, as the number clusters is larger
     # than the init_size
     with warnings.catch_warnings(record=True) as warn_queue:
@@ -247,41 +294,73 @@ def test_minibatch_init_with_large_k():
 
 def test_minibatch_k_means_random_init_dense_array():
     # increase n_init to make random init stable enough
-    mb_k_means = MiniBatchKMeans(init="random", k=n_clusters,
+    mb_k_means = MiniBatchKMeans(init="random", n_clusters=n_clusters,
                                  random_state=42, n_init=10).fit(X)
     _check_fitted_model(mb_k_means)
 
 
 def test_minibatch_k_means_random_init_sparse_csr():
     # increase n_init to make random init stable enough
-    mb_k_means = MiniBatchKMeans(init="random", k=n_clusters,
+    mb_k_means = MiniBatchKMeans(init="random", n_clusters=n_clusters,
                                  random_state=42, n_init=10).fit(X_csr)
     _check_fitted_model(mb_k_means)
 
 
 def test_minibatch_k_means_perfect_init_dense_array():
-    mb_k_means = MiniBatchKMeans(init=centers.copy(), k=n_clusters,
+    mb_k_means = MiniBatchKMeans(init=centers.copy(), n_clusters=n_clusters,
                                  random_state=42).fit(X)
     _check_fitted_model(mb_k_means)
 
 
 def test_minibatch_k_means_perfect_init_sparse_csr():
-    mb_k_means = MiniBatchKMeans(init=centers.copy(), k=n_clusters,
+    mb_k_means = MiniBatchKMeans(init=centers.copy(), n_clusters=n_clusters,
                                  random_state=42).fit(X_csr)
     _check_fitted_model(mb_k_means)
 
 
+def test_minibatch_reassign():
+    # Give a perfect initialization, but a large reassignment_ratio,
+    # as a result all the centers should be reassigned and the model
+    # should not longer be good
+    mb_k_means = MiniBatchKMeans(init=centers.copy(),
+                                 n_clusters=n_clusters, batch_size=1,
+                                 random_state=42)
+    mb_k_means.fit(X)
+    centers_before = mb_k_means.cluster_centers_.copy()
+    try:
+        old_stdout = sys.stdout
+        sys.stdout = StringIO()
+        # Turn on verbosity to smoke test the display code
+        _mini_batch_step(X, (X ** 2).sum(axis=1), mb_k_means.cluster_centers_,
+                         mb_k_means.counts_, np.zeros(X.shape[1], np.double),
+                         False, random_reassign=True, random_state=42,
+                         reassignment_ratio=1, verbose=True)
+    finally:
+        sys.stdout = old_stdout
+    centers_after = mb_k_means.cluster_centers_.copy()
+    # Check that all the centers have moved
+    assert_greater(((centers_before - centers_after)**2).sum(axis=1).min(),
+                   .2)
+
+
 def test_sparse_mb_k_means_callable_init():
 
     def test_init(X, k, random_state):
         return centers
 
-    mb_k_means = MiniBatchKMeans(init=test_init, random_state=42).fit(X_csr)
+    # Small test to check that giving the wrong number of centers
+    # raises a meaningful error
+    assert_raises(ValueError,
+                  MiniBatchKMeans(init=test_init, random_state=42).fit, X_csr)
+
+    # Now check that the fit actually works
+    mb_k_means = MiniBatchKMeans(n_clusters=3, init=test_init,
+                                 random_state=42).fit(X_csr)
     _check_fitted_model(mb_k_means)
 
 
 def test_mini_batch_k_means_random_init_partial_fit():
-    km = MiniBatchKMeans(k=n_clusters, init="random", random_state=42)
+    km = MiniBatchKMeans(n_clusters=n_clusters, init="random", random_state=42)
 
     # use the partial_fit API for online learning
     for X_minibatch in np.array_split(X, 10):
@@ -293,14 +372,20 @@ def test_mini_batch_k_means_random_init_partial_fit():
 
 
 def test_minibatch_default_init_size():
-    mb_k_means = MiniBatchKMeans(init=centers.copy(), k=n_clusters,
+    mb_k_means = MiniBatchKMeans(init=centers.copy(), n_clusters=n_clusters,
                                  batch_size=10, random_state=42).fit(X)
     assert_equal(mb_k_means.init_size_, 3 * mb_k_means.batch_size)
     _check_fitted_model(mb_k_means)
 
 
+def test_minibatch_tol():
+    mb_k_means = MiniBatchKMeans(n_clusters=n_clusters, batch_size=10,
+                                 random_state=42, tol=.01).fit(X)
+    _check_fitted_model(mb_k_means)
+
+
 def test_minibatch_set_init_size():
-    mb_k_means = MiniBatchKMeans(init=centers.copy(), k=n_clusters,
+    mb_k_means = MiniBatchKMeans(init=centers.copy(), n_clusters=n_clusters,
                                  init_size=666, random_state=42).fit(X)
     assert_equal(mb_k_means.init_size, 666)
     assert_equal(mb_k_means.init_size_, n_samples)
@@ -308,20 +393,21 @@ def test_minibatch_set_init_size():
 
 
 def test_k_means_invalid_init():
-    k_means = KMeans(init="invalid", n_init=1, k=n_clusters)
-    assert_raises(ValueError, k_means.fit, X)
+    km = KMeans(init="invalid", n_init=1, n_clusters=n_clusters)
+    assert_raises(ValueError, km.fit, X)
 
 
 def test_mini_match_k_means_invalid_init():
-    k_means = MiniBatchKMeans(init="invalid", n_init=1, k=n_clusters)
-    assert_raises(ValueError, k_means.fit, X)
+    km = MiniBatchKMeans(init="invalid", n_init=1, n_clusters=n_clusters)
+    assert_raises(ValueError, km.fit, X)
 
 
 def test_k_means_copyx():
     """Check if copy_x=False returns nearly equal X after de-centering."""
     my_X = X.copy()
-    k_means = KMeans(copy_x=False, k=n_clusters, random_state=42).fit(my_X)
-    _check_fitted_model(k_means)
+    km = KMeans(copy_x=False, n_clusters=n_clusters, random_state=42)
+    km.fit(my_X)
+    _check_fitted_model(km)
 
     # check if my_X is centered
     assert_array_almost_equal(my_X, X)
@@ -337,44 +423,46 @@ def test_k_means_non_collapsed():
     """
     my_X = np.array([[1.1, 1.1], [0.9, 1.1], [1.1, 0.9], [0.9, 1.1]])
     array_init = np.array([[1.0, 1.0], [5.0, 5.0], [-5.0, -5.0]])
-    k_means = KMeans(init=array_init, k=3, random_state=42, n_init=1)
-    k_means.fit(my_X)
+    km = KMeans(init=array_init, n_clusters=3, random_state=42, n_init=1)
+    km.fit(my_X)
 
     # centers must not been collapsed
-    assert_equal(len(np.unique(k_means.labels_)), 3)
+    assert_equal(len(np.unique(km.labels_)), 3)
 
-    centers = k_means.cluster_centers_
+    centers = km.cluster_centers_
     assert_true(np.linalg.norm(centers[0] - centers[1]) >= 0.1)
     assert_true(np.linalg.norm(centers[0] - centers[2]) >= 0.1)
     assert_true(np.linalg.norm(centers[1] - centers[2]) >= 0.1)
 
 
 def test_predict():
-    k_means = KMeans(k=n_clusters, random_state=42)
+    km = KMeans(n_clusters=n_clusters, random_state=42)
 
-    k_means.fit(X)
+    km.fit(X)
 
     # sanity check: predict centroid labels
-    pred = k_means.predict(k_means.cluster_centers_)
+    pred = km.predict(km.cluster_centers_)
     assert_array_equal(pred, np.arange(n_clusters))
 
     # sanity check: re-predict labeling for training set samples
-    pred = k_means.predict(X)
-    assert_array_equal(pred, k_means.labels_)
+    pred = km.predict(X)
+    assert_array_equal(pred, km.labels_)
 
     # re-predict labels for training set using fit_predict
-    pred = k_means.fit_predict(X)
-    assert_array_equal(pred, k_means.labels_)
+    pred = km.fit_predict(X)
+    assert_array_equal(pred, km.labels_)
 
 
 def test_score():
-    s1 = KMeans(k=n_clusters, max_iter=1, random_state=42).fit(X).score(X)
-    s2 = KMeans(k=n_clusters, max_iter=10, random_state=42).fit(X).score(X)
+    km1 = KMeans(n_clusters=n_clusters, max_iter=1, random_state=42)
+    s1 = km1.fit(X).score(X)
+    km2 = KMeans(n_clusters=n_clusters, max_iter=10, random_state=42)
+    s2 = km2.fit(X).score(X)
     assert_greater(s2, s1)
 
 
 def test_predict_minibatch_dense_input():
-    mb_k_means = MiniBatchKMeans(k=n_clusters, random_state=40).fit(X)
+    mb_k_means = MiniBatchKMeans(n_clusters=n_clusters, random_state=40).fit(X)
 
     # sanity check: predict centroid labels
     pred = mb_k_means.predict(mb_k_means.cluster_centers_)
@@ -386,7 +474,7 @@ def test_predict_minibatch_dense_input():
 
 
 def test_predict_minibatch_kmeanspp_init_sparse_input():
-    mb_k_means = MiniBatchKMeans(k=n_clusters, init='k-means++',
+    mb_k_means = MiniBatchKMeans(n_clusters=n_clusters, init='k-means++',
                                  n_init=10).fit(X_csr)
 
     # sanity check: re-predict labeling for training set samples
@@ -402,7 +490,7 @@ def test_predict_minibatch_kmeanspp_init_sparse_input():
 
 
 def test_predict_minibatch_random_init_sparse_input():
-    mb_k_means = MiniBatchKMeans(k=n_clusters, init='random',
+    mb_k_means = MiniBatchKMeans(n_clusters=n_clusters, init='random',
                                  n_init=10).fit(X_csr)
 
     # sanity check: re-predict labeling for training set samples
@@ -424,18 +512,19 @@ def test_input_dtypes():
     init_int = X_int[:2]
 
     fitted_models = [
-        KMeans(k=2).fit(X_list),
-        KMeans(k=2).fit(X_int),
-        KMeans(k=2, init=init_int, n_init=1).fit(X_list),
-        KMeans(k=2, init=init_int, n_init=1).fit(X_int),
+        KMeans(n_clusters=2).fit(X_list),
+        KMeans(n_clusters=2).fit(X_int),
+        KMeans(n_clusters=2, init=init_int, n_init=1).fit(X_list),
+        KMeans(n_clusters=2, init=init_int, n_init=1).fit(X_int),
         # mini batch kmeans is very unstable on such a small dataset hence
         # we use many inits
-        MiniBatchKMeans(k=2, n_init=10, batch_size=2).fit(X_list),
-        MiniBatchKMeans(k=2, n_init=10, batch_size=2).fit(X_int),
-        MiniBatchKMeans(k=2, n_init=10, batch_size=2).fit(X_int_csr),
-        MiniBatchKMeans(k=2, batch_size=2, init=init_int).fit(X_list),
-        MiniBatchKMeans(k=2, batch_size=2, init=init_int).fit(X_int),
-        MiniBatchKMeans(k=2, batch_size=2, init=init_int).fit(X_int_csr),
+        MiniBatchKMeans(n_clusters=2, n_init=10, batch_size=2).fit(X_list),
+        MiniBatchKMeans(n_clusters=2, n_init=10, batch_size=2).fit(X_int),
+        MiniBatchKMeans(n_clusters=2, n_init=10, batch_size=2).fit(X_int_csr),
+        MiniBatchKMeans(n_clusters=2, batch_size=2, init=init_int).fit(X_list),
+        MiniBatchKMeans(n_clusters=2, batch_size=2, init=init_int).fit(X_int),
+        MiniBatchKMeans(n_clusters=2, batch_size=2,
+                        init=init_int).fit(X_int_csr),
     ]
     expected_labels = [0, 1, 1, 0, 0, 1]
     scores = np.array([v_measure_score(expected_labels, km.labels_)
@@ -444,9 +533,9 @@ def test_input_dtypes():
 
 
 def test_transform():
-    k_means = KMeans(k=n_clusters)
-    k_means.fit(X)
-    X_new = k_means.transform(k_means.cluster_centers_)
+    km = KMeans(n_clusters=n_clusters)
+    km.fit(X)
+    X_new = km.transform(km.cluster_centers_)
 
     for c in range(n_clusters):
         assert_equal(X_new[c, c], 0)
@@ -455,6 +544,12 @@ def test_transform():
                 assert_greater(X_new[c, c2], 0)
 
 
+def test_fit_transform():
+    X1 = KMeans(n_clusters=3, random_state=51).fit(X).transform(X)
+    X2 = KMeans(n_clusters=3, random_state=51).fit_transform(X)
+    assert_array_equal(X1, X2)
+
+
 def test_n_init():
     """Check that increasing the number of init increases the quality"""
     n_runs = 5
@@ -462,7 +557,7 @@ def test_n_init():
     inertia = np.zeros((len(n_init_range), n_runs))
     for i, n_init in enumerate(n_init_range):
         for j in range(n_runs):
-            km = KMeans(k=n_clusters, init="random", n_init=n_init,
+            km = KMeans(n_clusters=n_clusters, init="random", n_init=n_init,
                         random_state=j).fit(X)
             inertia[i, j] = km.inertia_
 
@@ -471,3 +566,32 @@ def test_n_init():
                    " when n_init is increasing.") % list(inertia)
     for i in range(len(n_init_range) - 1):
         assert_true(inertia[i] >= inertia[i + 1], failure_msg)
+
+
+def test_k_means_function():
+    # test calling the k_means function directly
+    # catch output
+    old_stdout = sys.stdout
+    sys.stdout = StringIO()
+    try:
+        cluster_centers, labels, inertia = k_means(X, n_clusters=n_clusters,
+                                                   verbose=True)
+    finally:
+        sys.stdout = old_stdout
+    centers = cluster_centers
+    assert_equal(centers.shape, (n_clusters, n_features))
+
+    labels = labels
+    assert_equal(np.unique(labels).shape[0], n_clusters)
+
+    # check that the labels assignements are perfect (up to a permutation)
+    assert_equal(v_measure_score(true_labels, labels), 1.0)
+    assert_greater(inertia, 0.0)
+
+    # check warning when centers are passed
+    with warnings.catch_warnings(record=True) as w:
+        k_means(X, n_clusters=n_clusters, init=centers)
+        assert_equal(len(w), 1)
+
+    # to many clusters desired
+    assert_raises(ValueError, k_means, X, n_clusters=X.shape[0] + 1)
