@@ -83,8 +83,8 @@ def fetch_olivetti_faces(data_home=None, shuffle=False, random_state=0,
     if not exists(data_home):
         makedirs(data_home)
     if not exists(join(data_home, TARGET_FILENAME)):
-        print 'downloading Olivetti faces from %s to %s' % (DATA_URL,
-                            data_home)
+        print('downloading Olivetti faces from %s to %s'
+              % (DATA_URL, data_home))
         fhandle = urllib2.urlopen(DATA_URL)
         buf = StringIO(fhandle.read())
         mfile = loadmat(buf)
