@@ -16,14 +16,14 @@ from numpy.lib.stride_tricks import as_strided
 
 from ..base import BaseEstimator, TransformerMixin
 from ..externals.joblib import Parallel, delayed, cpu_count
-from ..utils import array2d, check_random_state, gen_even_slices, deprecated
+from ..utils import array2d, check_random_state, gen_even_slices
 from ..utils.extmath import randomized_svd
-from ..linear_model import Lasso, orthogonal_mp_gram, lars_path
+from ..linear_model import Lasso, orthogonal_mp_gram, LassoLars, Lars
 
 
-def _sparse_encode(X, dictionary, gram=None, cov=None, algorithm='lasso_lars',
-                  n_nonzero_coefs=None, alpha=None, copy_cov=True,
-                  init=None, max_iter=1000):
+def _sparse_encode(X, dictionary, gram, cov=None, algorithm='lasso_lars',
+                   regularization=None, copy_cov=True,
+                   init=None, max_iter=1000):
     """Generic sparse coding
 
     Each column of the result is the solution to a Lasso problem.
@@ -33,14 +33,15 @@ def _sparse_encode(X, dictionary, gram=None, cov=None, algorithm='lasso_lars',
     X: array of shape (n_samples, n_features)
         Data matrix.
 
-    dictionary: array of shape (n_atoms, n_features)
+    dictionary: array of shape (n_components, n_features)
         The dictionary matrix against which to solve the sparse coding of
         the data. Some of the algorithms assume normalized rows.
 
-    gram: array, shape=(n_atoms, n_atoms)
+    gram: None | array, shape=(n_components, n_components)
         Precomputed Gram matrix, dictionary * dictionary'
+        gram can be None if method is 'threshold'.
 
-    cov: array, shape=(n_atoms, n_samples)
+    cov: array, shape=(n_components, n_samples)
         Precomputed covariance, dictionary * X'
 
     algorithm: {'lasso_lars', 'lasso_cd', 'lars', 'omp', 'threshold'}
@@ -50,24 +51,15 @@ def _sparse_encode(X, dictionary, gram=None, cov=None, algorithm='lasso_lars',
         Lasso solution (linear_model.Lasso). lasso_lars will be faster if
         the estimated components are sparse.
         omp: uses orthogonal matching pursuit to estimate the sparse solution
-        threshold: squashes to zero all coefficients less than alpha from
-        the projection dictionary * data'
+        threshold: squashes to zero all coefficients less than regularization
+        from the projection dictionary * data'
 
-    n_nonzero_coefs: int, 0.1 * n_features by default
-        Number of nonzero coefficients to target in each column of the
-        solution. This is only used by `algorithm='lars'` and `algorithm='omp'`
-        and is overridden by `alpha` in the `omp` case.
+    regularization : int | float
+        The regularization parameter. It corresponds to alpha when
+        algorithm is 'lasso_lars', 'lasso_cd' or 'threshold'.
+        Otherwise it corresponds to n_nonzero_coefs.
 
-    alpha: float, 1. by default
-        If `algorithm='lasso_lars'` or `algorithm='lasso_cd'`, `alpha` is the
-        penalty applied to the L1 norm.
-        If `algorithm='threshold'`, `alpha` is the absolute value of the
-        threshold below which coefficients will be squashed to zero.
-        If `algorithm='omp'`, `alpha` is the tolerance parameter: the value of
-        the reconstruction error targeted. In this case, it overrides
-        `n_nonzero_coefs`.
-
-    init: array of shape (n_samples, n_atoms)
+    init: array of shape (n_samples, n_components)
         Initialization value of the sparse code. Only used if
         `algorithm='lasso_cd'`.
 
@@ -90,94 +82,63 @@ def _sparse_encode(X, dictionary, gram=None, cov=None, algorithm='lasso_lars',
     sklearn.linear_model.Lasso
     SparseCoder
     """
-    alpha = float(alpha) if alpha is not None else None
-    dictionary = np.asarray(dictionary)
-    X = np.asarray(X)
     if X.ndim == 1:
         X = X[:, np.newaxis]
     n_samples, n_features = X.shape
-    n_atoms = dictionary.shape[0]
-    # This will always use Gram
-    if gram is None:
-        # I think it's never safe to overwrite Gram when n_features > 1
-        # but I'd like to avoid the complicated logic.
-        # The parameter could be removed in this case. Discuss.
-        gram = np.dot(dictionary, dictionary.T)
     if cov is None and algorithm != 'lasso_cd':
         # overwriting cov is safe
         copy_cov = False
         cov = np.dot(dictionary, X.T)
 
     if algorithm == 'lasso_lars':
-        if alpha is None:
-            alpha = 1.
-        alpha /= n_features  # account for scaling
+        alpha = float(regularization) / n_features  # account for scaling
         try:
-            new_code = np.empty((n_samples, n_atoms))
             err_mgt = np.seterr(all='ignore')
-            for k in range(n_samples):
-                # A huge amount of time is spent in this loop. It needs to be
-                # tight.
-                _, _, coef_path_ = lars_path(dictionary.T, X[k], Xy=cov[:, k],
-                                             Gram=gram, alpha_min=alpha,
-                                             method='lasso')
-                new_code[k] = coef_path_[:, -1]
+            lasso_lars = LassoLars(alpha=alpha, fit_intercept=False,
+                                   verbose=False, normalize=False,
+                                   precompute=gram, fit_path=False)
+            lasso_lars.fit(dictionary.T, X.T, Xy=cov)
+            new_code = lasso_lars.coef_
         finally:
             np.seterr(**err_mgt)
 
     elif algorithm == 'lasso_cd':
-        if alpha is None:
-            alpha = 1.
-        alpha /= n_features  # account for scaling
-        new_code = np.empty((n_samples, n_atoms))
+        alpha = float(regularization) / n_features  # account for scaling
         clf = Lasso(alpha=alpha, fit_intercept=False, precompute=gram,
                     max_iter=max_iter)
-        for k in xrange(n_samples):
-            # A huge amount of time is spent in this loop. It needs to be
-            # tight
-
-            if init is not None:
-                clf.coef_ = init[k]  # Init with previous value of the code
-            clf.fit(dictionary.T, X[k])
-            new_code[k] = clf.coef_
+        clf.fit(dictionary.T, X.T, Xy=cov, coef_init=init)
+        new_code = clf.coef_
 
     elif algorithm == 'lars':
-        if n_nonzero_coefs is None:
-            n_nonzero_coefs = max(n_features / 10, 1)
         try:
-            new_code = np.empty((n_samples, n_atoms))
             err_mgt = np.seterr(all='ignore')
-            for k in xrange(n_samples):
-                # A huge amount of time is spent in this loop. It needs to be
-                # tight.
-                _, _, coef_path_ = lars_path(dictionary.T, X[k], Xy=cov[:, k],
-                                             Gram=gram, method='lar',
-                                             max_iter=n_nonzero_coefs)
-                new_code[k] = coef_path_[:, -1]
+            lars = Lars(fit_intercept=False, verbose=False, normalize=False,
+                        precompute=gram, n_nonzero_coefs=int(regularization),
+                        fit_path=False)
+            lars.fit(dictionary.T, X.T, Xy=cov)
+            new_code = lars.coef_
         finally:
             np.seterr(**err_mgt)
 
     elif algorithm == 'threshold':
-        if alpha is None:
-            alpha = 1.
-        new_code = (np.sign(cov) * np.maximum(np.abs(cov) - alpha, 0)).T
+        new_code = ((np.sign(cov) *
+                    np.maximum(np.abs(cov) - regularization, 0)).T)
 
     elif algorithm == 'omp':
-        if n_nonzero_coefs is None and alpha is None:
-            n_nonzero_coefs = max(n_features / 10, 1)
         norms_squared = np.sum((X ** 2), axis=1)
-        new_code = orthogonal_mp_gram(gram, cov, n_nonzero_coefs, alpha,
-                                      norms_squared, copy_Xy=copy_cov
-                                      ).T
+        new_code = orthogonal_mp_gram(gram, cov, regularization, None,
+                                      norms_squared, copy_Xy=copy_cov).T
     else:
-        raise NotImplemented('Sparse coding method %s not implemented' %
-                             algorithm)
+        raise ValueError('Sparse coding method must be "lasso_lars" '
+                         '"lasso_cd",  "lasso", "threshold" or "omp", got %s.'
+                         % algorithm)
     return new_code
 
 
+# XXX : could be moved to the linear_model module
 def sparse_encode(X, dictionary, gram=None, cov=None, algorithm='lasso_lars',
-                  n_nonzero_coefs=None, alpha=None, copy_gram=None,
-                  copy_cov=True, init=None, max_iter=1000, n_jobs=1):
+                  n_nonzero_coefs=None, alpha=None, copy_cov=True, init=None,
+                  max_iter=1000, n_jobs=1):
     """Sparse coding
 
     Each row of the result is the solution to a sparse coding problem.
@@ -190,15 +151,15 @@ def sparse_encode(X, dictionary, gram=None, cov=None, algorithm='lasso_lars',
     X: array of shape (n_samples, n_features)
         Data matrix
 
-    dictionary: array of shape (n_atoms, n_features)
+    dictionary: array of shape (n_components, n_features)
         The dictionary matrix against which to solve the sparse coding of
         the data. Some of the algorithms assume normalized rows for meaningful
         output.
 
-    gram: array, shape=(n_atoms, n_atoms)
+    gram: array, shape=(n_components, n_components)
         Precomputed Gram matrix, dictionary * dictionary'
 
-    cov: array, shape=(n_atoms, n_samples)
+    cov: array, shape=(n_components, n_samples)
         Precomputed covariance, dictionary' * X
 
     algorithm: {'lasso_lars', 'lasso_cd', 'lars', 'omp', 'threshold'}
@@ -225,7 +186,7 @@ def sparse_encode(X, dictionary, gram=None, cov=None, algorithm='lasso_lars',
         the reconstruction error targeted. In this case, it overrides
         `n_nonzero_coefs`.
 
-    init: array of shape (n_samples, n_atoms)
+    init: array of shape (n_samples, n_components)
         Initialization value of the sparse codes. Only used if
         `algorithm='lasso_cd'`.
 
@@ -241,7 +202,7 @@ def sparse_encode(X, dictionary, gram=None, cov=None, algorithm='lasso_lars',
 
     Returns
     -------
-    code: array of shape (n_samples, n_atoms)
+    code: array of shape (n_samples, n_components)
         The sparse codes
 
     See also
@@ -251,61 +212,61 @@ def sparse_encode(X, dictionary, gram=None, cov=None, algorithm='lasso_lars',
     sklearn.linear_model.Lasso
     SparseCoder
     """
-    warnings.warn("Please note: the interface of sparse_encode has changed: "
-                  "It now follows the dictionary learning API and it also "
-                  "handles parallelization. Please read the docstring for "
-                  "more information.", stacklevel=2)
-    if copy_gram is not None:
-        warnings.warn("copy_gram in sparse_encode is deprecated: it"
-            "lead to errors.", DeprecationWarning, stacklevel=2)
-    dictionary = np.asarray(dictionary)
-    X = np.asarray(X)
+    dictionary = array2d(dictionary)
+    X = array2d(X)
     n_samples, n_features = X.shape
-    n_atoms = dictionary.shape[0]
-    if gram is None:
+    n_components = dictionary.shape[0]
+
+    if gram is None and algorithm != 'threshold':
         gram = np.dot(dictionary, dictionary.T)
-    if cov is None and algorithm != 'lasso_cd':
+    if cov is None:
         copy_cov = False
         cov = np.dot(dictionary, X.T)
+
+    if algorithm in ('lars', 'omp'):
+        regularization = n_nonzero_coefs
+        if regularization is None:
+            regularization = max(n_features / 10, 1)
+    else:
+        regularization = alpha
+        if regularization is None:
+            regularization = 1.
+
     if n_jobs == 1 or algorithm == 'threshold':
-        return _sparse_encode(X, dictionary, gram=gram, cov=cov,
-                  algorithm=algorithm, n_nonzero_coefs=n_nonzero_coefs,
-                  alpha=alpha, copy_cov=copy_cov,
-                  init=init, max_iter=max_iter)
-    code = np.empty((n_samples, n_atoms))
+        return _sparse_encode(X, dictionary, gram, cov=cov,
+                              algorithm=algorithm,
+                              regularization=regularization, copy_cov=copy_cov,
+                              init=init, max_iter=max_iter)
+
+    # Enter parallel code block
+    code = np.empty((n_samples, n_components))
     slices = list(gen_even_slices(n_samples, n_jobs))
+
     code_views = Parallel(n_jobs=n_jobs)(
-                delayed(sparse_encode)(X[this_slice], dictionary, gram,
-                                       cov[:, this_slice], algorithm,
-                                       n_nonzero_coefs, alpha,
-                                       copy_cov=copy_cov,
-                                       init=init[this_slice] if init is not
-                                       None else None,
-                                       max_iter=max_iter)
-                for this_slice in slices)
+        delayed(_sparse_encode)(
+            X[this_slice], dictionary, gram, cov[:, this_slice], algorithm,
+            regularization=regularization, copy_cov=copy_cov,
+            init=init[this_slice] if init is not None else None,
+            max_iter=max_iter)
+        for this_slice in slices)
     for this_slice, this_view in zip(slices, code_views):
         code[this_slice] = this_view
     return code
 
 
-@deprecated('Use sparse_encode instead')
-def sparse_encode_parallel():
-    pass
-
-
 def _update_dict(dictionary, Y, code, verbose=False, return_r2=False,
                  random_state=None):
     """Update the dense dictionary factor in place.
 
     Parameters
     ----------
-    dictionary: array of shape (n_features, n_atoms)
+    dictionary: array of shape (n_features, n_components)
         Value of the dictionary at the previous iteration.
 
     Y: array of shape (n_features, n_samples)
         Data matrix.
 
-    code: array of shape (n_atoms, n_samples)
+    code: array of shape (n_components, n_samples)
         Sparse coding of the data against which to optimize the dictionary.
 
     verbose:
@@ -320,11 +281,11 @@ def _update_dict(dictionary, Y, code, verbose=False, return_r2=False,
 
     Returns
     -------
-    dictionary: array of shape (n_features, n_atoms)
+    dictionary: array of shape (n_features, n_components)
         Updated dictionary.
 
     """
-    n_atoms = len(code)
+    n_components = len(code)
     n_samples = Y.shape[0]
     random_state = check_random_state(random_state)
     # Residuals, computed 'in-place' for efficiency
@@ -332,7 +293,7 @@ def _update_dict(dictionary, Y, code, verbose=False, return_r2=False,
     R += Y
     R = np.asfortranarray(R)
     ger, = linalg.get_blas_funcs(('ger',), (dictionary, code))
-    for k in xrange(n_atoms):
+    for k in xrange(n_components):
         # R <- 1.0 * U_k * V_k^T + R
         R = ger(1.0, dictionary[:, k], code[k, :], a=R, overwrite_a=True)
         dictionary[:, k] = np.dot(R, code[k, :].T)
@@ -365,9 +326,10 @@ def _update_dict(dictionary, Y, code, verbose=False, return_r2=False,
     return dictionary
 
 
-def dict_learning(X, n_atoms, alpha, max_iter=100, tol=1e-8,
+def dict_learning(X, n_components, alpha, max_iter=100, tol=1e-8,
                   method='lars', n_jobs=1, dict_init=None, code_init=None,
-                  callback=None, verbose=False, random_state=None):
+                  callback=None, verbose=False, random_state=None,
+                  n_atoms=None):
     """Solves a dictionary learning matrix factorization problem.
 
     Finds the best dictionary and the corresponding sparse code for
@@ -375,7 +337,7 @@ def dict_learning(X, n_atoms, alpha, max_iter=100, tol=1e-8,
 
         (U^*, V^*) = argmin 0.5 || X - U V ||_2^2 + alpha * || U ||_1
                      (U,V)
-                    with || V_k ||_2 = 1 for all  0 <= k < n_atoms
+                    with || V_k ||_2 = 1 for all  0 <= k < n_components
 
     where V is the dictionary and U is the sparse code.
 
@@ -384,7 +346,7 @@ def dict_learning(X, n_atoms, alpha, max_iter=100, tol=1e-8,
     X: array of shape (n_samples, n_features)
         Data matrix.
 
-    n_atoms: int,
+    n_components: int,
         Number of dictionary atoms to extract.
 
     alpha: int,
@@ -406,10 +368,10 @@ def dict_learning(X, n_atoms, alpha, max_iter=100, tol=1e-8,
     n_jobs: int,
         Number of parallel jobs to run, or -1 to autodetect.
 
-    dict_init: array of shape (n_atoms, n_features),
+    dict_init: array of shape (n_components, n_features),
         Initial value for the dictionary for warm restart scenarios.
 
-    code_init: array of shape (n_samples, n_atoms),
+    code_init: array of shape (n_samples, n_components),
         Initial value for the sparse code for warm restart scenarios.
 
     callback:
@@ -423,10 +385,10 @@ def dict_learning(X, n_atoms, alpha, max_iter=100, tol=1e-8,
 
     Returns
     -------
-    code: array of shape (n_samples, n_atoms)
+    code: array of shape (n_samples, n_components)
         The sparse code factor in the matrix factorization.
 
-    dictionary: array of shape (n_atoms, n_features),
+    dictionary: array of shape (n_components, n_features),
         The dictionary factor in the matrix factorization.
 
     errors: array
@@ -440,6 +402,13 @@ def dict_learning(X, n_atoms, alpha, max_iter=100, tol=1e-8,
     SparsePCA
     MiniBatchSparsePCA
     """
+
+    if not n_atoms is None:
+        n_components = n_atoms
+        warnings.warn("Parameter n_atoms has been renamed to"
+                      "'n_components' and will be removed in release 0.14.",
+                      DeprecationWarning, stacklevel=2)
+
     if method not in ('lars', 'cd'):
         raise ValueError('Coding method not supported as a fit algorithm.')
     method = 'lasso_' + method
@@ -452,8 +421,8 @@ def dict_learning(X, n_atoms, alpha, max_iter=100, tol=1e-8,
     if n_jobs == -1:
         n_jobs = cpu_count()
 
-    # Init U and V with SVD of Y
-    if code_init is not None and code_init is not None:
+    # Init the code and the dictionary with SVD of Y
+    if code_init is not None and dict_init is not None:
         code = np.array(code_init, order='F')
         # Don't copy V, it will happen below
         dictionary = dict_init
@@ -461,13 +430,13 @@ def dict_learning(X, n_atoms, alpha, max_iter=100, tol=1e-8,
         code, S, dictionary = linalg.svd(X, full_matrices=False)
         dictionary = S[:, np.newaxis] * dictionary
     r = len(dictionary)
-    if n_atoms <= r:  # True even if n_atoms=None
-        code = code[:, :n_atoms]
-        dictionary = dictionary[:n_atoms, :]
+    if n_components <= r:  # True even if n_components=None
+        code = code[:, :n_components]
+        dictionary = dictionary[:n_components, :]
     else:
-        code = np.c_[code, np.zeros((len(code), n_atoms - r))]
+        code = np.c_[code, np.zeros((len(code), n_components - r))]
         dictionary = np.r_[dictionary,
-                           np.zeros((n_atoms - r, dictionary.shape[1]))]
+                           np.zeros((n_components - r, dictionary.shape[1]))]
 
     # Fortran-order dict, as we are going to access its row vectors
     dictionary = np.array(dictionary, order='F')
@@ -487,8 +456,8 @@ def dict_learning(X, n_atoms, alpha, max_iter=100, tol=1e-8,
             sys.stdout.flush()
         elif verbose:
             print ("Iteration % 3i "
-                "(elapsed time: % 3is, % 4.1fmn, current cost % 7.3f)" %
-                    (ii, dt, dt / 60, current_cost))
+                   "(elapsed time: % 3is, % 4.1fmn, current cost % 7.3f)"
+                   % (ii, dt, dt / 60, current_cost))
 
         # Update code
         code = sparse_encode(X, dictionary, algorithm=method, alpha=alpha,
@@ -519,10 +488,11 @@ def dict_learning(X, n_atoms, alpha, max_iter=100, tol=1e-8,
     return code, dictionary, errors
 
 
-def dict_learning_online(X, n_atoms, alpha, n_iter=100, return_code=True,
-                         dict_init=None, callback=None, chunk_size=3,
-                         verbose=False, shuffle=True, n_jobs=1,
-                         method='lars', iter_offset=0, random_state=None):
+def dict_learning_online(X, n_components=2, alpha=1, n_iter=100,
+                         return_code=True, dict_init=None, callback=None,
+                         batch_size=3, verbose=False, shuffle=True, n_jobs=1,
+                         method='lars', iter_offset=0, random_state=None,
+                         n_atoms=None, chunk_size=None):
     """Solves a dictionary learning matrix factorization problem online.
 
     Finds the best dictionary and the corresponding sparse code for
@@ -530,7 +500,7 @@ def dict_learning_online(X, n_atoms, alpha, n_iter=100, return_code=True,
 
         (U^*, V^*) = argmin 0.5 || X - U V ||_2^2 + alpha * || U ||_1
                      (U,V)
-                     with || V_k ||_2 = 1 for all  0 <= k < n_atoms
+                     with || V_k ||_2 = 1 for all  0 <= k < n_components
 
     where V is the dictionary and U is the sparse code. This is
     accomplished by repeatedly iterating over mini-batches by slicing
@@ -539,58 +509,58 @@ def dict_learning_online(X, n_atoms, alpha, n_iter=100, return_code=True,
     Parameters
     ----------
     X: array of shape (n_samples, n_features)
-        data matrix
+        Data matrix.
 
-    n_atoms: int,
-        number of dictionary atoms to extract
+    n_components : int,
+        Number of dictionary atoms to extract.
 
-    alpha: int,
-        sparsity controlling parameter
+    alpha : int,
+        Sparsity controlling parameter.
 
-    n_iter: int,
-        number of iterations to perform
+    n_iter : int,
+        Number of iterations to perform.
 
-    return_code: boolean,
-        whether to also return the code U or just the dictionary V
+    return_code : boolean,
+        Whether to also return the code U or just the dictionary V.
 
-    dict_init: array of shape (n_atoms, n_features),
-        initial value for the dictionary for warm restart scenarios
+    dict_init : array of shape (n_components, n_features),
+        Initial value for the dictionary for warm restart scenarios.
 
-    callback:
-        callable that gets invoked every five iterations
+    callback :
+        Callable that gets invoked every five iterations.
 
-    chunk_size: int,
-        the number of samples to take in each batch
+    batch_size : int,
+        The number of samples to take in each batch.
 
-    verbose:
-        degree of output the procedure will print
+    verbose :
+        Degree of output the procedure will print.
 
-    shuffle: boolean,
-        whether to shuffle the data before splitting it in batches
+    shuffle : boolean,
+        Whether to shuffle the data before splitting it in batches.
 
-    n_jobs: int,
-        number of parallel jobs to run, or -1 to autodetect.
+    n_jobs : int,
+        Number of parallel jobs to run, or -1 to autodetect.
 
-    method: {'lars', 'cd'}
+    method : {'lars', 'cd'}
         lars: uses the least angle regression method to solve the lasso problem
         (linear_model.lars_path)
         cd: uses the coordinate descent method to compute the
         Lasso solution (linear_model.Lasso). Lars will be faster if
         the estimated components are sparse.
 
-    iter_offset: int, default 0
-        number of previous iterations completed on the dictionary used for
-        initialization
+    iter_offset : int, default 0
+        Number of previous iterations completed on the dictionary used for
+        initialization.
 
-    random_state: int or RandomState
+    random_state : int or RandomState
         Pseudo number generator state used for random sampling.
 
     Returns
     -------
-    code: array of shape (n_samples, n_atoms),
+    code : array of shape (n_samples, n_components),
         the sparse code (only returned if `return_code=True`)
 
-    dictionary: array of shape (n_atoms, n_features),
+    dictionary : array of shape (n_components, n_features),
         the solutions to the dictionary learning problem
 
     See also
@@ -602,6 +572,19 @@ def dict_learning_online(X, n_atoms, alpha, n_iter=100, return_code=True,
     MiniBatchSparsePCA
 
     """
+
+    if n_atoms is not None:
+        n_components = n_atoms
+        warnings.warn("Parameter n_atoms has been renamed to "
+                      "'n_components' and will be removed in release 0.14.",
+                      DeprecationWarning, stacklevel=2)
+
+    if chunk_size is not None:
+        chunk_size = batch_size
+        warnings.warn("Parameter chunk_size has been renamed to "
+                      "'batch_size' and will be removed in release 0.14.",
+                      DeprecationWarning, stacklevel=2)
+
     if method not in ('lars', 'cd'):
         raise ValueError('Coding method not supported as a fit algorithm.')
     method = 'lasso_' + method
@@ -619,20 +602,20 @@ def dict_learning_online(X, n_atoms, alpha, n_iter=100, return_code=True,
     if dict_init is not None:
         dictionary = dict_init
     else:
-        _, S, dictionary = randomized_svd(X, n_atoms)
+        _, S, dictionary = randomized_svd(X, n_components)
         dictionary = S[:, np.newaxis] * dictionary
     r = len(dictionary)
-    if n_atoms <= r:
-        dictionary = dictionary[:n_atoms, :]
+    if n_components <= r:
+        dictionary = dictionary[:n_components, :]
     else:
         dictionary = np.r_[dictionary,
-                           np.zeros((n_atoms - r, dictionary.shape[1]))]
+                           np.zeros((n_components - r, dictionary.shape[1]))]
     dictionary = np.ascontiguousarray(dictionary.T)
 
     if verbose == 1:
         print '[dict_learning]',
 
-    n_batches = floor(float(len(X)) / chunk_size)
+    n_batches = floor(float(len(X)) / batch_size)
     if shuffle:
         X_train = X.copy()
         random_state.shuffle(X_train)
@@ -642,9 +625,9 @@ def dict_learning_online(X, n_atoms, alpha, n_iter=100, return_code=True,
     batches = itertools.cycle(batches)
 
     # The covariance of the dictionary
-    A = np.zeros((n_atoms, n_atoms))
+    A = np.zeros((n_components, n_components))
     # The data approximation
-    B = np.zeros((n_features, n_atoms))
+    B = np.zeros((n_features, n_components))
 
     for ii, this_X in itertools.izip(xrange(iter_offset, iter_offset + n_iter),
                                      batches):
@@ -654,18 +637,18 @@ def dict_learning_online(X, n_atoms, alpha, n_iter=100, return_code=True,
             sys.stdout.flush()
         elif verbose:
             if verbose > 10 or ii % ceil(100. / verbose) == 0:
-                print ("Iteration % 3i (elapsed time: % 3is, % 4.1fmn)" %
-                    (ii, dt, dt / 60))
+                print ("Iteration % 3i (elapsed time: % 3is, % 4.1fmn)"
+                       % (ii, dt, dt / 60))
 
         this_code = sparse_encode(this_X, dictionary.T, algorithm=method,
                                   alpha=alpha).T
 
         # Update the auxiliary variables
-        if ii < chunk_size - 1:
-            theta = float((ii + 1) * chunk_size)
+        if ii < batch_size - 1:
+            theta = float((ii + 1) * batch_size)
         else:
-            theta = float(chunk_size ** 2 + ii + 1 - chunk_size)
-        beta = (theta + 1 - chunk_size) / (theta + 1)
+            theta = float(batch_size ** 2 + ii + 1 - batch_size)
+        beta = (theta + 1 - batch_size) / (theta + 1)
 
         A *= beta
         A += np.dot(this_code, this_code.T)
@@ -700,11 +683,12 @@ def dict_learning_online(X, n_atoms, alpha, n_iter=100, return_code=True,
 class SparseCodingMixin(TransformerMixin):
     """Sparse coding mixin"""
 
-    def _set_sparse_coding_params(self, n_atoms, transform_algorithm='omp',
+    def _set_sparse_coding_params(self, n_components,
+                                  transform_algorithm='omp',
                                   transform_n_nonzero_coefs=None,
                                   transform_alpha=None, split_sign=False,
                                   n_jobs=1):
-        self.n_atoms = n_atoms
+        self.n_components = n_components
         self.transform_algorithm = transform_algorithm
         self.transform_n_nonzero_coefs = transform_n_nonzero_coefs
         self.transform_alpha = transform_alpha
@@ -762,7 +746,7 @@ class SparseCoder(BaseEstimator, SparseCodingMixin):
 
     Parameters
     ----------
-    dictionary : array, [n_atoms, n_features]
+    dictionary : array, [n_components, n_features]
         The dictionary atoms used for sparse coding. Lines are assumed to be
         normalized to unit norm.
 
@@ -802,7 +786,7 @@ class SparseCoder(BaseEstimator, SparseCodingMixin):
 
     Attributes
     ----------
-    `components_` : array, [n_atoms, n_features]
+    `components_` : array, [n_components, n_features]
         The unchanged dictionary atoms
 
     See also
@@ -842,11 +826,11 @@ class DictionaryLearning(BaseEstimator, SparseCodingMixin):
 
         (U^*,V^*) = argmin 0.5 || Y - U V ||_2^2 + alpha * || U ||_1
                     (U,V)
-                    with || V_k ||_2 = 1 for all  0 <= k < n_atoms
+                    with || V_k ||_2 = 1 for all  0 <= k < n_components
 
     Parameters
     ----------
-    n_atoms : int,
+    n_components : int,
         number of dictionary elements to extract
 
     alpha : int,
@@ -899,10 +883,10 @@ class DictionaryLearning(BaseEstimator, SparseCodingMixin):
     n_jobs : int,
         number of parallel jobs to run
 
-    code_init : array of shape (n_samples, n_atoms),
+    code_init : array of shape (n_samples, n_components),
         initial value for the code, for warm restart
 
-    dict_init : array of shape (n_atoms, n_features),
+    dict_init : array of shape (n_components, n_features),
         initial values for the dictionary, for warm restart
 
     verbose :
@@ -913,7 +897,7 @@ class DictionaryLearning(BaseEstimator, SparseCodingMixin):
 
     Attributes
     ----------
-    `components_` : array, [n_atoms, n_features]
+    `components_` : array, [n_components, n_features]
         dictionary atoms extracted from the data
 
     `error_` : array
@@ -933,12 +917,19 @@ class DictionaryLearning(BaseEstimator, SparseCodingMixin):
     SparsePCA
     MiniBatchSparsePCA
     """
-    def __init__(self, n_atoms, alpha=1, max_iter=1000, tol=1e-8,
+    def __init__(self, n_components=None, alpha=1, max_iter=1000, tol=1e-8,
                  fit_algorithm='lars', transform_algorithm='omp',
                  transform_n_nonzero_coefs=None, transform_alpha=None,
                  n_jobs=1, code_init=None, dict_init=None, verbose=False,
-                 split_sign=False, random_state=None):
-        self._set_sparse_coding_params(n_atoms, transform_algorithm,
+                 split_sign=False, random_state=None, n_atoms=None):
+
+        if n_atoms is not None:
+            n_components = n_atoms
+            warnings.warn("Parameter n_atoms has been renamed to "
+                          "'n_components' and will be removed in release"
+                          " 0.14.", DeprecationWarning, stacklevel=2)
+
+        self._set_sparse_coding_params(n_components, transform_algorithm,
                                        transform_n_nonzero_coefs,
                                        transform_alpha, split_sign, n_jobs)
         self.alpha = alpha
@@ -965,8 +956,13 @@ class DictionaryLearning(BaseEstimator, SparseCodingMixin):
             Returns the object itself
         """
         self.random_state = check_random_state(self.random_state)
-        X = np.asarray(X)
-        V, U, E = dict_learning(X, self.n_atoms, self.alpha,
+        X = array2d(X)
+        if self.n_components is None:
+            n_components = X.shape[1]
+        else:
+            n_components = self.n_components
+
+        V, U, E = dict_learning(X, n_components, self.alpha,
                                 tol=self.tol, max_iter=self.max_iter,
                                 method=self.fit_algorithm,
                                 n_jobs=self.n_jobs,
@@ -989,11 +985,11 @@ class MiniBatchDictionaryLearning(BaseEstimator, SparseCodingMixin):
 
        (U^*,V^*) = argmin 0.5 || Y - U V ||_2^2 + alpha * || U ||_1
                     (U,V)
-                    with || V_k ||_2 = 1 for all  0 <= k < n_atoms
+                    with || V_k ||_2 = 1 for all  0 <= k < n_components
 
     Parameters
     ----------
-    n_atoms : int,
+    n_components : int,
         number of dictionary elements to extract
 
     alpha : int,
@@ -1043,13 +1039,13 @@ class MiniBatchDictionaryLearning(BaseEstimator, SparseCodingMixin):
     n_jobs : int,
         number of parallel jobs to run
 
-    dict_init : array of shape (n_atoms, n_features),
+    dict_init : array of shape (n_components, n_features),
         initial value of the dictionary for warm restart scenarios
 
     verbose :
         degree of verbosity of the printed output
 
-    chunk_size : int,
+    batch_size : int,
         number of samples in each mini-batch
 
     shuffle : bool,
@@ -1060,7 +1056,7 @@ class MiniBatchDictionaryLearning(BaseEstimator, SparseCodingMixin):
 
     Attributes
     ----------
-    `components_` : array, [n_atoms, n_features]
+    `components_` : array, [n_components, n_features]
         components extracted from the data
 
     Notes
@@ -1078,13 +1074,26 @@ class MiniBatchDictionaryLearning(BaseEstimator, SparseCodingMixin):
     MiniBatchSparsePCA
 
     """
-    def __init__(self, n_atoms, alpha=1, n_iter=1000,
-                 fit_algorithm='lars', n_jobs=1, chunk_size=3,
+    def __init__(self, n_components=None, alpha=1, n_iter=1000,
+                 fit_algorithm='lars', n_jobs=1, batch_size=3,
                  shuffle=True, dict_init=None, transform_algorithm='omp',
                  transform_n_nonzero_coefs=None, transform_alpha=None,
-                 verbose=False, split_sign=False, random_state=None):
+                 verbose=False, split_sign=False, random_state=None,
+                 n_atoms=None, chunk_size=None):
 
-        self._set_sparse_coding_params(n_atoms, transform_algorithm,
+        if n_atoms is not None:
+            n_components = n_atoms
+            warnings.warn("Parameter n_atoms has been renamed to"
+                          "'n_components' and will be removed in release"
+                          " 0.14.", DeprecationWarning, stacklevel=2)
+
+        if chunk_size is not None:
+            chunk_size = batch_size
+            warnings.warn("Parameter chunk_size has been renamed to"
+                          "'batch_size' and will be removed in release"
+                          " 0.14.", DeprecationWarning, stacklevel=2)
+
+        self._set_sparse_coding_params(n_components, transform_algorithm,
                                        transform_n_nonzero_coefs,
                                        transform_alpha, split_sign, n_jobs)
         self.alpha = alpha
@@ -1093,7 +1102,7 @@ class MiniBatchDictionaryLearning(BaseEstimator, SparseCodingMixin):
         self.dict_init = dict_init
         self.verbose = verbose
         self.shuffle = shuffle
-        self.chunk_size = chunk_size
+        self.batch_size = batch_size
         self.split_sign = split_sign
         self.random_state = random_state
 
@@ -1112,13 +1121,18 @@ class MiniBatchDictionaryLearning(BaseEstimator, SparseCodingMixin):
             Returns the instance itself.
         """
         self.random_state = check_random_state(self.random_state)
-        X = np.asarray(X)
-        U = dict_learning_online(X, self.n_atoms, self.alpha,
+        X = array2d(X)
+        if self.n_components is None:
+            n_components = X.shape[1]
+        else:
+            n_components = self.n_components
+
+        U = dict_learning_online(X, n_components, self.alpha,
                                  n_iter=self.n_iter, return_code=False,
                                  method=self.fit_algorithm,
                                  n_jobs=self.n_jobs,
                                  dict_init=self.dict_init,
-                                 chunk_size=self.chunk_size,
+                                 batch_size=self.batch_size,
                                  shuffle=self.shuffle, verbose=self.verbose,
                                  random_state=self.random_state)
         self.components_ = U
@@ -1144,11 +1158,11 @@ class MiniBatchDictionaryLearning(BaseEstimator, SparseCodingMixin):
             dict_init = self.components_
         else:
             dict_init = self.dict_init
-        U = dict_learning_online(X, self.n_atoms, self.alpha,
+        U = dict_learning_online(X, self.n_components, self.alpha,
                                  n_iter=self.n_iter,
                                  method=self.fit_algorithm,
                                  n_jobs=self.n_jobs, dict_init=dict_init,
-                                 chunk_size=len(X), shuffle=False,
+                                 batch_size=len(X), shuffle=False,
                                  verbose=self.verbose, return_code=False,
                                  iter_offset=iter_offset,
                                  random_state=self.random_state)
