@@ -2,9 +2,11 @@
 # Author: Vlad Niculae, Gael Varoquaux, Alexandre Gramfort
 # License: BSD
 
+import warnings
+
 import numpy as np
 
-from ..utils import check_random_state
+from ..utils import check_random_state, array2d
 from ..linear_model import ridge_regression
 from ..base import BaseEstimator, TransformerMixin
 from .dict_learning import dict_learning, dict_learning_online
@@ -46,10 +48,10 @@ class SparsePCA(BaseEstimator, TransformerMixin):
     n_jobs : int,
         Number of parallel jobs to run.
 
-    U_init : array of shape (n_samples, n_atoms),
+    U_init : array of shape (n_samples, n_components),
         Initial values for the loadings for warm restart scenarios.
 
-    V_init : array of shape (n_atoms, n_features),
+    V_init : array of shape (n_components, n_features),
         Initial values for the components for warm restart scenarios.
 
     verbose :
@@ -72,8 +74,8 @@ class SparsePCA(BaseEstimator, TransformerMixin):
     MiniBatchSparsePCA
     DictionaryLearning
     """
-    def __init__(self, n_components, alpha=1, ridge_alpha=0.01, max_iter=1000,
-                 tol=1e-8, method='lars', n_jobs=1, U_init=None,
+    def __init__(self, n_components=None, alpha=1, ridge_alpha=0.01,
+                 max_iter=1000, tol=1e-8, method='lars', n_jobs=1, U_init=None,
                  V_init=None, verbose=False, random_state=None):
         self.n_components = n_components
         self.alpha = alpha
@@ -102,10 +104,14 @@ class SparsePCA(BaseEstimator, TransformerMixin):
             Returns the instance itself.
         """
         self.random_state = check_random_state(self.random_state)
-        X = np.asarray(X)
+        X = array2d(X)
+        if self.n_components is None:
+            n_components = X.shape[1]
+        else:
+            n_components = self.n_components
         code_init = self.V_init.T if self.V_init is not None else None
         dict_init = self.U_init.T if self.U_init is not None else None
-        Vt, _, E = dict_learning(X.T, self.n_components, self.alpha,
+        Vt, _, E = dict_learning(X.T, n_components, self.alpha,
                                  tol=self.tol, max_iter=self.max_iter,
                                  method=self.method, n_jobs=self.n_jobs,
                                  verbose=self.verbose,
@@ -176,7 +182,7 @@ class MiniBatchSparsePCA(SparsePCA):
     callback : callable,
         callable that gets invoked every five iterations
 
-    chunk_size : int,
+    batch_size : int,
         the number of features to take in each mini batch
 
     verbose :
@@ -212,15 +218,22 @@ class MiniBatchSparsePCA(SparsePCA):
     SparsePCA
     DictionaryLearning
     """
-    def __init__(self, n_components, alpha=1, ridge_alpha=0.01, n_iter=100,
-                 callback=None, chunk_size=3, verbose=False, shuffle=True,
-                 n_jobs=1, method='lars', random_state=None):
+    def __init__(self, n_components=None, alpha=1, ridge_alpha=0.01,
+                 n_iter=100, callback=None, batch_size=3, verbose=False,
+                 shuffle=True, n_jobs=1, method='lars', random_state=None,
+                 chunk_size=None):
+
+        if chunk_size is not None:
+            chunk_size = batch_size
+            warnings.warn("Parameter chunk_size has been renamed to "
+                          "'batch_size' and will be removed in release 0.14.",
+                          DeprecationWarning, stacklevel=2)
         self.n_components = n_components
         self.alpha = alpha
         self.ridge_alpha = ridge_alpha
         self.n_iter = n_iter
         self.callback = callback
-        self.chunk_size = chunk_size
+        self.batch_size = batch_size
         self.verbose = verbose
         self.shuffle = shuffle
         self.n_jobs = n_jobs
@@ -242,12 +255,16 @@ class MiniBatchSparsePCA(SparsePCA):
             Returns the instance itself.
         """
         self.random_state = check_random_state(self.random_state)
-        X = np.asarray(X)
-        Vt, _ = dict_learning_online(X.T, self.n_components, alpha=self.alpha,
+        X = array2d(X)
+        if self.n_components is None:
+            n_components = X.shape[1]
+        else:
+            n_components = self.n_components
+        Vt, _ = dict_learning_online(X.T, n_components, alpha=self.alpha,
                                      n_iter=self.n_iter, return_code=True,
                                      dict_init=None, verbose=self.verbose,
                                      callback=self.callback,
-                                     chunk_size=self.chunk_size,
+                                     batch_size=self.batch_size,
                                      shuffle=self.shuffle,
                                      n_jobs=self.n_jobs, method=self.method,
                                      random_state=self.random_state)
