@@ -1,10 +1,13 @@
 import numpy as np
-from .. import nmf
-from nose.tools import assert_true, assert_false, raises
-from numpy.testing import assert_array_almost_equal
+from sklearn.decomposition import nmf
 
+from sklearn.utils.testing import assert_true
+from sklearn.utils.testing import assert_false
+from sklearn.utils.testing import raises
+from sklearn.utils.testing import assert_array_almost_equal
 from sklearn.utils.testing import assert_greater
 
+
 random_state = np.random.mtrand.RandomState(0)
 
 
@@ -72,7 +75,7 @@ def test_projgrad_nmf_fit_nn_output():
 def test_projgrad_nmf_fit_close():
     """Test that the fit is not too far away"""
     assert_true(nmf.ProjectedGradientNMF(5, init='nndsvda').fit(np.abs(
-      random_state.randn(6, 5))).reconstruction_err_ < 0.05)
+        random_state.randn(6, 5))).reconstruction_err_ < 0.05)
 
 
 @raises(ValueError)
@@ -108,6 +111,12 @@ def test_projgrad_nmf_transform():
     assert_true(np.allclose(transf, m.transform(A), atol=1e-2, rtol=0))
 
 
+def test_n_components_greater_n_features():
+    """Smoke test for the case of more components than features."""
+    A = np.abs(random_state.randn(30, 10))
+    nmf.ProjectedGradientNMF(n_components=15, sparseness='data').fit(A)
+
+
 def test_projgrad_nmf_sparseness():
     """Test sparseness
 
@@ -117,10 +126,10 @@ def test_projgrad_nmf_sparseness():
 
     A = np.abs(random_state.randn(10, 10))
     m = nmf.ProjectedGradientNMF(n_components=5).fit(A)
-    data_sp = nmf.ProjectedGradientNMF(n_components=5,
-                  sparseness='data').fit(A).data_sparseness_
-    comp_sp = nmf.ProjectedGradientNMF(n_components=5,
-                  sparseness='components').fit(A).comp_sparseness_
+    data_sp = nmf.ProjectedGradientNMF(
+        n_components=5, sparseness='data').fit(A).data_sparseness_
+    comp_sp = nmf.ProjectedGradientNMF(
+        n_components=5, sparseness='components').fit(A).comp_sparseness_
     assert_greater(data_sp, m.data_sparseness_)
     assert_greater(comp_sp, m.comp_sparseness_)
 
@@ -131,12 +140,23 @@ def test_sparse_input():
 
     A = np.abs(random_state.randn(10, 10))
     A[:, 2 * np.arange(5)] = 0
-    T1 = nmf.ProjectedGradientNMF(n_components=5, init=999).fit_transform(A)
+    T1 = nmf.ProjectedGradientNMF(n_components=5, init='random',
+                                  random_state=999).fit_transform(A)
 
-    A = csr_matrix(A)
-    T2 = nmf.ProjectedGradientNMF(n_components=5, init=999).fit_transform(A)
+    A_sparse = csr_matrix(A)
+    T2 = nmf.ProjectedGradientNMF(n_components=5, init='random',
+                                  random_state=999).fit_transform(A_sparse)
     assert_array_almost_equal(T1, T2)
 
+    # same with sparseness
+
+    T2 = nmf.ProjectedGradientNMF(
+        n_components=5, init='random', sparseness='data',
+        random_state=999).fit_transform(A_sparse)
+    T1 = nmf.ProjectedGradientNMF(
+        n_components=5, init='random', sparseness='data',
+        random_state=999).fit_transform(A)
+
 
 if __name__ == '__main__':
     import nose
