@@ -3,26 +3,33 @@ from sklearn.feature_extraction.text import strip_tags
 from sklearn.feature_extraction.text import strip_accents_unicode
 from sklearn.feature_extraction.text import strip_accents_ascii
 
+from sklearn.feature_extraction.text import HashingVectorizer
 from sklearn.feature_extraction.text import CountVectorizer
 from sklearn.feature_extraction.text import TfidfTransformer
 from sklearn.feature_extraction.text import TfidfVectorizer
 
-from sklearn.utils.testing import assert_less, assert_greater
-
 from sklearn.grid_search import GridSearchCV
 from sklearn.pipeline import Pipeline
 from sklearn.svm import LinearSVC
 
 import numpy as np
-from nose.tools import assert_equal, assert_equals, \
-            assert_false, assert_not_equal, assert_true
+from nose import SkipTest
+from nose.tools import assert_equal
+from nose.tools import assert_false
+from nose.tools import assert_not_equal
+from nose.tools import assert_true
+from nose.tools import assert_almost_equal
 from numpy.testing import assert_array_almost_equal
 from numpy.testing import assert_array_equal
 from numpy.testing import assert_raises
+from sklearn.utils.testing import assert_in, assert_less, assert_greater
 
+from collections import defaultdict, Mapping
+from functools import partial
 import pickle
 from StringIO import StringIO
 
+
 JUNK_FOOD_DOCS = (
     "the pizza pizza beer copyright",
     "the pizza burger beer copyright",
@@ -102,46 +109,50 @@ def test_to_ascii():
 
 
 def test_word_analyzer_unigrams():
-    wa = CountVectorizer(strip_accents='ascii').build_analyzer()
-    text = u"J'ai mang\xe9 du kangourou  ce midi, c'\xe9tait pas tr\xeas bon."
-    expected = [u'ai', u'mange', u'du', u'kangourou', u'ce', u'midi',
-                u'etait', u'pas', u'tres', u'bon']
-    assert_equal(wa(text), expected)
+    for Vectorizer in (CountVectorizer, HashingVectorizer):
+        wa = Vectorizer(strip_accents='ascii').build_analyzer()
+        text = (u"J'ai mang\xe9 du kangourou  ce midi, "
+                u"c'\xe9tait pas tr\xeas bon.")
+        expected = [u'ai', u'mange', u'du', u'kangourou', u'ce', u'midi',
+                    u'etait', u'pas', u'tres', u'bon']
+        assert_equal(wa(text), expected)
 
-    text = "This is a test, really.\n\n I met Harry yesterday."
-    expected = [u'this', u'is', u'test', u'really', u'met', u'harry',
-                u'yesterday']
-    assert_equal(wa(text), expected)
+        text = "This is a test, really.\n\n I met Harry yesterday."
+        expected = [u'this', u'is', u'test', u'really', u'met', u'harry',
+                    u'yesterday']
+        assert_equal(wa(text), expected)
 
-    wa = CountVectorizer(input='file').build_analyzer()
-    text = StringIO("This is a test with a file-like object!")
-    expected = [u'this', u'is', u'test', u'with', u'file', u'like',
-                u'object']
-    assert_equal(wa(text), expected)
+        wa = Vectorizer(input='file').build_analyzer()
+        text = StringIO("This is a test with a file-like object!")
+        expected = [u'this', u'is', u'test', u'with', u'file', u'like',
+                    u'object']
+        assert_equal(wa(text), expected)
 
-    # with custom preprocessor
-    wa = CountVectorizer(preprocessor=uppercase).build_analyzer()
-    text = u"J'ai mang\xe9 du kangourou  ce midi, c'\xe9tait pas tr\xeas bon."
-    expected = [u'AI', u'MANGE', u'DU', u'KANGOUROU', u'CE', u'MIDI',
-                u'ETAIT', u'PAS', u'TRES', u'BON']
-    assert_equal(wa(text), expected)
+        # with custom preprocessor
+        wa = Vectorizer(preprocessor=uppercase).build_analyzer()
+        text = (u"J'ai mang\xe9 du kangourou  ce midi, "
+                u" c'\xe9tait pas tr\xeas bon.")
+        expected = [u'AI', u'MANGE', u'DU', u'KANGOUROU', u'CE', u'MIDI',
+                    u'ETAIT', u'PAS', u'TRES', u'BON']
+        assert_equal(wa(text), expected)
 
-    # with custom tokenizer
-    wa = CountVectorizer(tokenizer=split_tokenize,
-                         strip_accents='ascii').build_analyzer()
-    text = u"J'ai mang\xe9 du kangourou  ce midi, c'\xe9tait pas tr\xeas bon."
-    expected = [u"j'ai", u'mange', u'du', u'kangourou', u'ce', u'midi,',
-                u"c'etait", u'pas', u'tres', u'bon.']
-    assert_equal(wa(text), expected)
+        # with custom tokenizer
+        wa = Vectorizer(tokenizer=split_tokenize,
+                        strip_accents='ascii').build_analyzer()
+        text = (u"J'ai mang\xe9 du kangourou  ce midi, "
+                u"c'\xe9tait pas tr\xeas bon.")
+        expected = [u"j'ai", u'mange', u'du', u'kangourou', u'ce', u'midi,',
+                    u"c'etait", u'pas', u'tres', u'bon.']
+        assert_equal(wa(text), expected)
 
 
 def test_word_analyzer_unigrams_and_bigrams():
     wa = CountVectorizer(analyzer="word", strip_accents='unicode',
-                         min_n=1, max_n=2).build_analyzer()
+                         ngram_range=(1, 2)).build_analyzer()
 
     text = u"J'ai mang\xe9 du kangourou  ce midi, c'\xe9tait pas tr\xeas bon."
-    expected = [u'ai', u'mange', u'du', u'kangourou', u'ce', u'midi', u'etait',
-                u'pas', u'tres', u'bon', u'ai mange', u'mange du',
+    expected = [u'ai', u'mange', u'du', u'kangourou', u'ce', u'midi',
+                u'etait', u'pas', u'tres', u'bon', u'ai mange', u'mange du',
                 u'du kangourou', u'kangourou ce', u'ce midi', u'midi etait',
                 u'etait pas', u'pas tres', u'tres bon']
     assert_equal(wa(text), expected)
@@ -155,17 +166,17 @@ def test_unicode_decode_error():
 
     # Then let the Analyzer try to decode it as ascii. It should fail,
     # because we have given it an incorrect charset.
-    wa = CountVectorizer(min_n=1, max_n=2, charset='ascii').build_analyzer()
+    wa = CountVectorizer(ngram_range=(1, 2), charset='ascii').build_analyzer()
     assert_raises(UnicodeDecodeError, wa, text_bytes)
 
-    ca = CountVectorizer(analyzer='char', min_n=3, max_n=6,
+    ca = CountVectorizer(analyzer='char', ngram_range=(3, 6),
                          charset='ascii').build_analyzer()
     assert_raises(UnicodeDecodeError, ca, text_bytes)
 
 
 def test_char_ngram_analyzer():
     cnga = CountVectorizer(analyzer='char', strip_accents='unicode',
-                           min_n=3, max_n=6).build_analyzer()
+                           ngram_range=(3, 6)).build_analyzer()
 
     text = u"J'ai mang\xe9 du kangourou  ce midi, c'\xe9tait pas tr\xeas bon"
     expected = [u"j'a", u"'ai", u'ai ', u'i m', u' ma']
@@ -181,27 +192,45 @@ def test_char_ngram_analyzer():
     assert_equal(cnga(text)[-5:], expected)
 
     cnga = CountVectorizer(input='file', analyzer='char',
-                           min_n=3, max_n=6).build_analyzer()
+                           ngram_range=(3, 6)).build_analyzer()
     text = StringIO("This is a test with a file-like object!")
     expected = [u'thi', u'his', u'is ', u's i', u' is']
     assert_equal(cnga(text)[:5], expected)
 
 
-def test_countvectorizer_custom_vocabulary():
-    what_we_like = ["pizza", "beer"]
-    vect = CountVectorizer(vocabulary=what_we_like)
-    vect.fit(JUNK_FOOD_DOCS)
-    assert_equal(set(vect.vocabulary_), set(what_we_like))
-    X = vect.transform(JUNK_FOOD_DOCS)
-    assert_equal(X.shape[1], len(what_we_like))
+def test_char_wb_ngram_analyzer():
+    cnga = CountVectorizer(analyzer='char_wb', strip_accents='unicode',
+                           ngram_range=(3, 6)).build_analyzer()
 
-    # try again with a dict vocabulary
+    text = "This \n\tis a test, really.\n\n I met Harry yesterday"
+    expected = [u' th', u'thi', u'his', u'is ', u' thi']
+    assert_equal(cnga(text)[:5], expected)
+
+    expected = [u'yester', u'esterd', u'sterda', u'terday', u'erday ']
+    assert_equal(cnga(text)[-5:], expected)
+
+    cnga = CountVectorizer(input='file', analyzer='char_wb',
+                           ngram_range=(3, 6)).build_analyzer()
+    text = StringIO("A test with a file-like object!")
+    expected = [u' a ', u' te', u'tes', u'est', u'st ', u' tes']
+    assert_equal(cnga(text)[:6], expected)
+
+
+def test_countvectorizer_custom_vocabulary():
     vocab = {"pizza": 0, "beer": 1}
-    vect = CountVectorizer(vocabulary=vocab)
-    vect.fit(JUNK_FOOD_DOCS)
-    assert_equal(vect.vocabulary_, vocab)
-    X = vect.transform(JUNK_FOOD_DOCS)
-    assert_equal(X.shape[1], len(what_we_like))
+    terms = set(vocab.keys())
+
+    # Try a few of the supported types.
+    for typ in [dict, list, iter, partial(defaultdict, int)]:
+        v = typ(vocab)
+        vect = CountVectorizer(vocabulary=v)
+        vect.fit(JUNK_FOOD_DOCS)
+        if isinstance(v, Mapping):
+            assert_equal(vect.vocabulary_, vocab)
+        else:
+            assert_equal(set(vect.vocabulary_), terms)
+        X = vect.transform(JUNK_FOOD_DOCS)
+        assert_equal(X.shape[1], len(terms))
 
 
 def test_countvectorizer_custom_vocabulary_pipeline():
@@ -215,6 +244,22 @@ def test_countvectorizer_custom_vocabulary_pipeline():
     assert_equal(X.shape[1], len(what_we_like))
 
 
+def test_countvectorizer_empty_vocabulary():
+    try:
+        CountVectorizer(vocabulary=[])
+        assert False, "we shouldn't get here"
+    except ValueError as e:
+        assert_in("empty vocabulary", str(e))
+
+    try:
+        v = CountVectorizer(min_df=1, max_df=1.0, stop_words="english")
+        # fit on stopwords only
+        v.fit(["to be or not to be", "and me too", "and so do you"])
+        assert False, "we shouldn't get here"
+    except ValueError as e:
+        assert_in("empty vocabulary", str(e))
+
+
 def test_fit_countvectorizer_twice():
     cv = CountVectorizer()
     X1 = cv.fit_transform(ALL_FOOD_DOCS[:5])
@@ -260,10 +305,21 @@ def test_tfidf_no_smoothing():
          [1, 0, 0]]
     tr = TfidfTransformer(smooth_idf=False, norm='l2')
 
+    # First we need to verify that numpy here provides div 0 warnings
+    with warnings.catch_warnings(record=True) as w:
+        1. / np.array([0.])
+        numpy_provides_div0_warning = len(w) == 1
+
     with warnings.catch_warnings(record=True) as w:
         tfidf = tr.fit_transform(X).toarray()
+        if not numpy_provides_div0_warning:
+            raise SkipTest("Numpy does not provide div 0 warnings.")
         assert_equal(len(w), 1)
-        assert_true("divide by zero encountered in divide" in w[0].message)
+        # For Python 3 compatibility
+        if hasattr(w[0].message, 'args'):
+            assert_true("divide by zero" in w[0].message.args[0])
+        else:
+            assert_true("divide by zero" in w[0].message)
 
 
 def test_sublinear_tf():
@@ -284,7 +340,7 @@ def test_vectorizer():
     n_train = len(ALL_FOOD_DOCS) - 1
 
     # test without vocabulary
-    v1 = CountVectorizer(max_df=0.5)
+    v1 = CountVectorizer(max_df=0.5, min_df=1)
     counts_train = v1.fit_transform(train_data)
     if hasattr(counts_train, 'tocsr'):
         counts_train = counts_train.tocsr()
@@ -340,7 +396,7 @@ def test_vectorizer():
     # test the direct tfidf vectorizer
     # (equivalent to term count vectorizer + tfidf transformer)
     train_data = iter(ALL_FOOD_DOCS[:-1])
-    tv = TfidfVectorizer(norm='l1')
+    tv = TfidfVectorizer(norm='l1', min_df=1)
     assert_false(tv.fixed_vocabulary)
 
     tv.max_df = v1.max_df
@@ -356,8 +412,46 @@ def test_vectorizer():
     assert_raises(ValueError, v3.transform, train_data)
 
 
+def test_hashing_vectorizer():
+    v = HashingVectorizer()
+    X = v.transform(ALL_FOOD_DOCS)
+    token_nnz = X.nnz
+    assert_equal(X.shape, (len(ALL_FOOD_DOCS), v.n_features))
+    assert_equal(X.dtype, v.dtype)
+
+    # By default the hashed values receive a random sign and l2 normalization
+    # makes the feature values bounded
+    assert_true(np.min(X.data) > -1)
+    assert_true(np.min(X.data) < 0)
+    assert_true(np.max(X.data) > 0)
+    assert_true(np.max(X.data) < 1)
+
+    # Check that the rows are normalized
+    for i in range(X.shape[0]):
+        assert_almost_equal(np.linalg.norm(X[0].data, 2), 1.0)
+
+    # Check vectorization with some non-default parameters
+    v = HashingVectorizer(ngram_range=(1, 2), non_negative=True, norm='l1')
+    X = v.transform(ALL_FOOD_DOCS)
+    assert_equal(X.shape, (len(ALL_FOOD_DOCS), v.n_features))
+    assert_equal(X.dtype, v.dtype)
+
+    # ngrams generate more non zeros
+    ngrams_nnz = X.nnz
+    assert_true(ngrams_nnz > token_nnz)
+    assert_true(ngrams_nnz < 2 * token_nnz)
+
+    # makes the feature values bounded
+    assert_true(np.min(X.data) > 0)
+    assert_true(np.max(X.data) < 1)
+
+    # Check that the rows are normalized
+    for i in range(X.shape[0]):
+        assert_almost_equal(np.linalg.norm(X[0].data, 1), 1.0)
+
+
 def test_feature_names():
-    cv = CountVectorizer(max_df=0.5)
+    cv = CountVectorizer(max_df=0.5, min_df=1)
     X = cv.fit_transform(ALL_FOOD_DOCS)
 
     n_samples, n_features = X.shape
@@ -385,26 +479,50 @@ def test_vectorizer_max_features():
         # test bounded number of extracted features
         vectorizer = vec_factory(max_df=0.6, max_features=4)
         vectorizer.fit(ALL_FOOD_DOCS)
-        assert_equals(set(vectorizer.vocabulary_), expected_vocabulary)
+        assert_equal(set(vectorizer.vocabulary_), expected_vocabulary)
 
 
 def test_vectorizer_max_df():
     test_data = [u'abc', u'dea']  # the letter a occurs in both strings
-    vect = CountVectorizer(analyzer='char', min_n=1, max_n=1, max_df=1.0)
+    vect = CountVectorizer(analyzer='char', max_df=1.0, min_df=1)
     vect.fit(test_data)
     assert_true(u'a' in vect.vocabulary_.keys())
-    assert_equals(len(vect.vocabulary_.keys()), 5)
+    assert_equal(len(vect.vocabulary_.keys()), 5)
 
     vect.max_df = 0.5
     vect.fit(test_data)
     assert_true(u'a' not in vect.vocabulary_.keys())  # 'a' is ignored
-    assert_equals(len(vect.vocabulary_.keys()), 4)  # the others remain
+    assert_equal(len(vect.vocabulary_.keys()), 4)  # the others remain
+
+    # absolute count: if in more than one
+    vect.max_df = 1
+    vect.fit(test_data)
+    assert_true(u'a' not in vect.vocabulary_.keys())  # 'a' is ignored
+    assert_equal(len(vect.vocabulary_.keys()), 4)  # the others remain
 
 
-def test_binary_occurrences():
+def test_vectorizer_min_df():
+    test_data = [u'abc', u'dea', u'eat']  # the letter a occurs in both strings
+    vect = CountVectorizer(analyzer='char', max_df=1.0, min_df=1)
+    vect.fit(test_data)
+    assert_true(u'a' in vect.vocabulary_.keys())
+    assert_equal(len(vect.vocabulary_.keys()), 6)
+
+    vect.min_df = 2
+    vect.fit(test_data)
+    assert_true(u'c' not in vect.vocabulary_.keys())  # 'c' is ignored
+    assert_equal(len(vect.vocabulary_.keys()), 2)  # only e, a remain
+
+    vect.min_df = .5
+    vect.fit(test_data)
+    assert_true(u'c' not in vect.vocabulary_.keys())  # 'c' is ignored
+    assert_equal(len(vect.vocabulary_.keys()), 2)  # only e, a remain
+
+
+def test_count_binary_occurrences():
     # by default multiple occurrences are counted as longs
     test_data = [u'aaabc', u'abbde']
-    vect = CountVectorizer(analyzer='char', min_n=1, max_n=1, max_df=1.0)
+    vect = CountVectorizer(analyzer='char', max_df=1.0, min_df=1)
     X = vect.fit_transform(test_data).toarray()
     assert_array_equal(['a', 'b', 'c', 'd', 'e'], vect.get_feature_names())
     assert_array_equal([[3, 1, 1, 0, 0],
@@ -412,23 +530,48 @@ def test_binary_occurrences():
 
     # using boolean features, we can fetch the binary occurrence info
     # instead.
-    vect = CountVectorizer(analyzer='char', min_n=1, max_n=1, max_df=1.0,
-                           binary=True)
+    vect = CountVectorizer(analyzer='char', max_df=1.0,
+                           binary=True, min_df=1)
     X = vect.fit_transform(test_data).toarray()
     assert_array_equal([[1, 1, 1, 0, 0],
                         [1, 1, 0, 1, 1]], X)
 
     # check the ability to change the dtype
-    vect = CountVectorizer(analyzer='char', min_n=1, max_n=1, max_df=1.0,
+    vect = CountVectorizer(analyzer='char', max_df=1.0,
                            binary=True, dtype=np.float32)
     X_sparse = vect.fit_transform(test_data)
     assert_equal(X_sparse.dtype, np.float32)
 
 
+def test_hashed_binary_occurrences():
+    # by default multiple occurrences are counted as longs
+    test_data = [u'aaabc', u'abbde']
+    vect = HashingVectorizer(analyzer='char', non_negative=True,
+                             norm=None)
+    X = vect.transform(test_data)
+    assert_equal(np.max(X[0:1].data), 3)
+    assert_equal(np.max(X[1:2].data), 2)
+    assert_equal(X.dtype, np.float64)
+
+    # using boolean features, we can fetch the binary occurrence info
+    # instead.
+    vect = HashingVectorizer(analyzer='char', non_negative=True, binary=True,
+                             norm=None)
+    X = vect.transform(test_data)
+    assert_equal(np.max(X.data), 1)
+    assert_equal(X.dtype, np.float64)
+
+    # check the ability to change the dtype
+    vect = HashingVectorizer(analyzer='char', non_negative=True, binary=True,
+                             norm=None, dtype=np.float64)
+    X = vect.transform(test_data)
+    assert_equal(X.dtype, np.float64)
+
+
 def test_vectorizer_inverse_transform():
     # raw documents
     data = ALL_FOOD_DOCS
-    for vectorizer in (TfidfVectorizer(), CountVectorizer()):
+    for vectorizer in (TfidfVectorizer(min_df=1), CountVectorizer(min_df=1)):
         transformed_data = vectorizer.fit_transform(data)
         inversed_data = vectorizer.inverse_transform(transformed_data)
         analyze = vectorizer.build_analyzer()
@@ -457,11 +600,11 @@ def test_count_vectorizer_pipeline_grid_selection():
     y_train = y[1:-1]
     y_test = np.array([y[0], y[-1]])
 
-    pipeline = Pipeline([('vect', CountVectorizer()),
+    pipeline = Pipeline([('vect', CountVectorizer(min_df=1)),
                          ('svc', LinearSVC())])
 
     parameters = {
-        'vect__max_n': (1, 2),
+        'vect__ngram_range': [(1, 1), (1, 2)],
         'svc__loss': ('l1', 'l2')
     }
 
@@ -479,7 +622,7 @@ def test_count_vectorizer_pipeline_grid_selection():
     # to 100% accuracy models
     assert_equal(grid_search.best_score_, 1.0)
     best_vectorizer = grid_search.best_estimator_.named_steps['vect']
-    assert_equal(best_vectorizer.max_n, 1)
+    assert_equal(best_vectorizer.ngram_range, (1, 1))
 
 
 def test_vectorizer_pipeline_grid_selection():
@@ -495,11 +638,11 @@ def test_vectorizer_pipeline_grid_selection():
     y_train = y[1:-1]
     y_test = np.array([y[0], y[-1]])
 
-    pipeline = Pipeline([('vect', TfidfVectorizer()),
+    pipeline = Pipeline([('vect', TfidfVectorizer(min_df=1)),
                          ('svc', LinearSVC())])
 
     parameters = {
-        'vect__max_n': (1, 2),
+        'vect__ngram_range': [(1, 1), (1, 2)],
         'vect__norm': ('l1', 'l2'),
         'svc__loss': ('l1', 'l2'),
     }
@@ -518,11 +661,46 @@ def test_vectorizer_pipeline_grid_selection():
     # to 100% accuracy models
     assert_equal(grid_search.best_score_, 1.0)
     best_vectorizer = grid_search.best_estimator_.named_steps['vect']
-    assert_equal(best_vectorizer.max_n, 1)
+    assert_equal(best_vectorizer.ngram_range, (1, 1))
     assert_equal(best_vectorizer.norm, 'l2')
     assert_false(best_vectorizer.fixed_vocabulary)
 
 
+def test_vectorizer_unicode():
+    # tests that the count vectorizer works with cyrillic.
+    document = (
+        u"\xd0\x9c\xd0\xb0\xd1\x88\xd0\xb8\xd0\xbd\xd0\xbd\xd0\xbe\xd0"
+        u"\xb5 \xd0\xbe\xd0\xb1\xd1\x83\xd1\x87\xd0\xb5\xd0\xbd\xd0\xb8\xd0"
+        u"\xb5 \xe2\x80\x94 \xd0\xbe\xd0\xb1\xd1\x88\xd0\xb8\xd1\x80\xd0\xbd"
+        u"\xd1\x8b\xd0\xb9 \xd0\xbf\xd0\xbe\xd0\xb4\xd1\x80\xd0\xb0\xd0\xb7"
+        u"\xd0\xb4\xd0\xb5\xd0\xbb \xd0\xb8\xd1\x81\xd0\xba\xd1\x83\xd1\x81"
+        u"\xd1\x81\xd1\x82\xd0\xb2\xd0\xb5\xd0\xbd\xd0\xbd\xd0\xbe\xd0\xb3"
+        u"\xd0\xbe \xd0\xb8\xd0\xbd\xd1\x82\xd0\xb5\xd0\xbb\xd0\xbb\xd0"
+        u"\xb5\xd0\xba\xd1\x82\xd0\xb0, \xd0\xb8\xd0\xb7\xd1\x83\xd1\x87"
+        u"\xd0\xb0\xd1\x8e\xd1\x89\xd0\xb8\xd0\xb9 \xd0\xbc\xd0\xb5\xd1\x82"
+        u"\xd0\xbe\xd0\xb4\xd1\x8b \xd0\xbf\xd0\xbe\xd1\x81\xd1\x82\xd1\x80"
+        u"\xd0\xbe\xd0\xb5\xd0\xbd\xd0\xb8\xd1\x8f \xd0\xb0\xd0\xbb\xd0\xb3"
+        u"\xd0\xbe\xd1\x80\xd0\xb8\xd1\x82\xd0\xbc\xd0\xbe\xd0\xb2, \xd1\x81"
+        u"\xd0\xbf\xd0\xbe\xd1\x81\xd0\xbe\xd0\xb1\xd0\xbd\xd1\x8b\xd1\x85 "
+        u"\xd0\xbe\xd0\xb1\xd1\x83\xd1\x87\xd0\xb0\xd1\x82\xd1\x8c\xd1\x81\xd1"
+        u"\x8f.")
+
+    vect = CountVectorizer(min_df=1)
+    X_counted = vect.fit_transform([document])
+    assert_equal(X_counted.shape, (1, 15))
+
+    vect = HashingVectorizer(norm=None, non_negative=True)
+    X_hashed = vect.transform([document])
+    assert_equal(X_hashed.shape, (1, 2 ** 20))
+
+    # No collisions on such a small dataset
+    assert_equal(X_counted.nnz, X_hashed.nnz)
+
+    # When norm is None and non_negative, the tokens are counted up to
+    # collisions
+    assert_array_equal(np.sort(X_counted.data), np.sort(X_hashed.data))
+
+
 def test_tfidf_vectorizer_with_fixed_vocabulary():
     # non regression smoke test for inheritance issues
     vocabulary = ['pizza', 'celeri']
@@ -536,6 +714,10 @@ def test_tfidf_vectorizer_with_fixed_vocabulary():
 
 def test_pickling_vectorizer():
     instances = [
+        HashingVectorizer(),
+        HashingVectorizer(norm='l1'),
+        HashingVectorizer(binary=True),
+        HashingVectorizer(ngram_range=(1, 2)),
         CountVectorizer(),
         CountVectorizer(preprocessor=strip_tags),
         CountVectorizer(analyzer=lazy_analyze),
@@ -550,6 +732,7 @@ def test_pickling_vectorizer():
         s = pickle.dumps(orig)
         copy = pickle.loads(s)
         assert_equal(type(copy), orig.__class__)
+        assert_equal(copy.get_params(), orig.get_params())
         assert_array_equal(
             copy.fit_transform(JUNK_FOOD_DOCS).toarray(),
             orig.fit_transform(JUNK_FOOD_DOCS).toarray())
