@@ -11,13 +11,15 @@ Generalized Linear Model for a complete discussion.
 
 from math import log
 import sys
+import warnings
 
 import numpy as np
 from scipy import linalg, interpolate
 from scipy.linalg.lapack import get_lapack_funcs
 
 from .base import LinearModel
-from ..utils import array2d, arrayfuncs, deprecated
+from ..base import RegressorMixin
+from ..utils import array2d, arrayfuncs, as_float_array
 from ..cross_validation import check_cv
 from ..externals.joblib import Parallel, delayed
 
@@ -25,7 +27,7 @@ from ..externals.joblib import Parallel, delayed
 def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
               alpha_min=0, method='lar', copy_X=True,
               eps=np.finfo(np.float).eps,
-              copy_Gram=True, verbose=False):
+              copy_Gram=True, verbose=0, return_path=True):
     """Compute Least Angle Regression and Lasso path
 
     The optimization objective for Lasso is::
@@ -34,39 +36,42 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
 
     Parameters
     -----------
-    X: array, shape: (n_samples, n_features)
-        Input data
+    X : array, shape: (n_samples, n_features)
+        Input data.
 
-    y: array, shape: (n_samples)
-        Input targets
+    y : array, shape: (n_samples)
+        Input targets.
 
-    max_iter: integer, optional
+    max_iter : integer, optional
         Maximum number of iterations to perform, set to infinity for no limit.
 
-    Gram: None, 'auto', array, shape: (n_features, n_features), optional
+    Gram : None, 'auto', array, shape: (n_features, n_features), optional
         Precomputed Gram matrix (X' * X), if 'auto', the Gram
         matrix is precomputed from the given X, if there are more samples
-        than features
+        than features.
 
-    alpha_min: float, optional
+    alpha_min : float, optional
         Minimum correlation along the path. It corresponds to the
         regularization parameter alpha parameter in the Lasso.
 
-    method: {'lar', 'lasso'}
+    method : {'lar', 'lasso'}
         Specifies the returned model. Select 'lar' for Least Angle
         Regression, 'lasso' for the Lasso.
 
-    eps: float, optional
+    eps : float, optional
         The machine-precision regularization in the computation of the
         Cholesky diagonal factors. Increase this for very ill-conditioned
         systems.
 
-    copy_X: bool
+    copy_X : bool
         If False, X is overwritten.
 
-    copy_Gram: bool
+    copy_Gram : bool
         If False, Gram is overwritten.
 
+    verbose : int (default=0)
+        Controls output verbosity.
+
     Returns
     --------
     alphas: array, shape: (max_features + 1,)
@@ -98,8 +103,13 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
     n_samples = y.size
     max_features = min(max_iter, n_features)
 
-    coefs = np.zeros((max_features + 1, n_features))
-    alphas = np.zeros(max_features + 1)
+    if return_path:
+        coefs = np.zeros((max_features + 1, n_features))
+        alphas = np.zeros(max_features + 1)
+    else:
+        coef, prev_coef = np.zeros(n_features), np.zeros(n_features)
+        alpha, prev_alpha = np.array([0.]), np.array([0.])  # better ideas?
+
     n_iter, n_active = 0, 0
     active, indices = list(), np.arange(n_features)
     # holds the sign of covariance
@@ -138,6 +148,7 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
             sys.stdout.flush()
 
     tiny = np.finfo(np.float).tiny  # to avoid division by 0 warning
+    tiny32 = np.finfo(np.float32).tiny  # to avoid division by 0 warning
 
     while True:
         if Cov.size:
@@ -147,17 +158,25 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
         else:
             C = 0.
 
-        alphas[n_iter] = C / n_samples
-        if alphas[n_iter] < alpha_min:  # early stopping
-            # interpolation factor 0 <= ss < 1
-            if n_iter > 0:
-                # In the first iteration, all alphas are zero, the formula
-                # below would make ss a NaN
-                ss = (alphas[n_iter - 1] - alpha_min) / (alphas[n_iter - 1] -
-                                                    alphas[n_iter])
-                coefs[n_iter] = coefs[n_iter - 1] + ss * (coefs[n_iter] -
-                                coefs[n_iter - 1])
-            alphas[n_iter] = alpha_min
+        if return_path:
+            alpha = alphas[n_iter, np.newaxis]
+            coef = coefs[n_iter]
+            prev_alpha = alphas[n_iter - 1, np.newaxis]
+            prev_coef = coefs[n_iter - 1]
+
+        alpha[0] = C / n_samples
+        if alpha[0] <= alpha_min:  # early stopping
+            if not alpha[0] == alpha_min:
+                # interpolation factor 0 <= ss < 1
+                if n_iter > 0:
+                    # In the first iteration, all alphas are zero, the formula
+                    # below would make ss a NaN
+                    ss = ((prev_alpha[0] - alpha_min) /
+                          (prev_alpha[0] - alpha[0]))
+                    coef[:] = prev_coef + ss * (coef - prev_coef)
+                alpha[0] = alpha_min
+            if return_path:
+                coefs[n_iter] = coef
             break
 
         if n_iter >= max_iter or n_active >= n_features:
@@ -179,6 +198,7 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
 
             Cov[C_idx], Cov[0] = swap(Cov[C_idx], Cov[0])
             indices[n], indices[m] = indices[m], indices[n]
+            Cov_not_shortened = Cov
             Cov = Cov[1:]  # remove Cov[0]
 
             if Gram is None:
@@ -201,19 +221,68 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
             diag = max(np.sqrt(np.abs(c - v)), eps)
             L[n_active, n_active] = diag
 
+            if diag < 1e-7:
+                # The system is becoming too ill-conditioned.
+                # We have degenerate vectors in our active set.
+                # We'll 'drop for good' the last regressor added
+                warnings.warn('Regressors in active set degenerate. '
+                              'Dropping a regressor, after %i iterations, '
+                              'i.e. alpha=%.3e, '
+                              'with an active set of %i regressors, and '
+                              'the smallest cholesky pivot element being %.3e'
+                              % (n_iter, alphas[n_iter], n_active, diag))
+                # XXX: need to figure a 'drop for good' way
+                Cov = Cov_not_shortened
+                Cov[0] = 0
+                Cov[C_idx], Cov[0] = swap(Cov[C_idx], Cov[0])
+                continue
+
             active.append(indices[n_active])
             n_active += 1
 
             if verbose > 1:
                 print "%s\t\t%s\t\t%s\t\t%s\t\t%s" % (n_iter, active[-1], '',
-                                                            n_active, C)
+                                                      n_active, C)
+
+        if method == 'lasso' and n_iter > 0 and prev_alpha[0] < alpha[0]:
+            # alpha is increasing. This is because the updates of Cov are
+            # bringing in too much numerical error that is greater than
+            # than the remaining correlation with the
+            # regressors. Time to bail out
+            warnings.warn('Early stopping the lars path, as the residues '
+                          'are small and the current value of alpha is no '
+                          'longer well controled. %i iterations, alpha=%.3e, '
+                          'previous alpha=%.3e, with an active set of %i '
+                          'regressors.'
+                          % (n_iter, alpha, prev_alpha, n_active))
+            break
+
         # least squares solution
         least_squares, info = solve_cholesky(L[:n_active, :n_active],
-                               sign_active[:n_active], lower=True)
+                                             sign_active[:n_active],
+                                             lower=True)
 
-        # is this really needed ?
-        AA = 1. / np.sqrt(np.sum(least_squares * sign_active[:n_active]))
-        least_squares *= AA
+        if least_squares.size == 1 and least_squares == 0:
+            # This happens because sign_active[:n_active] = 0
+            least_squares[...] = 1
+            AA = 1.
+        else:
+            # is this really needed ?
+            AA = 1. / np.sqrt(np.sum(least_squares * sign_active[:n_active]))
+
+            if not np.isfinite(AA):
+                # L is too ill-conditioned
+                i = 0
+                L_ = L[:n_active, :n_active].copy()
+                while not np.isfinite(AA):
+                    L_.flat[::n_active + 1] += (2 ** i) * eps
+                    least_squares, info = solve_cholesky(
+                        L_, sign_active[:n_active], lower=True)
+                    tmp = max(np.sum(least_squares * sign_active[:n_active]),
+                              eps)
+                    AA = 1. / np.sqrt(tmp)
+                    i += 1
+            least_squares *= AA
 
         if Gram is None:
             # equiangular direction of variables in the active set
@@ -234,7 +303,7 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
 
         # TODO: better names for these variables: z
         drop = False
-        z = -coefs[n_iter, active] / least_squares
+        z = -coef[active] / (least_squares + tiny32)
         z_pos = arrayfuncs.min_pos(z)
         if z_pos < gamma_:
             # some coefficients have changed sign
@@ -249,14 +318,24 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
 
         n_iter += 1
 
-        if n_iter >= coefs.shape[0]:
-            # resize the coefs and alphas array
-            add_features = 2 * max(1, (max_features - n_active))
-            coefs.resize((n_iter + add_features, n_features))
-            alphas.resize(n_iter + add_features)
+        if return_path:
+            if n_iter >= coefs.shape[0]:
+                del coef, alpha, prev_alpha, prev_coef
+                # resize the coefs and alphas array
+                add_features = 2 * max(1, (max_features - n_active))
+                coefs.resize((n_iter + add_features, n_features))
+                alphas.resize(n_iter + add_features)
+            coef = coefs[n_iter]
+            prev_coef = coefs[n_iter - 1]
+            alpha = alphas[n_iter, np.newaxis]
+            prev_alpha = alphas[n_iter - 1, np.newaxis]
+        else:
+            # mimic the effect of incrementing n_iter on the array references
+            prev_coef = coef
+            prev_alpha[0] = alpha[0]
+            coef = np.zeros_like(coef)
 
-        coefs[n_iter, active] = coefs[n_iter - 1, active] + \
-                                gamma_ * least_squares
+        coef[active] = prev_coef[active] + gamma_ * least_squares
 
         # update correlations
         Cov -= gamma_ * corr_eq_dir
@@ -274,19 +353,17 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
                 # propagate dropped variable
                 for i in range(idx, n_active):
                     X.T[i], X.T[i + 1] = swap(X.T[i], X.T[i + 1])
-                    indices[i], indices[i + 1] = \
-                            indices[i + 1], indices[i]  # yeah this is stupid
+                    # yeah this is stupid
+                    indices[i], indices[i + 1] = indices[i + 1], indices[i]
 
                 # TODO: this could be updated
-                residual = y - np.dot(X[:, :n_active],
-                                      coefs[n_iter, active])
+                residual = y - np.dot(X[:, :n_active], coef[active])
                 temp = np.dot(X.T[n_active], residual)
 
                 Cov = np.r_[temp, Cov]
             else:
                 for i in range(idx, n_active):
-                    indices[i], indices[i + 1] = \
-                                indices[i + 1], indices[i]
+                    indices[i], indices[i + 1] = indices[i + 1], indices[i]
                     Gram[i], Gram[i + 1] = swap(Gram[i], Gram[i + 1])
                     Gram[:, i], Gram[:, i + 1] = swap(Gram[:, i],
                                                       Gram[:, i + 1])
@@ -298,7 +375,7 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
                 # wrong as Xy is not swapped with the rest of variables
 
                 # TODO: this could be updated
-                residual = y - np.dot(X, coefs[n_iter])
+                residual = y - np.dot(X, coef)
                 temp = np.dot(X.T[drop_idx], residual)
                 Cov = np.r_[temp, Cov]
 
@@ -308,17 +385,20 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
                 print "%s\t\t%s\t\t%s\t\t%s\t\t%s" % (n_iter, '', drop_idx,
                                                       n_active, abs(temp))
 
-    # resize coefs in case of early stop
-    alphas = alphas[:n_iter + 1]
-    coefs = coefs[:n_iter + 1]
+    if return_path:
+        # resize coefs in case of early stop
+        alphas = alphas[:n_iter + 1]
+        coefs = coefs[:n_iter + 1]
 
-    return alphas, active, coefs.T
+        return alphas, active, coefs.T
+    else:
+        return alpha, active, coef
 
 
 ###############################################################################
 # Estimator classes
 
-class Lars(LinearModel):
+class Lars(LinearModel, RegressorMixin):
     """Least Angle Regression model a.k.a. LAR
 
     Parameters
@@ -352,14 +432,24 @@ class Lars(LinearModel):
         optimization-based algorithms, this parameter does not control
         the tolerance of the optimization.
 
+    fit_path : boolean
+        If True the full path is stored in the `coef_path_` attribute.
+        If you compute the solution for a large problem or many targets,
+        setting fit_path to False will lead to a speedup, especially
+        with a small alpha.
 
     Attributes
     ----------
+    `coef_path_` : array, shape = [n_features, n_alpha]
+        The varying values of the coefficients along the path. It is not \
+    present if the fit_path parameter is False.
+
+
     `coef_` : array, shape = [n_features]
-        parameter vector (w in the fomulation formula)
+        Parameter vector (w in the fomulation formula).
 
     `intercept_` : float
-        independent term in decision function.
+        Independent term in decision function.
 
     Examples
     --------
@@ -367,9 +457,9 @@ class Lars(LinearModel):
     >>> clf = linear_model.Lars(n_nonzero_coefs=1)
     >>> clf.fit([[-1, 1], [0, 0], [1, 1]], [-1.1111, 0, -1.1111])
     ... # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
-    Lars(copy_X=True, eps=..., fit_intercept=True, n_nonzero_coefs=1,
-       normalize=True, precompute='auto', verbose=False)
-    >>> print clf.coef_ # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
+    Lars(copy_X=True, eps=..., fit_intercept=True, fit_path=True,
+       n_nonzero_coefs=1, normalize=True, precompute='auto', verbose=False)
+    >>> print(clf.coef_) # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
     [ 0. -1.11...]
 
     See also
@@ -381,7 +471,7 @@ class Lars(LinearModel):
     """
     def __init__(self, fit_intercept=True, verbose=False, normalize=True,
                  precompute='auto', n_nonzero_coefs=500,
-                 eps=np.finfo(np.float).eps, copy_X=True):
+                 eps=np.finfo(np.float).eps, copy_X=True, fit_path=True):
         self.fit_intercept = fit_intercept
         self.verbose = verbose
         self.normalize = normalize
@@ -390,29 +480,33 @@ class Lars(LinearModel):
         self.n_nonzero_coefs = n_nonzero_coefs
         self.eps = eps
         self.copy_X = copy_X
+        self.fit_path = fit_path
 
     def _get_gram(self):
         # precompute if n_samples > n_features
         precompute = self.precompute
         if hasattr(precompute, '__array__'):
-            # copy as it's going to be modified
-            Gram = precompute.copy()
+            Gram = precompute
         elif precompute == 'auto':
             Gram = 'auto'
         else:
             Gram = None
         return Gram
 
-    def fit(self, X, y):
+    def fit(self, X, y, Xy=None):
         """Fit the model using X, y as training data.
 
         parameters
         ----------
         X : array-like, shape = [n_samples, n_features]
-            training data.
+            Training data.
 
-        y : array-like, shape = [n_samples]
-            target values.
+        y : array-like, shape = [n_samples] or [n_samples, n_targets]
+            Target values.
+
+        Xy : array-like, shape = [n_samples] or [n_samples, n_targets], optional
+            Xy = np.dot(X.T, y) that can be precomputed. It is useful
+            only when the Gram matrix is precomputed.
 
         returns
         -------
@@ -421,11 +515,18 @@ class Lars(LinearModel):
         """
         X = array2d(X)
         y = np.asarray(y)
+        n_features = X.shape[1]
 
         X, y, X_mean, y_mean, X_std = self._center_data(X, y,
                                                         self.fit_intercept,
                                                         self.normalize,
                                                         self.copy_X)
+
+        if y.ndim == 1:
+            y = y[:, np.newaxis]
+
+        n_targets = y.shape[1]
+
         alpha = getattr(self, 'alpha', 0.)
         if hasattr(self, 'n_nonzero_coefs'):
             alpha = 0.  # n_nonzero_coefs parametrization takes priority
@@ -433,18 +534,50 @@ class Lars(LinearModel):
         else:
             max_iter = self.max_iter
 
-        Gram = self._get_gram()
+        precompute = self.precompute
+        if not hasattr(precompute, '__array__') and (
+                precompute is True or
+                (precompute == 'auto' and X.shape[0] > X.shape[1]) or
+                (precompute == 'auto' and y.shape[1] > 1)):
+            Gram = np.dot(X.T, X)
+        else:
+            Gram = self._get_gram()
 
-        self.alphas_, self.active_, self.coef_path_ = lars_path(X, y,
-                  Gram=Gram, copy_X=self.copy_X,
-                  copy_Gram=False, alpha_min=alpha,
-                  method=self.method, verbose=max(0, self.verbose - 1),
-                  max_iter=max_iter, eps=self.eps)
+        self.alphas_ = []
 
-        self.coef_ = self.coef_path_[:, -1]
+        if self.fit_path:
+            self.coef_ = []
+            self.active_ = []
+            self.coef_path_ = []
+            for k in xrange(n_targets):
+                this_Xy = None if Xy is None else Xy[:, k]
+                alphas, active, coef_path = lars_path(
+                    X, y[:, k], Gram=Gram, Xy=this_Xy, copy_X=self.copy_X,
+                    copy_Gram=True, alpha_min=alpha, method=self.method,
+                    verbose=max(0, self.verbose - 1), max_iter=max_iter,
+                    eps=self.eps, return_path=True)
+                self.alphas_.append(alphas)
+                self.active_.append(active)
+                self.coef_path_.append(coef_path)
+                self.coef_.append(coef_path[:, -1])
 
+            if n_targets == 1:
+                self.alphas_, self.active_, self.coef_path_, self.coef_ = [
+                    a[0] for a in (self.alphas_, self.active_, self.coef_path_,
+                                   self.coef_)]
+        else:
+            self.coef_ = np.empty((n_targets, n_features))
+            for k in xrange(n_targets):
+                this_Xy = None if Xy is None else Xy[:, k]
+                alphas, _, self.coef_[k] = lars_path(
+                    X, y[:, k], Gram=Gram, Xy=this_Xy, copy_X=self.copy_X,
+                    copy_Gram=True, alpha_min=alpha, method=self.method,
+                    verbose=max(0, self.verbose - 1), max_iter=max_iter,
+                    eps=self.eps, return_path=False)
+                self.alphas_.append(alphas)
+            if n_targets == 1:
+                self.alphas_ = self.alphas_[0]
         self._set_intercept(X_mean, y_mean, X_std)
-
         return self
 
 
@@ -459,6 +592,13 @@ class LassoLars(Lars):
 
     Parameters
     ----------
+    alpha : float
+        Constant that multiplies the penalty term. Defaults to 1.0.
+        alpha = 0 is equivalent to an ordinary least square, solved
+        by the LinearRegression object in the scikit. For numerical
+        reasons, using alpha = 0 with the LassoLars object is not advised
+        and you should prefer the LinearRegression object.
+
     fit_intercept : boolean
         whether to calculate the intercept for this model. If set
         to false, no intercept will be used in calculations
@@ -488,14 +628,23 @@ class LassoLars(Lars):
         optimization-based algorithms, this parameter does not control
         the tolerance of the optimization.
 
+    fit_path : boolean
+        If True the full path is stored in the `coef_path_` attribute.
+        If you compute the solution for a large problem or many targets,
+        setting fit_path to False will lead to a speedup, especially
+        with a small alpha.
 
     Attributes
     ----------
+    `coef_path_` : array, shape = [n_features, n_alpha]
+        The varying values of the coefficients along the path. It is not \
+    present if fit_path parameter is False.
+
     `coef_` : array, shape = [n_features]
-        parameter vector (w in the fomulation formula)
+        Parameter vector (w in the fomulation formula).
 
     `intercept_` : float
-        independent term in decision function.
+        Independent term in decision function.
 
     Examples
     --------
@@ -504,8 +653,9 @@ class LassoLars(Lars):
     >>> clf.fit([[-1, 1], [0, 0], [1, 1]], [-1, 0, -1])
     ... # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
     LassoLars(alpha=0.01, copy_X=True, eps=..., fit_intercept=True,
-         max_iter=500, normalize=True, precompute='auto', verbose=False)
-    >>> print clf.coef_ # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
+         fit_path=True, max_iter=500, normalize=True, precompute='auto',
+         verbose=False)
+    >>> print(clf.coef_) # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
     [ 0.         -0.963257...]
 
     See also
@@ -522,7 +672,7 @@ class LassoLars(Lars):
 
     def __init__(self, alpha=1.0, fit_intercept=True, verbose=False,
                  normalize=True, precompute='auto', max_iter=500,
-                 eps=np.finfo(np.float).eps, copy_X=True):
+                 eps=np.finfo(np.float).eps, copy_X=True, fit_path=True):
         self.alpha = alpha
         self.fit_intercept = fit_intercept
         self.max_iter = max_iter
@@ -532,17 +682,7 @@ class LassoLars(Lars):
         self.precompute = precompute
         self.copy_X = copy_X
         self.eps = eps
-
-
-# Deprecated classes
-@deprecated("Use Lars instead")
-class LARS(Lars):
-    pass
-
-
-@deprecated("Use LassoLars instead")
-class LassoLARS(LassoLars):
-    pass
+        self.fit_path = fit_path
 
 
 ###############################################################################
@@ -618,7 +758,9 @@ def _lars_path_residues(X_train, y_train, X_test, y_test, Gram=None,
         X_train -= X_mean
         X_test -= X_mean
         y_mean = y_train.mean(axis=0)
+        y_train = as_float_array(y_train, copy=False)
         y_train -= y_mean
+        y_test = as_float_array(y_test, copy=False)
         y_test -= y_mean
 
     if normalize:
@@ -626,10 +768,9 @@ def _lars_path_residues(X_train, y_train, X_test, y_test, Gram=None,
         nonzeros = np.flatnonzero(norms)
         X_train[:, nonzeros] /= norms[nonzeros]
 
-    alphas, active, coefs = lars_path(X_train, y_train, Gram=Gram,
-                            copy_X=False, copy_Gram=False,
-                            method=method, verbose=max(0, verbose - 1),
-                            max_iter=max_iter, eps=eps)
+    alphas, active, coefs = lars_path(
+        X_train, y_train, Gram=Gram, copy_X=False, copy_Gram=False,
+        method=method, verbose=max(0, verbose - 1), max_iter=max_iter, eps=eps)
     if normalize:
         coefs[nonzeros] /= norms[nonzeros][:, np.newaxis]
     residues = np.array([(np.dot(X_test, coef) - y_test)
@@ -637,7 +778,7 @@ def _lars_path_residues(X_train, y_train, X_test, y_test, Gram=None,
     return alphas, active, coefs, residues
 
 
-class LarsCV(LARS):
+class LarsCV(Lars):
     """Cross-validated Least Angle Regression model
 
     Parameters
@@ -688,14 +829,27 @@ class LarsCV(LARS):
         parameter vector (w in the fomulation formula)
 
     `intercept_` : float
-        independent term in decision function.
+        independent term in decision function
 
-    `coef_path`: array, shape = [n_features, n_alpha]
+    `coef_path_`: array, shape = [n_features, n_alpha]
         the varying values of the coefficients along the path
 
+    `alpha_`: float
+        the estimated regularization parameter alpha
+
+    `alphas_`: array, shape = [n_alpha]
+        the different values of alpha along the path
+
+    `cv_alphas_`: array, shape = [n_cv_alphas]
+        all the values of alpha along the path for the different folds
+
+    `cv_mse_path_`: array, shape = [n_folds, n_cv_alphas]
+        the mean square error on left-out for each fold along the path
+        (alpha values given by cv_alphas)
+
     See also
     --------
-    lars_path, LassoLARS, LassoLarsCV
+    lars_path, LassoLars, LassoLarsCV
     """
 
     method = 'lar'
@@ -731,7 +885,8 @@ class LarsCV(LARS):
         self : object
             returns an instance of self.
         """
-        X = np.asarray(X)
+        self.fit_path = True
+        X = array2d(X)
 
         # init cross-validation generator
         cv = check_cv(self.cv, X, y, classifier=False)
@@ -739,15 +894,12 @@ class LarsCV(LARS):
         Gram = 'auto' if self.precompute else None
 
         cv_paths = Parallel(n_jobs=self.n_jobs, verbose=self.verbose)(
-                    delayed(_lars_path_residues)(X[train], y[train],
-                            X[test], y[test], Gram=Gram,
-                            copy=False, method=self.method,
-                            verbose=max(0, self.verbose - 1),
-                            normalize=self.normalize,
-                            fit_intercept=self.fit_intercept,
-                            max_iter=self.max_iter,
-                            eps=self.eps)
-                    for train, test in cv)
+            delayed(_lars_path_residues)(
+                X[train], y[train], X[test], y[test], Gram=Gram, copy=False,
+                method=self.method, verbose=max(0, self.verbose - 1),
+                normalize=self.normalize, fit_intercept=self.fit_intercept,
+                max_iter=self.max_iter, eps=self.eps)
+            for train, test in cv)
         all_alphas = np.concatenate(list(zip(*cv_paths))[0])
         # Unique also sorts
         all_alphas = np.unique(all_alphas)
@@ -779,14 +931,29 @@ class LarsCV(LARS):
         best_alpha = all_alphas[i_best_alpha]
 
         # Store our parameters
-        self.alpha = best_alpha
-        self.cv_alphas = all_alphas
+        self.alpha_ = best_alpha
+        self.cv_alphas_ = all_alphas
         self.cv_mse_path_ = mse_path
 
         # Now compute the full model
-        LARS.fit(self, X, y)
+        # it will call a lasso internally when self if LassoLarsCV
+        # as self.method == 'lasso'
+        Lars.fit(self, X, y)
         return self
 
+    @property
+    def alpha(self):
+        # impedance matching for the above Lars.fit (should not be documented)
+        return self.alpha_
+
+    @property
+    def cv_alphas(self):
+        warnings.warn("Use cv_alphas_. Using cv_alphas is deprecated"
+                      "since version 0.12, and backward compatibility "
+                      "won't be maintained from version 0.14 onward. ",
+                      DeprecationWarning, stacklevel=2)
+        return self.cv_alphas_
+
 
 class LassoLarsCV(LarsCV):
     """Cross-validated Lasso, using the LARS algorithm
@@ -844,20 +1011,22 @@ class LassoLarsCV(LarsCV):
     `intercept_` : float
         independent term in decision function.
 
-    `coef_path`: array, shape = [n_features, n_alpha]
+    `coef_path_`: array, shape = [n_features, n_alpha]
         the varying values of the coefficients along the path
 
+    `alpha_`: float
+        the estimated regularization parameter alpha
+
     `alphas_`: array, shape = [n_alpha]
         the different values of alpha along the path
 
-    `cv_alphas`: array, shape = [n_cv_alphas]
+    `cv_alphas_`: array, shape = [n_cv_alphas]
         all the values of alpha along the path for the different folds
 
     `cv_mse_path_`: array, shape = [n_folds, n_cv_alphas]
         the mean square error on left-out for each fold along the path
         (alpha values given by cv_alphas)
 
-
     Notes
     -----
 
@@ -947,7 +1116,7 @@ class LassoLarsIC(LassoLars):
     LassoLarsIC(copy_X=True, criterion='bic', eps=..., fit_intercept=True,
           max_iter=500, normalize=True, precompute='auto',
           verbose=False)
-    >>> print clf.coef_ # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
+    >>> print(clf.coef_) # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
     [ 0.  -1.11...]
 
     Notes
@@ -968,8 +1137,6 @@ class LassoLarsIC(LassoLars):
     def __init__(self, criterion='aic', fit_intercept=True, verbose=False,
                  normalize=True, precompute='auto', max_iter=500,
                  eps=np.finfo(np.float).eps, copy_X=True):
-        if criterion not in ['aic', 'bic']:
-            raise ValueError('criterion should be either bic or aic')
         self.criterion = criterion
         self.fit_intercept = fit_intercept
         self.max_iter = max_iter
@@ -995,22 +1162,20 @@ class LassoLarsIC(LassoLars):
         self : object
             returns an instance of self.
         """
+        self.fit_path = True
         X = array2d(X)
         y = np.asarray(y)
 
-        X, y, Xmean, ymean, Xstd = LinearModel._center_data(X, y,
-                                                    self.fit_intercept,
-                                                    self.normalize,
-                                                    self.copy_X)
+        X, y, Xmean, ymean, Xstd = LinearModel._center_data(
+            X, y, self.fit_intercept, self.normalize, self.copy_X)
         max_iter = self.max_iter
 
         Gram = self._get_gram()
 
-        alphas_, active_, coef_path_ = lars_path(X, y,
-                  Gram=Gram, copy_X=copy_X,
-                  copy_Gram=False, alpha_min=0.0,
-                  method='lasso', verbose=self.verbose,
-                  max_iter=max_iter, eps=self.eps)
+        alphas_, active_, coef_path_ = lars_path(
+            X, y, Gram=Gram, copy_X=copy_X, copy_Gram=True, alpha_min=0.0,
+            method='lasso', verbose=self.verbose, max_iter=max_iter,
+            eps=self.eps)
 
         n_samples = X.shape[0]
 
