@@ -12,6 +12,7 @@ from scipy import linalg
 from scipy.linalg.lapack import get_lapack_funcs
 
 from .base import LinearModel
+from ..base import RegressorMixin
 from ..utils import array2d
 from ..utils.arrayfuncs import solve_triangular
 
@@ -251,7 +252,6 @@ def orthogonal_mp(X, y, n_nonzero_coefs=None, tol=None, precompute_gram=False,
     orthogonal_mp_gram
     lars_path
     decomposition.sparse_encode
-    decomposition.sparse_encode_parallel
 
     Notes
     -----
@@ -266,26 +266,24 @@ def orthogonal_mp(X, y, n_nonzero_coefs=None, tol=None, precompute_gram=False,
     http://www.cs.technion.ac.il/~ronrubin/Publications/KSVD-OMP-v2.pdf
 
     """
-    X = np.asarray(X)
+    X = array2d(X, order='F', copy=copy_X)
+    copy_X = False
     y = np.asarray(y)
     if y.ndim == 1:
         y = y[:, np.newaxis]
-    if copy_X:
-        X = X.copy('F')
-        copy_X = False
-    else:
-        X = np.asfortranarray(X)
     if y.shape[1] > 1:  # subsequent targets will be affected
         copy_X = True
-    if n_nonzero_coefs == None and tol == None:
-        n_nonzero_coefs = int(0.1 * X.shape[1])
+    if n_nonzero_coefs is None and tol is None:
+        # default for n_nonzero_coefs is 0.1 * n_features
+        # but at least one.
+        n_nonzero_coefs = max(int(0.1 * X.shape[1]), 1)
     if tol is not None and tol < 0:
         raise ValueError("Epsilon cannot be negative")
     if tol is None and n_nonzero_coefs <= 0:
         raise ValueError("The number of atoms must be positive")
     if tol is None and n_nonzero_coefs > X.shape[1]:
-        raise ValueError("The number of atoms cannot be more than the number \
-                          of features")
+        raise ValueError("The number of atoms cannot be more than the number "
+                         "of features")
     if precompute_gram == 'auto':
         precompute_gram = X.shape[0] > X.shape[1]
     if precompute_gram:
@@ -353,7 +351,6 @@ def orthogonal_mp_gram(Gram, Xy, n_nonzero_coefs=None, tol=None,
     orthogonal_mp
     lars_path
     decomposition.sparse_encode
-    decomposition.sparse_encode_parallel
 
     Notes
     -----
@@ -368,25 +365,28 @@ def orthogonal_mp_gram(Gram, Xy, n_nonzero_coefs=None, tol=None,
     http://www.cs.technion.ac.il/~ronrubin/Publications/KSVD-OMP-v2.pdf
 
     """
-    Gram = np.asarray(Gram)
+    Gram = array2d(Gram, order='F', copy=copy_Gram)
     Xy = np.asarray(Xy)
+    if Xy.ndim > 1 and Xy.shape[1] > 1:
+        # or subsequent target will be affected
+        copy_Gram = True
     if Xy.ndim == 1:
         Xy = Xy[:, np.newaxis]
         if tol is not None:
             norms_squared = [norms_squared]
 
-    if n_nonzero_coefs == None and tol is None:
+    if n_nonzero_coefs is None and tol is None:
         n_nonzero_coefs = int(0.1 * len(Gram))
-    if tol is not None and norms_squared == None:
-        raise ValueError('Gram OMP needs the precomputed norms in order \
-                          to evaluate the error sum of squares.')
+    if tol is not None and norms_squared is None:
+        raise ValueError('Gram OMP needs the precomputed norms in order '
+                         'to evaluate the error sum of squares.')
     if tol is not None and tol < 0:
-        raise ValueError("Epsilon cennot be negative")
+        raise ValueError("Epsilon cannot be negative")
     if tol is None and n_nonzero_coefs <= 0:
         raise ValueError("The number of atoms must be positive")
     if tol is None and n_nonzero_coefs > len(Gram):
-        raise ValueError("The number of atoms cannot be more than the number \
-                          of features")
+        raise ValueError("The number of atoms cannot be more than the number "
+                         "of features")
     coef = np.zeros((len(Gram), Xy.shape[1]))
     for k in range(Xy.shape[1]):
         x, idx = _gram_omp(Gram, Xy[:, k], n_nonzero_coefs,
@@ -396,7 +396,7 @@ def orthogonal_mp_gram(Gram, Xy, n_nonzero_coefs=None, tol=None,
     return np.squeeze(coef)
 
 
-class OrthogonalMatchingPursuit(LinearModel):
+class OrthogonalMatchingPursuit(LinearModel, RegressorMixin):
     """Orthogonal Mathching Pursuit model (OMP)
 
     Parameters
@@ -465,12 +465,11 @@ class OrthogonalMatchingPursuit(LinearModel):
     Lars
     LassoLars
     decomposition.sparse_encode
-    decomposition.sparse_encode_parallel
 
     """
-    def __init__(self, copy_X=True, copy_Gram=True,
-            copy_Xy=True, n_nonzero_coefs=None, tol=None,
-            fit_intercept=True, normalize=True, precompute_gram=False):
+    def __init__(self, copy_X=True, copy_Gram=True, copy_Xy=True,
+                 n_nonzero_coefs=None, tol=None, fit_intercept=True,
+                 normalize=True, precompute_gram=False):
         self.n_nonzero_coefs = n_nonzero_coefs
         self.tol = tol
         self.fit_intercept = fit_intercept
@@ -516,32 +515,21 @@ class OrthogonalMatchingPursuit(LinearModel):
         if y.ndim == 1:
             y = y[:, np.newaxis]
 
-        if self.n_nonzero_coefs == None and self.tol is None:
-            self.n_nonzero_coefs = int(0.1 * n_features)
-        if (Gram is not None or Xy is not None) and (self.fit_intercept is True
-                                                 or self.normalize is True):
-            warnings.warn('Mean subtraction (fit_intercept) and '
-                 'normalization cannot be applied on precomputed Gram '
-                 'and Xy matrices. Your precomputed values are ignored '
-                 'and recomputed. To avoid this, do the scaling yourself '
-                 'and call with fit_intercept and normalize set to False.',
-                 RuntimeWarning, stacklevel=2)
+        if self.n_nonzero_coefs is None and self.tol is None:
+            # default for n_nonzero_coefs is 0.1 * n_features
+            # but at least one.
+            self.n_nonzero_coefs = max(int(0.1 * n_features), 1)
+        if (Gram is not None or Xy is not None) and (self.fit_intercept
+                                                     or self.normalize):
+            warnings.warn('Mean subtraction (fit_intercept) and normalization '
+                          'cannot be applied on precomputed Gram and Xy '
+                          'matrices. Your precomputed values are ignored and '
+                          'recomputed. To avoid this, do the scaling yourself '
+                          'and call with fit_intercept and normalize set to '
+                          'False.', RuntimeWarning, stacklevel=2)
             Gram, Xy = None, None
 
         if Gram is not None:
-            Gram = array2d(Gram)
-
-            if self.copy_Gram:
-                copy_Gram = False
-                Gram = Gram.copy('F')
-            else:
-                Gram = np.asfortranarray(Gram)
-
-            copy_Gram = self.copy_Gram
-
-            if y.shape[1] > 1:  # subsequent targets will be affected
-                copy_Gram = True
-
             if Xy is None:
                 Xy = np.dot(X.T, y)
             else:
@@ -560,7 +548,7 @@ class OrthogonalMatchingPursuit(LinearModel):
             norms_sq = np.sum(y ** 2, axis=0) if self.tol is not None else None
             self.coef_ = orthogonal_mp_gram(Gram, Xy, self.n_nonzero_coefs,
                                             self.tol, norms_sq,
-                                            copy_Gram, True).T
+                                            self.copy_Gram, True).T
         else:
             precompute_gram = self.precompute_gram
             if precompute_gram == 'auto':
