@@ -2,80 +2,73 @@
 Ridge regression
 """
 
-# Author:   Mathieu Blondel <mathieu@mblondel.org>
-#           Reuben Fletcher-Costin <reuben.fletchercostin@gmail.com>
+# Author: Mathieu Blondel <mathieu@mblondel.org>
+#         Reuben Fletcher-Costin <reuben.fletchercostin@gmail.com>
+#         Fabian Pedregosa <fabian@fseoane.net>
 # License: Simplified BSD
 
 
+from abc import ABCMeta, abstractmethod
 import warnings
-import numpy as np
 
-from .base import LinearModel
+import numpy as np
+from scipy import linalg
+from scipy import sparse
+from scipy.sparse import linalg as sp_linalg
+
+from .base import LinearClassifierMixin, LinearModel
+from ..base import RegressorMixin
 from ..utils.extmath import safe_sparse_dot
 from ..utils import safe_asarray
 from ..preprocessing import LabelBinarizer
 from ..grid_search import GridSearchCV
 
 
-def _solve(A, b, solver, tol):
-    # helper method for ridge_regression, A is symmetric positive
-
-    if solver == 'auto':
-        if hasattr(A, 'todense'):
-            solver = 'sparse_cg'
-        else:
-            solver = 'dense_cholesky'
-
-    if solver == 'sparse_cg':
-        if b.ndim < 2:
-            from scipy.sparse import linalg as sp_linalg
-            sol, error = sp_linalg.cg(A, b, tol=tol)
-            if error:
-                raise ValueError("Failed with error code %d" % error)
-            return sol
-        else:
-            # sparse_cg cannot handle a 2-d b.
-            sol = []
-            for j in range(b.shape[1]):
-                sol.append(_solve(A, b[:, j], solver="sparse_cg", tol=tol))
-            return np.array(sol).T
-
-    elif solver == 'dense_cholesky':
-        from scipy import linalg
-        if hasattr(A, 'todense'):
-            A = A.todense()
-        return linalg.solve(A, b, sym_pos=True, overwrite_a=True)
-    else:
-        raise NotImplementedError('Solver %s not implemented' % solver)
-
-
-def ridge_regression(X, y, alpha, sample_weight=1.0, solver='auto', tol=1e-3):
+def ridge_regression(X, y, alpha, sample_weight=1.0, solver='auto',
+                     max_iter=None, tol=1e-3):
     """Solve the ridge equation by the method of normal equations.
 
     Parameters
     ----------
-    X : {array-like, sparse matrix}, shape = [n_samples, n_features]
+    X : {array-like, sparse matrix, LinearOperator},
+        shape = [n_samples, n_features]
         Training data
 
-    y : array-like, shape = [n_samples] or [n_samples, n_responses]
+    y : array-like, shape = [n_samples] or [n_samples, n_targets]
         Target values
 
+    max_iter : int, optional
+        Maximum number of iterations for conjugate gradient solver.
+        The default value is determined by scipy.sparse.linalg.
+
     sample_weight : float or numpy array of shape [n_samples]
         Individual weights for each sample
 
-    solver : {'auto', 'dense_cholesky', 'sparse_cg'}, optional
-        Solver to use in the computational routines. 'delse_cholesky'
-        will use the standard scipy.linalg.solve function, 'sparse_cg'
-        will use the conjugate gradient solver as found in
-        scipy.sparse.linalg.cg while 'auto' will chose the most
-        appropriate depending on the matrix X.
+    solver : {'auto', 'dense_cholesky', 'lsqr', 'sparse_cg'}
+        Solver to use in the computational routines:
+
+        - 'auto' chooses the solver automatically based on the type of data.
+
+        - 'dense_cholesky' uses the standard scipy.linalg.solve function to
+          obtain a closed-form solution.
+
+        - 'sparse_cg' uses the conjugate gradient solver as found in
+          scipy.sparse.linalg.cg. As an iterative algorithm, this solver is
+          more appropriate than 'dense_cholesky' for large-scale data
+          (possibility to set `tol` and `max_iter`).
+
+        - 'lsqr' uses the dedicated regularized least-squares routine
+          scipy.sparse.linalg.lsqr. It is the fatest but may not be available
+          in old scipy versions. It also uses an iterative procedure.
+
+        All three solvers support both dense and sparse data.
 
     tol: float
         Precision of the solution.
 
     Returns
     -------
-    coef: array, shape = [n_features] or [n_responses, n_features]
+    coef: array, shape = [n_features] or [n_targets, n_features]
         Weight vector(s).
 
     Notes
@@ -84,80 +77,209 @@ def ridge_regression(X, y, alpha, sample_weight=1.0, solver='auto', tol=1e-3):
     """
 
     n_samples, n_features = X.shape
-    is_sparse = False
+    has_sw = isinstance(sample_weight, np.ndarray) or sample_weight != 1.0
 
-    if hasattr(X, 'todense'):  # lazy import of scipy.sparse
-        from scipy import sparse
-        is_sparse = sparse.issparse(X)
-
-    if is_sparse:
-        if n_features > n_samples or \
-           isinstance(sample_weight, np.ndarray) or \
-           sample_weight != 1.0:
-
-            I = sparse.lil_matrix((n_samples, n_samples))
-            I.setdiag(np.ones(n_samples) * alpha * sample_weight)
-            c = _solve(X * X.T + I, y, solver, tol)
-            coef = X.T * c
+    if solver == 'auto':
+        # cholesky if it's a dense array and cg in
+        # any other case
+        if hasattr(X, '__array__'):
+            solver = 'dense_cholesky'
         else:
-            I = sparse.lil_matrix((n_features, n_features))
-            I.setdiag(np.ones(n_features) * alpha)
-            coef = _solve(X.T * X + I, X.T * y, solver, tol)
-    else:
-        if n_features > n_samples or \
-           isinstance(sample_weight, np.ndarray) or \
-           sample_weight != 1.0:
+            solver = 'sparse_cg'
 
+    elif solver == 'lsqr' and not hasattr(sp_linalg, 'lsqr'):
+        warnings.warn("""lsqr not available on this machine, falling back
+                      to sparse_cg.""")
+        solver = 'sparse_cg'
+
+    if has_sw:
+        solver = 'dense_cholesky'
+
+    if solver == 'sparse_cg':
+        # gradient descent
+        X1 = sp_linalg.aslinearoperator(X)
+        if y.ndim == 1:
+            y1 = np.reshape(y, (-1, 1))
+        else:
+            y1 = y
+        coefs = np.empty((y1.shape[1], n_features))
+
+        if n_features > n_samples:
+            def mv(x):
+                return X1.matvec(X1.rmatvec(x)) + alpha * x
+        else:
+            def mv(x):
+                return X1.rmatvec(X1.matvec(x)) + alpha * x
+
+        for i in range(y1.shape[1]):
+            y_column = y1[:, i]
+            if n_features > n_samples:
+                # kernel ridge
+                # w = X.T * inv(X X^t + alpha*Id) y
+                C = sp_linalg.LinearOperator(
+                    (n_samples, n_samples), matvec=mv, dtype=X.dtype)
+                coef, info = sp_linalg.cg(C, y_column, tol=tol)
+                coefs[i] = X1.rmatvec(coef)
+            else:
+                # ridge
+                # w = inv(X^t X + alpha*Id) * X.T y
+                y_column = X1.rmatvec(y_column)
+                C = sp_linalg.LinearOperator(
+                    (n_features, n_features), matvec=mv, dtype=X.dtype)
+                coefs[i], info = sp_linalg.cg(C, y_column, maxiter=max_iter,
+                                              tol=tol)
+            if info != 0:
+                raise ValueError("Failed with error code %d" % info)
+
+        if y.ndim == 1:
+            coefs = np.ravel(coefs)
+
+        return coefs
+    elif solver == "lsqr":
+        if y.ndim == 1:
+            y1 = np.reshape(y, (-1, 1))
+        else:
+            y1 = y
+        coefs = np.empty((y1.shape[1], n_features))
+
+        # According to the lsqr documentation, alpha = damp^2.
+        sqrt_alpha = np.sqrt(alpha)
+
+        for i in range(y1.shape[1]):
+            y_column = y1[:, i]
+            coefs[i] = sp_linalg.lsqr(X, y_column, damp=sqrt_alpha,
+                                      atol=tol, btol=tol, iter_lim=max_iter)[0]
+
+        if y.ndim == 1:
+            coefs = np.ravel(coefs)
+
+        return coefs
+    else:
+        # normal equations (cholesky) method
+        if n_features > n_samples or has_sw:
             # kernel ridge
             # w = X.T * inv(X X^t + alpha*Id) y
-            A = np.dot(X, X.T)
-            A.flat[::n_samples + 1] += alpha * sample_weight
-            coef = np.dot(X.T, _solve(A, y, solver, tol))
+            K = safe_sparse_dot(X, X.T, dense_output=True)
+            if has_sw:
+                # We are doing a little danse with the sample weights to
+                # avoid copying the original X, which could be big
+                sw = np.sqrt(sample_weight)
+                if y.ndim == 1:
+                    y = y * sw
+                else:
+                    # Deal with multiple-output problems
+                    y = y * sw[:, np.newaxis]
+                K *= np.outer(sw, sw)
+            K.flat[::n_samples + 1] += alpha
+            dual_coef = linalg.solve(K, y,
+                                     sym_pos=True, overwrite_a=True)
+            if has_sw:
+                if dual_coef.ndim == 1:
+                    dual_coef *= sw
+                else:
+                    # Deal with multiple-output problems
+                    dual_coef *= sw[:, np.newaxis]
+            coef = safe_sparse_dot(X.T, dual_coef, dense_output=True)
         else:
             # ridge
             # w = inv(X^t X + alpha*Id) * X.T y
-            A = np.dot(X.T, X)
+            A = safe_sparse_dot(X.T, X, dense_output=True)
             A.flat[::n_features + 1] += alpha
-            coef = _solve(A, np.dot(X.T, y), solver, tol)
+            Xy = safe_sparse_dot(X.T, y, dense_output=True)
+            coef = linalg.solve(A, Xy, sym_pos=True, overwrite_a=True)
 
-    return coef.T
+        return coef.T
 
 
-class Ridge(LinearModel):
+class _BaseRidge(LinearModel):
+    __metaclass__ = ABCMeta
+
+    @abstractmethod
+    def __init__(self, alpha=1.0, fit_intercept=True, normalize=False,
+                 copy_X=True, max_iter=None, tol=1e-3, solver="auto"):
+        self.alpha = alpha
+        self.fit_intercept = fit_intercept
+        self.normalize = normalize
+        self.copy_X = copy_X
+        self.max_iter = max_iter
+        self.tol = tol
+        self.solver = solver
+
+    def fit(self, X, y, sample_weight=1.0, solver=None):
+        X = safe_asarray(X, dtype=np.float)
+        y = np.asarray(y, dtype=np.float)
+
+        X, y, X_mean, y_mean, X_std = self._center_data(
+            X, y, self.fit_intercept, self.normalize, self.copy_X,
+            sample_weight=sample_weight)
+
+        self.coef_ = ridge_regression(X, y,
+                                      alpha=self.alpha,
+                                      sample_weight=sample_weight,
+                                      solver=solver,
+                                      max_iter=self.max_iter,
+                                      tol=self.tol)
+        self._set_intercept(X_mean, y_mean, X_std)
+        return self
+
+
+class Ridge(_BaseRidge, RegressorMixin):
     """Linear least squares with l2 regularization.
 
     This model solves a regression model where the loss function is
     the linear least squares function and regularization is given by
     the l2-norm. Also known as Ridge Regression or Tikhonov regularization.
     This estimator has built-in support for multi-variate regression
-    (i.e., when y is a 2d-array of shape [n_samples, n_responses]).
+    (i.e., when y is a 2d-array of shape [n_samples, n_targets]).
 
     Parameters
     ----------
     alpha : float
-        Small positive values of alpha improve the conditioning of the
-        problem and reduce the variance of the estimates.
-        Alpha corresponds to (2*C)^-1 in other linear models such as
-        LogisticRegression or LinearSVC.
+        Small positive values of alpha improve the conditioning of the problem
+        and reduce the variance of the estimates.  Alpha corresponds to
+        ``(2*C)^-1`` in other linear models such as LogisticRegression or
+        LinearSVC.
+
+    copy_X : boolean, optional, default True
+        If True, X will be copied; else, it may be overwritten.
 
     fit_intercept : boolean
         Whether to calculate the intercept for this model. If set
         to false, no intercept will be used in calculations
         (e.g. data is expected to be already centered).
 
+    max_iter : int, optional
+        Maximum number of iterations for conjugate gradient solver.
+        The default value is determined by scipy.sparse.linalg.
+
     normalize : boolean, optional
         If True, the regressors X are normalized
 
-    copy_X : boolean, optional, default True
-        If True, X will be copied; else, it may be overwritten.
+    solver : {'auto', 'dense_cholesky', 'lsqr', 'sparse_cg'}
+        Solver to use in the computational routines:
 
-    tol: float
+        - 'auto' chooses the solver automatically based on the type of data.
+
+        - 'dense_cholesky' uses the standard scipy.linalg.solve function to
+          obtain a closed-form solution.
+
+        - 'sparse_cg' uses the conjugate gradient solver as found in
+          scipy.sparse.linalg.cg. As an iterative algorithm, this solver is
+          more appropriate than 'dense_cholesky' for large-scale data
+          (possibility to set `tol` and `max_iter`).
+
+        - 'lsqr' uses the dedicated regularized least-squares routine
+          scipy.sparse.linalg.lsqr. It is the fatest but may not be available
+          in old scipy versions. It also uses an iterative procedure.
+
+        All three solvers support both dense and sparse data.
+
+    tol : float
         Precision of the solution.
 
     Attributes
     ----------
-
-    `coef_` : array, shape = [n_features] or [n_responses, n_features]
+    `coef_` : array, shape = [n_features] or [n_targets, n_features]
         Weight vector(s).
 
     See also
@@ -174,19 +296,16 @@ class Ridge(LinearModel):
     >>> X = np.random.randn(n_samples, n_features)
     >>> clf = Ridge(alpha=1.0)
     >>> clf.fit(X, y) # doctest: +NORMALIZE_WHITESPACE
-    Ridge(alpha=1.0, copy_X=True, fit_intercept=True, normalize=False,
-       tol=0.001)
+    Ridge(alpha=1.0, copy_X=True, fit_intercept=True, max_iter=None,
+          normalize=False, solver='auto', tol=0.001)
     """
-
     def __init__(self, alpha=1.0, fit_intercept=True, normalize=False,
-            copy_X=True, tol=1e-3):
-        self.alpha = alpha
-        self.fit_intercept = fit_intercept
-        self.normalize = normalize
-        self.copy_X = copy_X
-        self.tol = tol
+                 copy_X=True, max_iter=None, tol=1e-3, solver="auto"):
+        super(Ridge, self).__init__(alpha=alpha, fit_intercept=fit_intercept,
+                                    normalize=normalize, copy_X=copy_X,
+                                    max_iter=max_iter, tol=tol, solver=solver)
 
-    def fit(self, X, y, sample_weight=1.0, solver='auto'):
+    def fit(self, X, y, sample_weight=1.0, solver=None):
         """Fit Ridge regression model
 
         Parameters
@@ -194,71 +313,73 @@ class Ridge(LinearModel):
         X : {array-like, sparse matrix}, shape = [n_samples, n_features]
             Training data
 
-        y : array-like, shape = [n_samples] or [n_samples, n_responses]
+        y : array-like, shape = [n_samples] or [n_samples, n_targets]
             Target values
 
         sample_weight : float or numpy array of shape [n_samples]
             Individual weights for each sample
 
-        solver : {'auto', 'dense_cholesky', 'sparse_cg'}
-            Solver to use in the computational
-            routines. 'delse_cholesky' will use the standard
-            scipy.linalg.solve function, 'sparse_cg' will use the
-            conjugate gradient solver as found in
-            scipy.sparse.linalg.cg while 'auto' will chose the most
-            appropriate depending on the matrix X.
-
         Returns
         -------
         self : returns an instance of self.
         """
-        X = safe_asarray(X, dtype=np.float)
-        y = np.asarray(y, dtype=np.float)
+        if solver is None:
+            solver = self.solver
+        else:
+            # The fit method should be removed from Ridge when this warning is
+            # removed
+            warnings.warn("""solver option in fit is deprecated and will be
+                          removed in v0.14.""")
 
-        X, y, X_mean, y_mean, X_std = \
-           self._center_data(X, y, self.fit_intercept,
-                   self.normalize, self.copy_X)
-
-        self.coef_ = ridge_regression(X, y, self.alpha, sample_weight,
-                                      solver, self.tol)
-        self._set_intercept(X_mean, y_mean, X_std)
-        return self
+        return _BaseRidge.fit(self, X, y, solver=solver,
+                              sample_weight=sample_weight)
 
 
-class RidgeClassifier(Ridge):
+class RidgeClassifier(LinearClassifierMixin, _BaseRidge):
     """Classifier using Ridge regression.
 
     Parameters
     ----------
     alpha : float
-        Small positive values of alpha improve the conditioning of the
-        problem and reduce the variance of the estimates.
-        Alpha corresponds to (2*C)^-1 in other linear models such as
-        LogisticRegression or LinearSVC.
-
-    fit_intercept : boolean
-        Whether to calculate the intercept for this model. If set
-        to false, no intercept will be used in calculations
-        (e.g. data is expected to be already centered).
-
-    normalize : boolean, optional
-        If True, the regressors X are normalized
-
-    copy_X : boolean, optional, default True
-        If True, X will be copied; else, it may be overwritten.
-
-    tol: float
-        Precision of the solution.
+        Small positive values of alpha improve the conditioning of the problem
+        and reduce the variance of the estimates.  Alpha corresponds to
+        ``(2*C)^-1`` in other linear models such as LogisticRegression or
+        LinearSVC.
 
     class_weight : dict, optional
         Weights associated with classes in the form
         {class_label : weight}. If not given, all classes are
         supposed to have weight one.
 
+    copy_X : boolean, optional, default True
+        If True, X will be copied; else, it may be overwritten.
+
+    fit_intercept : boolean
+        Whether to calculate the intercept for this model. If set to false, no
+        intercept will be used in calculations (e.g. data is expected to be
+        already centered).
+
+    max_iter : int, optional
+        Maximum number of iterations for conjugate gradient solver.
+        The default value is determined by scipy.sparse.linalg.
+
+    normalize : boolean, optional
+        If True, the regressors X are normalized
+
+    solver : {'auto', 'dense_cholesky', 'lsqr', 'sparse_cg'}
+        Solver to use in the computational
+        routines. 'dense_cholesky' will use the standard
+        scipy.linalg.solve function, 'sparse_cg' will use the
+        conjugate gradient solver as found in
+        scipy.sparse.linalg.cg while 'auto' will chose the most
+        appropriate depending on the matrix X. 'lsqr' uses
+        a direct regularized least-squares routine provided by scipy.
+
+    tol : float
+        Precision of the solution.
 
     Attributes
     ----------
-
     `coef_` : array, shape = [n_features] or [n_classes, n_features]
         Weight vector(s).
 
@@ -273,13 +394,14 @@ class RidgeClassifier(Ridge):
     advantage of the multi-variate response support in Ridge.
     """
     def __init__(self, alpha=1.0, fit_intercept=True, normalize=False,
-            copy_X=True, tol=1e-3, class_weight=None):
-        super(RidgeClassifier, self).__init__(alpha=alpha,
-                fit_intercept=fit_intercept, normalize=normalize,
-                copy_X=copy_X, tol=tol)
+                 copy_X=True, max_iter=None, tol=1e-3, class_weight=None,
+                 solver="auto"):
+        super(RidgeClassifier, self).__init__(
+            alpha=alpha, fit_intercept=fit_intercept, normalize=normalize,
+            copy_X=copy_X, max_iter=max_iter, tol=tol, solver=solver)
         self.class_weight = class_weight
 
-    def fit(self, X, y, solver='auto'):
+    def fit(self, X, y, solver=None):
         """Fit Ridge regression model.
 
         Parameters
@@ -290,14 +412,6 @@ class RidgeClassifier(Ridge):
         y : array-like, shape = [n_samples]
             Target values
 
-        solver : {'auto', 'dense_cholesky', 'sparse_cg'}
-            Solver to use in the computational
-            routines. 'delse_cholesky' will use the standard
-            scipy.linalg.solve function, 'sparse_cg' will use the
-            conjugate gradient solver as found in
-            scipy.sparse.linalg.cg while 'auto' will chose the most
-            appropriate depending on the matrix X.
-
         Returns
         -------
         self : returns an instance of self.
@@ -307,28 +421,22 @@ class RidgeClassifier(Ridge):
         else:
             class_weight = self.class_weight
 
+        if solver is None:
+            solver = self.solver
+        else:
+            warnings.warn("""solver option in fit is deprecated and will be
+                          removed in v0.14.""")
+
         sample_weight_classes = np.array([class_weight.get(k, 1.0) for k in y])
-        self.label_binarizer = LabelBinarizer()
-        Y = self.label_binarizer.fit_transform(y)
-        Ridge.fit(self, X, Y, solver=solver, sample_weight=sample_weight_classes)
+        self._label_binarizer = LabelBinarizer(pos_label=1, neg_label=-1)
+        Y = self._label_binarizer.fit_transform(y)
+        _BaseRidge.fit(self, X, Y, solver=solver,
+                       sample_weight=sample_weight_classes)
         return self
 
-    def decision_function(self, X):
-        return Ridge.decision_function(self, X)
-
-    def predict(self, X):
-        """Predict target values according to the fitted model.
-
-        Parameters
-        ----------
-        X : array-like, shape = [n_samples, n_features]
-
-        Returns
-        -------
-        y : array, shape = [n_samples]
-        """
-        Y = self.decision_function(X)
-        return self.label_binarizer.inverse_transform(Y)
+    @property
+    def classes_(self):
+        return self._label_binarizer.classes_
 
 
 class _RidgeGCV(LinearModel):
@@ -371,8 +479,8 @@ class _RidgeGCV(LinearModel):
     """
 
     def __init__(self, alphas=[0.1, 1.0, 10.0], fit_intercept=True,
-            normalize=False, score_func=None, loss_func=None, copy_X=True,
-            gcv_mode=None):
+                 normalize=False, score_func=None, loss_func=None,
+                 copy_X=True, gcv_mode=None, store_cv_values=False):
         self.alphas = np.asarray(alphas)
         self.fit_intercept = fit_intercept
         self.normalize = normalize
@@ -380,11 +488,11 @@ class _RidgeGCV(LinearModel):
         self.loss_func = loss_func
         self.copy_X = copy_X
         self.gcv_mode = gcv_mode
+        self.store_cv_values = store_cv_values
 
     def _pre_compute(self, X, y):
         # even if X is very sparse, K is usually very dense
         K = safe_sparse_dot(X, X.T, dense_output=True)
-        from scipy import linalg
         v, Q = linalg.eigh(K)
         QT_y = np.dot(Q.T, y)
         return v, Q, QT_y
@@ -421,7 +529,6 @@ class _RidgeGCV(LinearModel):
         return y - (c / G_diag), c
 
     def _pre_compute_svd(self, X, y):
-        from scipy import sparse
         if sparse.issparse(X) and hasattr(X, 'toarray'):
             X = X.toarray()
         U, s, _ = np.linalg.svd(X, full_matrices=0)
@@ -455,7 +562,7 @@ class _RidgeGCV(LinearModel):
         X : {array-like, sparse matrix}, shape = [n_samples, n_features]
             Training data
 
-        y : array-like, shape = [n_samples] or [n_samples, n_responses]
+        y : array-like, shape = [n_samples] or [n_samples, n_targets]
             Target values
 
         sample_weight : float or array-like of shape [n_samples]
@@ -470,8 +577,9 @@ class _RidgeGCV(LinearModel):
 
         n_samples, n_features = X.shape
 
-        X, y, X_mean, y_mean, X_std = LinearModel._center_data(X, y,
-                self.fit_intercept, self.normalize, self.copy_X)
+        X, y, X_mean, y_mean, X_std = LinearModel._center_data(
+            X, y, self.fit_intercept, self.normalize, self.copy_X,
+            sample_weight=sample_weight)
 
         gcv_mode = self.gcv_mode
         with_sw = len(np.shape(sample_weight))
@@ -484,7 +592,7 @@ class _RidgeGCV(LinearModel):
         elif gcv_mode == "svd" and with_sw:
             # FIXME non-uniform sample weights not yet supported
             warnings.warn("non-uniform sample weights unsupported for svd, "
-                "forcing usage of eigen")
+                          "forcing usage of eigen")
             gcv_mode = 'eigen'
 
         if gcv_mode == 'eigen':
@@ -501,7 +609,7 @@ class _RidgeGCV(LinearModel):
 
         v, Q, QT_y = _pre_compute(X, y)
         n_y = 1 if len(y.shape) == 1 else y.shape[1]
-        M = np.zeros((n_samples * n_y, len(self.alphas)))
+        cv_values = np.zeros((n_samples * n_y, len(self.alphas)))
         C = []
 
         error = self.score_func is None and self.loss_func is None
@@ -511,26 +619,108 @@ class _RidgeGCV(LinearModel):
                 out, c = _errors(sample_weight * alpha, y, v, Q, QT_y)
             else:
                 out, c = _values(sample_weight * alpha, y, v, Q, QT_y)
-            M[:, i] = out.ravel()
+            cv_values[:, i] = out.ravel()
             C.append(c)
 
         if error:
-            best = M.mean(axis=0).argmin()
+            best = cv_values.mean(axis=0).argmin()
         else:
             func = self.score_func if self.score_func else self.loss_func
-            out = [func(y.ravel(), M[:, i]) for i in range(len(self.alphas))]
+            out = [func(y.ravel(), cv_values[:, i])
+                   for i in range(len(self.alphas))]
             best = np.argmax(out) if self.score_func else np.argmin(out)
 
-        self.best_alpha = self.alphas[best]
+        self.alpha_ = self.alphas[best]
         self.dual_coef_ = C[best]
         self.coef_ = safe_sparse_dot(self.dual_coef_.T, X)
 
         self._set_intercept(X_mean, y_mean, X_std)
 
+        if self.store_cv_values:
+            if len(y.shape) == 1:
+                cv_values_shape = n_samples, len(self.alphas)
+            else:
+                cv_values_shape = n_samples, n_y, len(self.alphas)
+            self.cv_values_ = cv_values.reshape(cv_values_shape)
+
+        return self
+
+    @property
+    def best_alpha(self):
+        warnings.warn("Use alpha_. Using best_alpha is deprecated"
+                      "since version 0.12, and backward compatibility "
+                      "won't be maintained from version 0.14 onward. ",
+                      DeprecationWarning, stacklevel=2)
+        return self.alpha_
+
+
+class _BaseRidgeCV(LinearModel):
+
+    def __init__(self, alphas=np.array([0.1, 1.0, 10.0]),
+                 fit_intercept=True, normalize=False, score_func=None,
+                 loss_func=None, cv=None, gcv_mode=None,
+                 store_cv_values=False):
+        self.alphas = alphas
+        self.fit_intercept = fit_intercept
+        self.normalize = normalize
+        self.score_func = score_func
+        self.loss_func = loss_func
+        self.cv = cv
+        self.gcv_mode = gcv_mode
+        self.store_cv_values = store_cv_values
+
+    def fit(self, X, y, sample_weight=1.0):
+        """Fit Ridge regression model
+
+        Parameters
+        ----------
+        X : array-like, shape = [n_samples, n_features]
+            Training data
+
+        y : array-like, shape = [n_samples] or [n_samples, n_targets]
+            Target values
+
+        sample_weight : float or array-like of shape [n_samples]
+            Sample weight
+
+        Returns
+        -------
+        self : Returns self.
+        """
+        if self.cv is None:
+            estimator = _RidgeGCV(self.alphas,
+                                  fit_intercept=self.fit_intercept,
+                                  normalize=self.normalize,
+                                  score_func=self.score_func,
+                                  loss_func=self.loss_func,
+                                  gcv_mode=self.gcv_mode,
+                                  store_cv_values=self.store_cv_values)
+            estimator.fit(X, y, sample_weight=sample_weight)
+            self.alpha_ = estimator.alpha_
+            if self.store_cv_values:
+                self.cv_values_ = estimator.cv_values_
+        else:
+            if self.store_cv_values:
+                raise ValueError("cv!=None and store_cv_values=True "
+                                 " are incompatible")
+            parameters = {'alpha': self.alphas}
+            # FIXME: sample_weight must be split into training/validation data
+            #        too!
+            #fit_params = {'sample_weight' : sample_weight}
+            fit_params = {}
+            gs = GridSearchCV(Ridge(fit_intercept=self.fit_intercept),
+                              parameters, fit_params=fit_params, cv=self.cv)
+            gs.fit(X, y)
+            estimator = gs.best_estimator_
+            self.alpha_ = gs.best_estimator_.alpha
+
+        self.coef_ = estimator.coef_
+        self.intercept_ = estimator.intercept_
+
         return self
 
 
-class RidgeCV(LinearModel):
+class RidgeCV(_BaseRidgeCV, RegressorMixin):
     """Ridge regression with built-in cross-validation.
 
     By default, it performs Generalized Cross-Validation, which is a form of
@@ -538,7 +728,7 @@ class RidgeCV(LinearModel):
 
     Parameters
     ----------
-    alphas: numpy array of shape [n_alpha]
+    alphas: numpy array of shape [n_alphas]
         Array of alpha values to try.
         Small positive values of alpha improve the conditioning of the
         problem and reduce the variance of the estimates.
@@ -567,12 +757,6 @@ class RidgeCV(LinearModel):
         If None, Generalized Cross-Validation (efficient Leave-One-Out)
         will be used.
 
-
-    Attributes
-    ----------
-    `coef_` : array, shape = [n_features] or [n_classes, n_features]
-        Weight vector(s).
-
     gcv_mode : {None, 'auto', 'svd', eigen'}, optional
         Flag indicating which strategy to use when performing
         Generalized Cross-Validation. Options are::
@@ -584,66 +768,37 @@ class RidgeCV(LinearModel):
         The 'auto' mode is the default and is intended to pick the cheaper \
         option of the two depending upon the shape of the training data.
 
+    store_cv_values : boolean, default=False
+        Flag indicating if the cross-validation values corresponding to
+        each alpha should be stored in the `cv_values_` attribute (see
+        below). This flag is only compatible with `cv=None` (i.e. using
+        Generalized Cross-Validation).
+
+    Attributes
+    ----------
+    `cv_values_` : array, shape = [n_samples, n_alphas] or \
+        shape = [n_samples, n_targets, n_alphas], optional
+        Cross-validation values for each alpha (if `store_cv_values=True` and \
+        `cv=None`). After `fit()` has been called, this attribute will \
+        contain the mean squared errors (by default) or the values of the \
+        `{loss,score}_func` function (if provided in the constructor).
+
+    `coef_` : array, shape = [n_features] or [n_targets, n_features]
+        Weight vector(s).
+
+    `alpha_` : float
+        Estimated regularization parameter.
+
     See also
     --------
     Ridge: Ridge regression
     RidgeClassifier: Ridge classifier
-    RidgeCV: Ridge regression with built-in cross validation
+    RidgeClassifierCV: Ridge classifier with built-in cross validation
     """
-
-    def __init__(self, alphas=np.array([0.1, 1.0, 10.0]), fit_intercept=True,
-            normalize=False, score_func=None, loss_func=None, cv=None,
-            gcv_mode=None):
-        self.alphas = alphas
-        self.fit_intercept = fit_intercept
-        self.normalize = normalize
-        self.score_func = score_func
-        self.loss_func = loss_func
-        self.cv = cv
-        self.gcv_mode = gcv_mode
-
-    def fit(self, X, y, sample_weight=1.0):
-        """Fit Ridge regression model
-
-        Parameters
-        ----------
-        X : array-like, shape = [n_samples, n_features]
-            Training data
-
-        y : array-like, shape = [n_samples] or [n_samples, n_responses]
-            Target values
-
-        sample_weight : float or array-like of shape [n_samples]
-            Sample weight
-
-        Returns
-        -------
-        self : Returns self.
-        """
-        if self.cv is None:
-            estimator = _RidgeGCV(self.alphas, self.fit_intercept,
-                    self.score_func, self.loss_func, gcv_mode=self.gcv_mode)
-            estimator.fit(X, y, sample_weight=sample_weight)
-            self.best_alpha = estimator.best_alpha
-        else:
-            parameters = {'alpha': self.alphas}
-            # FIXME: sample_weight must be split into training/validation data
-            #        too!
-            #fit_params = {'sample_weight' : sample_weight}
-            fit_params = {}
-            gs = GridSearchCV(Ridge(fit_intercept=self.fit_intercept),
-                              parameters, fit_params=fit_params, cv=self.cv)
-            gs.fit(X, y)
-            estimator = gs.best_estimator_
-            self.best_alpha = gs.best_estimator_.alpha
-
-        self.coef_ = estimator.coef_
-        self.intercept_ = estimator.intercept_
-
-        return self
+    pass
 
 
-class RidgeClassifierCV(RidgeCV):
+class RidgeClassifierCV(LinearClassifierMixin, _BaseRidgeCV):
     """Ridge classifier with built-in cross-validation.
 
     By default, it performs Generalized Cross-Validation, which is a form of
@@ -652,7 +807,7 @@ class RidgeClassifierCV(RidgeCV):
 
     Parameters
     ----------
-    alphas: numpy array of shape [n_alpha]
+    alphas: numpy array of shape [n_alphas]
         Array of alpha values to try.
         Small positive values of alpha improve the conditioning of the
         problem and reduce the variance of the estimates.
@@ -686,6 +841,21 @@ class RidgeClassifierCV(RidgeCV):
         {class_label : weight}. If not given, all classes are
         supposed to have weight one.
 
+    Attributes
+    ----------
+    `cv_values_` : array, shape = [n_samples, n_alphas] or \
+    shape = [n_samples, n_responses, n_alphas], optional
+        Cross-validation values for each alpha (if `store_cv_values=True` and
+    `cv=None`). After `fit()` has been called, this attribute will contain \
+    the mean squared errors (by default) or the values of the \
+    `{loss,score}_func` function (if provided in the constructor).
+
+    `coef_` : array, shape = [n_features] or [n_targets, n_features]
+        Weight vector(s).
+
+    `alpha_` : float
+        Estimated regularization parameter
+
     See also
     --------
     Ridge: Ridge regression
@@ -699,11 +869,11 @@ class RidgeClassifierCV(RidgeCV):
     advantage of the multi-variate response support in Ridge.
     """
     def __init__(self, alphas=np.array([0.1, 1.0, 10.0]), fit_intercept=True,
-            normalize=False, score_func=None, loss_func=None, cv=None,
-            class_weight=None):
-        super(RidgeClassifierCV, self).__init__(alphas=alphas,
-                fit_intercept=fit_intercept, normalize=normalize,
-                score_func=score_func, loss_func=loss_func, cv=cv)
+                 normalize=False, score_func=None, loss_func=None, cv=None,
+                 class_weight=None):
+        super(RidgeClassifierCV, self).__init__(
+            alphas=alphas, fit_intercept=fit_intercept, normalize=normalize,
+            score_func=score_func, loss_func=loss_func, cv=cv)
         self.class_weight = class_weight
 
     def fit(self, X, y, sample_weight=1.0, class_weight=None):
@@ -721,46 +891,20 @@ class RidgeClassifierCV(RidgeCV):
         sample_weight : float or numpy array of shape [n_samples]
             Sample weight
 
-        class_weight : dict, optional
-            Weights associated with classes in the form
-            {class_label : weight}. If not given, all classes are
-            supposed to have weight one.
-
         Returns
         -------
         self : object
             Returns self.
         """
-        if class_weight != None:
-            warnings.warn("'class_weight' is now an initialization parameter."
-                    "Using it in the 'fit' method is deprecated.",
-                    DeprecationWarning)
-            self.class_weight_ = class_weight
-        else:
-            self.class_weight_ = self.class_weight
-
-        if self.class_weight_ is None:
-            self.class_weight_ = {}
-
-        sample_weight2 = np.array([self.class_weight_.get(k, 1.0) for k in y])
-        self.label_binarizer = LabelBinarizer()
-        Y = self.label_binarizer.fit_transform(y)
-        RidgeCV.fit(self, X, Y, sample_weight=sample_weight * sample_weight2)
+        if self.class_weight is not None:
+            get_cw = self.class_weight.get
+            sample_weight = (sample_weight
+                             * np.array([get_cw(k, 1.0) for k in y]))
+        self._label_binarizer = LabelBinarizer(pos_label=1, neg_label=-1)
+        Y = self._label_binarizer.fit_transform(y)
+        _BaseRidgeCV.fit(self, X, Y, sample_weight=sample_weight)
         return self
 
-    def decision_function(self, X):
-        return RidgeCV.decision_function(self, X)
-
-    def predict(self, X):
-        """Predict target values according to the fitted model.
-
-        Parameters
-        ----------
-        X : array-like, shape = [n_samples, n_features]
-
-        Returns
-        -------
-        y : array, shape = [n_samples]
-        """
-        Y = self.decision_function(X)
-        return self.label_binarizer.inverse_transform(Y)
+    @property
+    def classes_(self):
+        return self._label_binarizer.classes_
