@@ -5,11 +5,10 @@
 # License: BSD, (C) INRIA 2011
 
 import numpy as np
-import warnings
 from scipy.linalg import eigh, svd, qr, solve
 from scipy.sparse import eye, csr_matrix
-from ..base import BaseEstimator
-from ..utils import array2d, check_random_state
+from ..base import BaseEstimator, TransformerMixin
+from ..utils import array2d, check_random_state, check_arrays
 from ..utils.arpack import eigsh
 from ..neighbors import NearestNeighbors
 
@@ -124,6 +123,8 @@ def null_space(M, k, k_skip=1, eigen_solver='arpack', tol=1E-6, max_iter=100,
         arpack : use arnoldi iteration in shift-invert mode.
                     For this method, M may be a dense matrix, sparse matrix,
                     or general linear operator.
+                    Warning: ARPACK can be unstable for some problems.  It is
+                    best to try several random seeds in order to check results.
         dense  : use standard dense matrix operations for the eigenvalue
                     decomposition.  For this method, M must be an array
                     or matrix type.  This method should be avoided for
@@ -136,9 +137,10 @@ def null_space(M, k, k_skip=1, eigen_solver='arpack', tol=1E-6, max_iter=100,
     max_iter : maximum number of iterations for 'arpack' method
         not used if eigen_solver=='dense'
 
-    random_state: numpy.RandomState, optional
-        The generator used to initialize the centers. Defaults to numpy.random.
-        Used to determine the starting vector for arpack iterations
+    random_state: numpy.RandomState or int, optional
+        The generator or seed used to determine the starting vector for arpack
+        iterations.  Defaults to numpy.random.
+
     """
     if eigen_solver == 'auto':
         if M.shape[0] > 200 and k + k_skip < 10:
@@ -161,9 +163,6 @@ def null_space(M, k, k_skip=1, eigen_solver='arpack', tol=1E-6, max_iter=100,
                              "ill-behaved.  method='dense' is recommended. "
                              "See online documentation for more information."
                              % msg)
-        except:
-            #let other errors pass through
-            raise
 
         return eigen_vectors[:, k_skip:], np.sum(eigen_values[k_skip:])
     elif eigen_solver == 'dense':
@@ -178,10 +177,9 @@ def null_space(M, k, k_skip=1, eigen_solver='arpack', tol=1E-6, max_iter=100,
 
 
 def locally_linear_embedding(
-    X, n_neighbors, n_components, reg=1e-3, eigen_solver='auto',
-    tol=1e-6, max_iter=100, method='standard',
-    hessian_tol=1E-4, modified_tol=1E-12,
-    random_state=None, out_dim=None):
+        X, n_neighbors, n_components, reg=1e-3, eigen_solver='auto', tol=1e-6,
+        max_iter=100, method='standard', hessian_tol=1E-4, modified_tol=1E-12,
+        random_state=None):
     """Perform a Locally Linear Embedding analysis on the data.
 
     Parameters
@@ -207,6 +205,8 @@ def locally_linear_embedding(
         arpack : use arnoldi iteration in shift-invert mode.
                     For this method, M may be a dense matrix, sparse matrix,
                     or general linear operator.
+                    Warning: ARPACK can be unstable for some problems.  It is
+                    best to try several random seeds in order to check results.
 
         dense  : use standard dense matrix operations for the eigenvalue
                     decomposition.  For this method, M must be an array
@@ -239,8 +239,9 @@ def locally_linear_embedding(
         Tolerance for modified LLE method.
         Only used if method == 'modified'
 
-    random_state: numpy.RandomState, optional
-        The generator used to initialize the centers. Defaults to numpy.random.
+    random_state: numpy.RandomState or int, optional
+        The generator or seed used to determine the starting vector for arpack
+        iterations.  Defaults to numpy.random.
 
     Returns
     -------
@@ -272,12 +273,6 @@ def locally_linear_embedding(
     if method not in ('standard', 'hessian', 'modified', 'ltsa'):
         raise ValueError("unrecognized method '%s'" % method)
 
-    if out_dim:
-        warnings.warn("Parameter ``out_dim`` was renamed to ``n_components`` "
-                "and is now deprecated.", DeprecationWarning,
-                stacklevel=2)
-        n_components = out_dim
-
     nbrs = NearestNeighbors(n_neighbors=n_neighbors + 1)
     nbrs.fit(X)
     X = nbrs._fit_X
@@ -313,7 +308,8 @@ def locally_linear_embedding(
 
         if n_neighbors <= n_components + dp:
             raise ValueError("for method='hessian', n_neighbors must be "
-                    "greater than [n_components * (n_components + 3) / 2]")
+                             "greater than "
+                             "[n_components * (n_components + 3) / 2]")
 
         neighbors = nbrs.kneighbors(X, n_neighbors=n_neighbors + 1,
                                     return_distance=False)
@@ -341,8 +337,8 @@ def locally_linear_embedding(
 
             j = 1 + n_components
             for k in range(n_components):
-                Yi[:, j:j + n_components - k] = \
-                        U[:, k:k + 1] * U[:, k:n_components]
+                Yi[:, j:j + n_components - k] = (U[:, k:k + 1]
+                                                 * U[:, k:n_components])
                 j += n_components - k
 
             Q, R = qr(Yi)
@@ -362,7 +358,7 @@ def locally_linear_embedding(
     elif method == 'modified':
         if n_neighbors < n_components:
             raise ValueError("modified LLE requires "
-                "n_neighbors >= n_components")
+                             "n_neighbors >= n_components")
 
         neighbors = nbrs.kneighbors(X, n_neighbors=n_neighbors + 1,
                                     return_distance=False)
@@ -500,7 +496,7 @@ def locally_linear_embedding(
                       tol=tol, max_iter=max_iter, random_state=random_state)
 
 
-class LocallyLinearEmbedding(BaseEstimator):
+class LocallyLinearEmbedding(BaseEstimator, TransformerMixin):
     """Locally Linear Embedding
 
     Parameters
@@ -521,6 +517,8 @@ class LocallyLinearEmbedding(BaseEstimator):
         arpack : use arnoldi iteration in shift-invert mode.
                     For this method, M may be a dense matrix, sparse matrix,
                     or general linear operator.
+                    Warning: ARPACK can be unstable for some problems.  It is
+                    best to try several random seeds in order to check results.
 
         dense  : use standard dense matrix operations for the eigenvalue
                     decomposition.  For this method, M must be an array
@@ -535,7 +533,7 @@ class LocallyLinearEmbedding(BaseEstimator):
         maximum number of iterations for the arpack solver.
         Not used if eigen_solver=='dense'.
 
-    method : string ['standard' | 'hessian' | 'modified']
+    method : string ['standard' | 'hessian' | 'modified' |'ltsa']
         standard : use the standard locally linear embedding algorithm.
                    see reference [1]
         hessian  : use the Hessian eigenmap method.  This method requires
@@ -558,9 +556,9 @@ class LocallyLinearEmbedding(BaseEstimator):
         algorithm to use for nearest neighbors search,
         passed to neighbors.NearestNeighbors instance
 
-    random_state: numpy.RandomState, optional
-        The generator used to initialize the centers. Defaults to numpy.random.
-        Used to determine the starting vector for arpack iterations
+    random_state: numpy.RandomState or int, optional
+        The generator or seed used to determine the starting vector for arpack
+        iterations.  Defaults to numpy.random.
 
     Attributes
     ----------
@@ -591,15 +589,9 @@ class LocallyLinearEmbedding(BaseEstimator):
     """
 
     def __init__(self, n_neighbors=5, n_components=2, reg=1E-3,
-            eigen_solver='auto', tol=1E-6, max_iter=100, method='standard',
-            hessian_tol=1E-4, modified_tol=1E-12, neighbors_algorithm='auto',
-            random_state=None, out_dim=None):
-
-        if out_dim:
-            warnings.warn("Parameter ``out_dim`` was renamed to "
-                "``n_components`` and is now deprecated.", DeprecationWarning,
-                stacklevel=2)
-        self.out_dim = out_dim
+                 eigen_solver='auto', tol=1E-6, max_iter=100,
+                 method='standard', hessian_tol=1E-4, modified_tol=1E-12,
+                 neighbors_algorithm='auto', random_state=None):
 
         self.n_neighbors = n_neighbors
         self.n_components = n_components
@@ -611,18 +603,14 @@ class LocallyLinearEmbedding(BaseEstimator):
         self.hessian_tol = hessian_tol
         self.modified_tol = modified_tol
         self.random_state = random_state
-        self.nbrs_ = NearestNeighbors(n_neighbors,
-                                      algorithm=neighbors_algorithm)
+        self.neighbors_algorithm = neighbors_algorithm
 
     def _fit_transform(self, X):
-        if self.out_dim:
-            warnings.warn("Parameter ``out_dim`` was renamed to "
-                "``n_components`` and is now deprecated.", DeprecationWarning,
-                stacklevel=3)
-            self.n_components = self.out_dim
-            self.out_dim = None
+        self.nbrs_ = NearestNeighbors(self.n_neighbors,
+                                      algorithm=self.neighbors_algorithm)
 
         self.random_state = check_random_state(self.random_state)
+        X, = check_arrays(X, sparse_format='dense')
         self.nbrs_.fit(X)
         self.embedding_, self.reconstruction_error_ = \
             locally_linear_embedding(
