@@ -5,18 +5,22 @@ better.
 """
 
 # Authors: Olivier Grisel <olivier.grisel@ensta.org>
+#          Wei LI <kuantkid@gmail.com>
 # License: BSD Style.
 
 from math import log
-from scipy.misc import comb
-from scipy.special import gammaln
 
+from scipy.misc import comb
+from scipy.sparse import coo_matrix
 import numpy as np
 
+from ...utils.fixes import unique
+from .expected_mutual_info_fast import expected_mutual_information
+
 
-# the exact version if faster for k == 2: use it by default globally in
-# this module instead of the float approximate variant
 def comb2(n):
+    # the exact version is faster for k == 2: use it by default globally in
+    # this module instead of the float approximate variant
     return comb(n, 2, exact=1)
 
 
@@ -53,30 +57,30 @@ def contingency_matrix(labels_true, labels_pred, eps=None):
     eps: None or float
         If a float, that value is added to all values in the contingency
         matrix. This helps to stop NaN propogation.
-        If None, nothing is adjusted.
+        If ``None``, nothing is adjusted.
 
     Returns
     -------
     contingency: array, shape=[n_classes_true, n_classes_pred]
-        Matrix C such that C[i][j] is the number of samples in true class i and
-        in predicted class j. If eps is None, the dtype of this array will be
-        integer. If eps is given, the dtype will be float.
+        Matrix :math:`C` such that :math:`C_{i, j}` is the number of samples in
+        true class :math:`i` and in predicted class :math:`j`. If
+        ``eps is None``, the dtype of this array will be integer. If ``eps`` is
+        given, the dtype will be float.
     """
-    classes = np.unique(labels_true)
-    clusters = np.unique(labels_pred)
-    # The cluster and class ids are not necessarily consecutive integers
-    # starting at 0 hence build a map
-    class_idx = dict((k, v) for v, k in enumerate(classes))
-    cluster_idx = dict((k, v) for v, k in enumerate(clusters))
-    # Build the contingency table
+    classes, class_idx = unique(labels_true, return_inverse=True)
+    clusters, cluster_idx = unique(labels_pred, return_inverse=True)
     n_classes = classes.shape[0]
     n_clusters = clusters.shape[0]
-    contingency = np.zeros((n_classes, n_clusters), dtype=np.int)
-    for c, k in zip(labels_true, labels_pred):
-        contingency[class_idx[c], cluster_idx[k]] += 1
+    # Using coo_matrix to accelerate simple histogram calculation,
+    # i.e. bins are consecutive integers
+    # Currently, coo_matrix is faster than histogram2d for simple cases
+    contingency = coo_matrix((np.ones(class_idx.shape[0]),
+                              (class_idx, cluster_idx)),
+                             shape=(n_classes, n_clusters),
+                             dtype=np.int).toarray()
     if eps is not None:
-        # Must be a float matrix to accept float eps
-        contingency = np.array(contingency, dtype='float') + eps
+        # don't use += as contingency is integer
+        contingency = contingency + eps
     return contingency
 
 
@@ -168,7 +172,7 @@ def adjusted_rand_score(labels_true, labels_pred):
     # Special limit cases: no clustering since the data is not split.
     # This is a perfect match hence return 1.0.
     if (classes.shape[0] == clusters.shape[0] == 1
-        or classes.shape[0] == clusters.shape[0] == 0):
+            or classes.shape[0] == clusters.shape[0] == 0):
         return 1.0
 
     contingency = contingency_matrix(labels_true, labels_pred)
@@ -184,7 +188,7 @@ def adjusted_rand_score(labels_true, labels_pred):
 
 
 def homogeneity_completeness_v_measure(labels_true, labels_pred):
-    """Compute the homogeneity and completeness and V-measure scores at once
+    """Compute the homogeneity and completeness and V-Measure scores at once
 
     Those metrics are based on normalized conditional entropy measures of
     the clustering labeling to evaluate given the knowledge of a Ground
@@ -203,8 +207,8 @@ def homogeneity_completeness_v_measure(labels_true, labels_pred):
     a permutation of the class or cluster label values won't change the
     score values in any way.
 
-    V-Measure is furthermore symmetric: swapping `labels_true` and
-    `label_pred` will give the same score. This does not hold for
+    V-Measure is furthermore symmetric: swapping ``labels_true`` and
+    ``label_pred`` will give the same score. This does not hold for
     homogeneity and completeness.
 
     Parameters
@@ -233,37 +237,17 @@ def homogeneity_completeness_v_measure(labels_true, labels_pred):
     v_measure_score
     """
     labels_true, labels_pred = check_clusterings(labels_true, labels_pred)
-    n_samples = labels_true.shape[0]
 
-    entropy_K_given_C = 0.
-    entropy_C_given_K = 0.
-    entropy_C = 0.
-    entropy_K = 0.
+    if len(labels_true) == 0:
+        return 1.0, 1.0, 1.0
 
-    classes = np.unique(labels_true)
-    clusters = np.unique(labels_pred)
+    entropy_C = entropy(labels_true)
+    entropy_K = entropy(labels_pred)
 
-    n_C = [float(np.sum(labels_true == c)) for c in classes]
-    n_K = [float(np.sum(labels_pred == k)) for k in clusters]
+    MI = mutual_info_score(labels_true, labels_pred)
 
-    for i in xrange(len(classes)):
-        entropy_C -= n_C[i] / n_samples * log(n_C[i] / n_samples)
-
-    for j in xrange(len(clusters)):
-        entropy_K -= n_K[j] / n_samples * log(n_K[j] / n_samples)
-
-    for i, c in enumerate(classes):
-        for j, k in enumerate(clusters):
-            # count samples at the intersection of class c and cluster k
-            n_CK = float(np.sum((labels_true == c) * (labels_pred == k)))
-
-            if n_CK != 0.0:
-                # turn label assignments into contribution to entropies
-                entropy_C_given_K -= n_CK / n_samples * log(n_CK / n_K[j])
-                entropy_K_given_C -= n_CK / n_samples * log(n_CK / n_C[i])
-
-    homogeneity = 1.0 - entropy_C_given_K / entropy_C if entropy_C else 1.0
-    completeness = 1.0 - entropy_K_given_C / entropy_K if entropy_K else 1.0
+    homogeneity = MI / (entropy_C) if entropy_C else 1.0
+    completeness = MI / (entropy_K) if entropy_K else 1.0
 
     if homogeneity + completeness == 0.0:
         v_measure_score = 0.0
@@ -284,8 +268,9 @@ def homogeneity_score(labels_true, labels_pred):
     a permutation of the class or cluster label values won't change the
     score value in any way.
 
-    This metric is not symmetric: switching `label_true` with `label_pred`
-    will return the completeness_score which will be different in general.
+    This metric is not symmetric: switching ``label_true`` with ``label_pred``
+    will return the :func:`completeness_score` which will be different in
+    general.
 
     Parameters
     ----------
@@ -303,9 +288,9 @@ def homogeneity_score(labels_true, labels_pred):
     References
     ----------
 
-    .. [1] Andrew Rosenberg and Julia Hirschberg `V-Measure: A conditional
-        entropy-based external cluster evaluation measure`, 2007
-        http://acl.ldc.upenn.edu/D/D07/D07-1043.pdf
+    .. [1] `Andrew Rosenberg and Julia Hirschberg, 2007. V-Measure: A
+       conditional entropy-based external cluster evaluation measure
+       <http://acl.ldc.upenn.edu/D/D07/D07-1043.pdf>`_
 
     See also
     --------
@@ -324,18 +309,22 @@ def homogeneity_score(labels_true, labels_pred):
     Non-pefect labelings that futher split classes into more clusters can be
     perfectly homogeneous::
 
-      >>> homogeneity_score([0, 0, 1, 1], [0, 0, 1, 2])
-      1.0
-      >>> homogeneity_score([0, 0, 1, 1], [0, 1, 2, 3])
-      1.0
+      >>> print ("%.6f" % homogeneity_score([0, 0, 1, 1], [0, 0, 1, 2]))
+      ...                                                  # doctest: +ELLIPSIS
+      1.0...
+      >>> print ("%.6f" % homogeneity_score([0, 0, 1, 1], [0, 1, 2, 3]))
+      ...                                                  # doctest: +ELLIPSIS
+      1.0...
 
     Clusters that include samples from different classes do not make for an
     homogeneous labeling::
 
-      >>> homogeneity_score([0, 0, 1, 1], [0, 1, 0, 1])
-      0.0
-      >>> homogeneity_score([0, 0, 1, 1], [0, 0, 0, 0])
-      0.0
+      >>> print ("%.6f" % homogeneity_score([0, 0, 1, 1], [0, 1, 0, 1]))
+      ...                                                  # doctest: +ELLIPSIS
+      0.0...
+      >>> print ("%.6f" % homogeneity_score([0, 0, 1, 1], [0, 0, 0, 0]))
+      ...                                                  # doctest: +ELLIPSIS
+      0.0...
 
     """
     return homogeneity_completeness_v_measure(labels_true, labels_pred)[0]
@@ -351,8 +340,9 @@ def completeness_score(labels_true, labels_pred):
     a permutation of the class or cluster label values won't change the
     score value in any way.
 
-    This metric is not symmetric: switching `label_true` with `label_pred`
-    will return the homogeneity_score which will be different in general.
+    This metric is not symmetric: switching ``label_true`` with ``label_pred``
+    will return the :func:`homogeneity_score` which will be different in
+    general.
 
     Parameters
     ----------
@@ -370,9 +360,9 @@ def completeness_score(labels_true, labels_pred):
     References
     ----------
 
-    .. [1] Andrew Rosenberg and Julia Hirschberg `V-Measure: A conditional
-        entropy-based external cluster evaluation measure`, 2007
-        http://acl.ldc.upenn.edu/D/D07/D07-1043.pdf
+    .. [1] `Andrew Rosenberg and Julia Hirschberg, 2007. V-Measure: A
+       conditional entropy-based external cluster evaluation measure
+       <http://acl.ldc.upenn.edu/D/D07/D07-1043.pdf>`_
 
     See also
     --------
@@ -391,17 +381,17 @@ def completeness_score(labels_true, labels_pred):
     Non-pefect labelings that assign all classes members to the same clusters
     are still complete::
 
-      >>> completeness_score([0, 0, 1, 1], [0, 0, 0, 0])
+      >>> print completeness_score([0, 0, 1, 1], [0, 0, 0, 0])
       1.0
-      >>> completeness_score([0, 1, 2, 3], [0, 0, 1, 1])
+      >>> print completeness_score([0, 1, 2, 3], [0, 0, 1, 1])
       1.0
 
     If classes members are splitted across different clusters, the
     assignment cannot be complete::
 
-      >>> completeness_score([0, 0, 1, 1], [0, 1, 0, 1])
+      >>> print completeness_score([0, 0, 1, 1], [0, 1, 0, 1])
       0.0
-      >>> completeness_score([0, 0, 0, 0], [0, 1, 2, 3])
+      >>> print completeness_score([0, 0, 0, 0], [0, 1, 2, 3])
       0.0
 
     """
@@ -409,7 +399,9 @@ def completeness_score(labels_true, labels_pred):
 
 
 def v_measure_score(labels_true, labels_pred):
-    """V-Measure cluster labeling given a ground truth
+    """V-Measure cluster labeling given a ground truth.
+
+    This score is identical to :func:`normalized_mutual_info_score`.
 
     The V-Measure is the hormonic mean between homogeneity and completeness::
 
@@ -419,8 +411,8 @@ def v_measure_score(labels_true, labels_pred):
     a permutation of the class or cluster label values won't change the
     score value in any way.
 
-    This metric is furthermore symmetric: switching `label_true` with
-    `label_pred` will return the same score value. This can be useful to
+    This metric is furthermore symmetric: switching ``label_true`` with
+    ``label_pred`` will return the same score value. This can be useful to
     measure the agreement of two independent label assignments strategies
     on the same dataset when the real ground truth is not known.
 
@@ -440,9 +432,9 @@ def v_measure_score(labels_true, labels_pred):
     References
     ----------
 
-    .. [Rosenberg2007] `V-Measure: A conditional entropy-based external cluster
-        evaluation measure Andrew Rosenberg and Julia Hirschberg, 2007`
-        http://acl.ldc.upenn.edu/D/D07/D07-1043.pdf
+    .. [1] `Andrew Rosenberg and Julia Hirschberg, 2007. V-Measure: A
+       conditional entropy-based external cluster evaluation measure
+       <http://acl.ldc.upenn.edu/D/D07/D07-1043.pdf>`_
 
     See also
     --------
@@ -463,31 +455,37 @@ def v_measure_score(labels_true, labels_pred):
     Labelings that assign all classes members to the same clusters
     are complete be not homogeneous, hence penalized::
 
-      >>> v_measure_score([0, 0, 1, 2], [0, 0, 1, 1])     # doctest: +ELLIPSIS
+      >>> print("%.6f" % v_measure_score([0, 0, 1, 2], [0, 0, 1, 1]))
+      ...                                                  # doctest: +ELLIPSIS
       0.8...
-      >>> v_measure_score([0, 1, 2, 3], [0, 0, 1, 1])     # doctest: +ELLIPSIS
+      >>> print("%.6f" % v_measure_score([0, 1, 2, 3], [0, 0, 1, 1]))
+      ...                                                  # doctest: +ELLIPSIS
       0.66...
 
     Labelings that have pure clusters with members coming from the same
     classes are homogeneous but un-necessary splits harms completeness
-    and thus penalize V-measure as well::
+    and thus penalize V-Measure as well::
 
-      >>> v_measure_score([0, 0, 1, 1], [0, 0, 1, 2])     # doctest: +ELLIPSIS
+      >>> print("%.6f" % v_measure_score([0, 0, 1, 1], [0, 0, 1, 2]))
+      ...                                                  # doctest: +ELLIPSIS
       0.8...
-      >>> v_measure_score([0, 0, 1, 1], [0, 1, 2, 3])     # doctest: +ELLIPSIS
+      >>> print("%.6f" % v_measure_score([0, 0, 1, 1], [0, 1, 2, 3]))
+      ...                                                  # doctest: +ELLIPSIS
       0.66...
 
     If classes members are completly splitted across different clusters,
-    the assignment is totally in-complete, hence the v-measure is null::
+    the assignment is totally incomplete, hence the V-Measure is null::
 
-      >>> v_measure_score([0, 0, 0, 0], [0, 1, 2, 3])
-      0.0
+      >>> print("%.6f" % v_measure_score([0, 0, 0, 0], [0, 1, 2, 3]))
+      ...                                                  # doctest: +ELLIPSIS
+      0.0...
 
     Clusters that include samples from totally different classes totally
     destroy the homogeneity of the labeling, hence::
 
-      >>> v_measure_score([0, 0, 1, 1], [0, 0, 0, 0])
-      0.0
+      >>> print("%.6f" % v_measure_score([0, 0, 1, 1], [0, 0, 0, 0]))
+      ...                                                  # doctest: +ELLIPSIS
+      0.0...
 
     """
     return homogeneity_completeness_v_measure(labels_true, labels_pred)[2]
@@ -496,20 +494,25 @@ def v_measure_score(labels_true, labels_pred):
 def mutual_info_score(labels_true, labels_pred, contingency=None):
     """Mutual Information between two clusterings
 
-    The Mutual Information is a measure of the similarity between two labels
-    of the same data. Where P(i) is the probability of a random sample occuring
-    in cluster U_i and P'(j) is the probability of a random sample occuring in
-    cluster V_j, the Mutual information  between clusterings U and V is given
-    as::
+    The Mutual Information is a measure of the similarity between two labels of
+    the same data. Where :math:`P(i)` is the probability of a random sample
+    occuring in cluster :math:`U_i` and :math:`P'(j)` is the probability of a
+    random sample occuring in cluster :math:`V_j`, the Mutual Information
+    between clusterings :math:`U` and :math:`V` is given as:
 
-        MI(U,V)=\sum_{i=1}^R \sum_{j=1}^C P(i,j)\log \frac{P(i,j)}{P(i)P'(j)}
+    .. math::
+
+        MI(U,V)=\sum_{i=1}^R \sum_{j=1}^C P(i,j)\log\\frac{P(i,j)}{P(i)P'(j)}
+
+    This is equal to the Kullback-Leibler divergence of the joint distribution
+    with the product distribution of the marginals.
 
     This metric is independent of the absolute values of the labels:
     a permutation of the class or cluster label values won't change the
     score value in any way.
 
-    This metric is furthermore symmetric: switching `label_true` with
-    `label_pred` will return the same score value. This can be useful to
+    This metric is furthermore symmetric: switching ``label_true`` with
+    ``label_pred`` will return the same score value. This can be useful to
     measure the agreement of two independent label assignments strategies
     on the same dataset when the real ground truth is not known.
 
@@ -522,9 +525,9 @@ def mutual_info_score(labels_true, labels_pred, contingency=None):
         A clustering of the data into disjoint subsets.
 
     contingency: None or array, shape = [n_classes_true, n_classes_pred]
-        A contingency matrix given by the contingency_matrix function.
-        If value is None, it will be computed, otherwise the given value is
-        used, with labels_true and labels_pred ignored.
+        A contingency matrix given by the :func:`contingency_matrix` function.
+        If value is ``None``, it will be computed, otherwise the given value is
+        used, with ``labels_true`` and ``labels_pred`` ignored.
 
     Returns
     -------
@@ -533,20 +536,27 @@ def mutual_info_score(labels_true, labels_pred, contingency=None):
 
     See also
     --------
-    adjusted_mutual_info_score: Adjusted Mutual Information
+    adjusted_mutual_info_score: Adjusted against chance Mutual Information
+    normalized_mutual_info_score: Normalized Mutual Information
     """
     if contingency is None:
         labels_true, labels_pred = check_clusterings(labels_true, labels_pred)
         contingency = contingency_matrix(labels_true, labels_pred)
     contingency = np.array(contingency, dtype='float')
-    contingency /= np.sum(contingency)
+    contingency_sum = np.sum(contingency)
     pi = np.sum(contingency, axis=1)
-    pi /= np.sum(pi)
     pj = np.sum(contingency, axis=0)
-    pj /= np.sum(pj)
     outer = np.outer(pi, pj)
     nnz = contingency != 0.0
-    mi = contingency[nnz] * np.log(contingency[nnz] / outer[nnz])
+    # normalized contingency
+    contingency_nm = contingency[nnz]
+    log_contingency_nm = np.log(contingency_nm)
+    contingency_nm /= contingency_sum
+    # log(a / b) should be calculated as log(a) - log(b) for
+    # possible loss of precision
+    log_outer = -np.log(outer[nnz]) + log(pi.sum()) + log(pj.sum())
+    mi = (contingency_nm * (log_contingency_nm - log(contingency_sum))
+          + contingency_nm * log_outer)
     return mi.sum()
 
 
@@ -557,7 +567,7 @@ def adjusted_mutual_info_score(labels_true, labels_pred):
     Information (MI) score to account for chance. It accounts for the fact that
     the MI is generally higher for two clusterings with a larger number of
     clusters, regardless of whether there is actually more information shared.
-    For two clusterings U and V, the AMI is given as::
+    For two clusterings :math:`U` and :math:`V`, the AMI is given as::
 
         AMI(U, V) = [MI(U, V) - E(MI(U, V))] / [max(H(U), H(V)) - E(MI(U, V))]
 
@@ -565,8 +575,8 @@ def adjusted_mutual_info_score(labels_true, labels_pred):
     a permutation of the class or cluster label values won't change the
     score value in any way.
 
-    This metric is furthermore symmetric: switching `label_true` with
-    `label_pred` will return the same score value. This can be useful to
+    This metric is furthermore symmetric: switching ``label_true`` with
+    ``label_pred`` will return the same score value. This can be useful to
     measure the agreement of two independent label assignments strategies
     on the same dataset when the real ground truth is not known.
 
@@ -609,6 +619,16 @@ def adjusted_mutual_info_score(labels_true, labels_pred):
       >>> adjusted_mutual_info_score([0, 0, 0, 0], [0, 1, 2, 3])
       0.0
 
+    References
+    ----------
+    .. [1] `Vinh, Epps, and Bailey, (2010). Information Theoretic Measures for
+       Clusterings Comparison: Variants, Properties, Normalization and
+       Correction for Chance, JMLR
+       <http://jmlr.csail.mit.edu/papers/volume11/vinh10a/vinh10a.pdf>`_
+
+    .. [2] `Wikipedia entry for the Adjusted Mutual Information
+       <http://en.wikipedia.org/wiki/Adjusted_Mutual_Information>`_
+
     """
     labels_true, labels_pred = check_clusterings(labels_true, labels_pred)
     n_samples = labels_true.shape[0]
@@ -617,7 +637,7 @@ def adjusted_mutual_info_score(labels_true, labels_pred):
     # Special limit cases: no clustering since the data is not split.
     # This is a perfect match hence return 1.0.
     if (classes.shape[0] == clusters.shape[0] == 1
-        or classes.shape[0] == clusters.shape[0] == 0):
+            or classes.shape[0] == clusters.shape[0] == 0):
         return 1.0
     contingency = contingency_matrix(labels_true, labels_pred)
     contingency = np.array(contingency, dtype='float')
@@ -632,57 +652,92 @@ def adjusted_mutual_info_score(labels_true, labels_pred):
     return ami
 
 
-def expected_mutual_information(contingency, n_samples):
-    """Calculate the expected mutual information for two labelings."""
-    R, C = contingency.shape
-    N = float(n_samples)
-    a = np.sum(contingency, axis=1, dtype='int')
-    b = np.sum(contingency, axis=0, dtype='int')
-    # There are three major terms to the EMI equation, which are multiplied to
-    # and then summed over varying nij values.
-    # While nijs[0] will never be used, having it simplifies the indexing.
-    nijs = np.arange(0, max(np.max(a), np.max(b)) + 1, dtype='float')
-    nijs[0] = 1  # Stops divide by zero warnings. As its not used, no issue.
-    # term1 is nij / N
-    term1 = nijs / N
-    # term2 is log((N*nij) / (a * b)) == log(N * nij) - log(a * b)
-    # term2 uses the outer product
-    log_ab_outer = np.log(np.outer(a, b))
-    # term2 uses N * nij
-    log_Nnij = np.log(N * nijs)
-    # term3 is large, and involved many factorials. Calculate these in log
-    # space to stop overflows.
-    gln_a = gammaln(a + 1)
-    gln_b = gammaln(b + 1)
-    gln_Na = gammaln(N - a + 1)
-    gln_Nb = gammaln(N - b + 1)
-    gln_N = gammaln(N + 1)
-    gln_nij = gammaln(nijs + 1)
-    # start and end values for nij terms for each summation.
-    start = np.array([[v - N + w for w in b] for v in a], dtype='int')
-    start = np.maximum(start, 1)
-    end = np.minimum(np.resize(a, (C, R)).T, np.resize(b, (R, C))) + 1
-    # emi itself is a summation over the various values.
-    emi = 0
-    for i in range(R):
-        for j in range(C):
-            for nij in range(start[i][j], end[i][j]):
-                term2 = log_Nnij[nij] - log_ab_outer[i][j]
-                # Numerators are positive, denominators are negative.
-                gln = (gln_a[i] + gln_b[j] + gln_Na[i] + gln_Nb[j]
-                       - gln_N - gln_nij[nij] - gammaln(a[i] - nij + 1)
-                       - gammaln(b[j] - nij + 1)
-                       - gammaln(N - a[i] - b[j] + nij + 1))
-                term3 = np.exp(gln)
-                # Add the product of all terms.
-                emi += (term1[nij] * term2 * term3)
-    return emi
+def normalized_mutual_info_score(labels_true, labels_pred):
+    """Normalized Mutual Information between two clusterings
+
+    Normalized Mutual Information (NMI) is an normalization of the Mutual
+    Information (MI) score to scale the results between 0 (no mutual
+    information) and 1 (perfect correlation). In this function, mutual
+    information is normalized by ``sqrt(H(labels_true) * H(labels_pred))``
+
+    This measure is not adjusted for chance. Therefore
+    :func:`adjusted_mustual_info_score` might be preferred.
+
+    This metric is independent of the absolute values of the labels:
+    a permutation of the class or cluster label values won't change the
+    score value in any way.
+
+    This metric is furthermore symmetric: switching ``label_true`` with
+    ``label_pred`` will return the same score value. This can be useful to
+    measure the agreement of two independent label assignments strategies
+    on the same dataset when the real ground truth is not known.
+
+    Parameters
+    ----------
+    labels_true : int array, shape = [n_samples]
+        A clustering of the data into disjoint subsets.
+
+    labels_pred : array, shape = [n_samples]
+        A clustering of the data into disjoint subsets.
+
+    Returns
+    -------
+    nmi: float
+       score between 0.0 and 1.0. 1.0 stands for perfectly complete labeling
+
+    See also
+    --------
+    adjusted_rand_score: Adjusted Rand Index
+    adjusted_mutual_info_score: Adjusted Mutual Information (adjusted
+        against chance)
+
+    Examples
+    --------
+
+    Perfect labelings are both homogeneous and complete, hence have
+    score 1.0::
+
+      >>> from sklearn.metrics.cluster import normalized_mutual_info_score
+      >>> normalized_mutual_info_score([0, 0, 1, 1], [0, 0, 1, 1])
+      1.0
+      >>> normalized_mutual_info_score([0, 0, 1, 1], [1, 1, 0, 0])
+      1.0
+
+    If classes members are completly splitted across different clusters,
+    the assignment is totally in-complete, hence the NMI is null::
+
+      >>> normalized_mutual_info_score([0, 0, 0, 0], [0, 1, 2, 3])
+      0.0
+
+    """
+    labels_true, labels_pred = check_clusterings(labels_true, labels_pred)
+    classes = np.unique(labels_true)
+    clusters = np.unique(labels_pred)
+    # Special limit cases: no clustering since the data is not split.
+    # This is a perfect match hence return 1.0.
+    if (classes.shape[0] == clusters.shape[0] == 1
+            or classes.shape[0] == clusters.shape[0] == 0):
+        return 1.0
+    contingency = contingency_matrix(labels_true, labels_pred)
+    contingency = np.array(contingency, dtype='float')
+    # Calculate the MI for the two clusterings
+    mi = mutual_info_score(labels_true, labels_pred,
+                           contingency=contingency)
+    # Calculate the expected value for the mutual information
+    # Calculate entropy for each labeling
+    h_true, h_pred = entropy(labels_true), entropy(labels_pred)
+    nmi = mi / max(np.sqrt(h_true * h_pred), 1e-10)
+    return nmi
 
 
 def entropy(labels):
     """Calculates the entropy for a labeling."""
-    pi = np.array([np.sum(labels == i) for i in np.unique(labels)],
-                  dtype='float')
+    if len(labels) == 0:
+        return 1.0
+    label_idx = unique(labels, return_inverse=True)[1]
+    pi = np.bincount(label_idx).astype(np.float)
     pi = pi[pi > 0]
-    pi /= np.sum(pi)
-    return -np.sum(pi * np.log(pi))
+    pi_sum = np.sum(pi)
+    # log(a / b) should be calculated as log(a) - log(b) for
+    # possible loss of precision
+    return -np.sum((pi / pi_sum) * (np.log(pi) - log(pi_sum)))
