@@ -13,52 +13,78 @@ use these estimators to turn a binary classifier or a regressor into a
 multiclass classifier. It is also possible to use these estimators with
 multiclass estimators in the hope that their accuracy or runtime performance
 improves.
-"""
+
+The one-vs-the-rest meta-classifier also implements a `predic_proba` method, so
+long as such a method is implemented by the base classifier. This method
+returns probabilities of class membership in both the single label and
+multilabel case.  Note that in the multilabel case, probabilities are the
+marginal probability that a given sample falls in the given class. As such, in
+the multilabel case the sum of these probabilities over all possible labels
+for a given sample *will not* sum to unity, as they do in the single label
+case.  """
 
 # Author: Mathieu Blondel <mathieu@mblondel.org>
 #
 # License: BSD Style.
 
 import numpy as np
+import warnings
 
 from .base import BaseEstimator, ClassifierMixin, clone, is_classifier
+from .base import MetaEstimatorMixin
 from .preprocessing import LabelBinarizer
 from .metrics.pairwise import euclidean_distances
 from .utils import check_random_state
+from .externals.joblib import Parallel
+from .externals.joblib import delayed
 
 
-def _fit_binary(estimator, X, y):
+def _fit_binary(estimator, X, y, classes=None):
     """Fit a single binary estimator."""
-    estimator = clone(estimator)
-    estimator.fit(X, y)
+    unique_y = np.unique(y)
+    if len(unique_y) == 1:
+        if classes is not None:
+            if y[0] == -1:
+                c = 0
+            else:
+                c = y[0]
+            warnings.warn("Label %s is present in all training examples." %
+                          str(classes[c]))
+        estimator = _ConstantPredictor().fit(X, unique_y)
+    else:
+        estimator = clone(estimator)
+        estimator.fit(X, y)
     return estimator
 
 
 def _predict_binary(estimator, X):
     """Make predictions using a single binary estimator."""
-    if hasattr(estimator, "decision_function"):
-        return np.ravel(estimator.decision_function(X))
-    else:
+    try:
+        score = np.ravel(estimator.decision_function(X))
+    except (AttributeError, NotImplementedError):
         # probabilities of the positive class
-        return estimator.predict_proba(X)[:, 1]
+        score = estimator.predict_proba(X)[:, 1]
+    return score
 
 
 def _check_estimator(estimator):
     """Make sure that an estimator implements the necessary methods."""
-    if not hasattr(estimator, "decision_function") and \
-       not hasattr(estimator, "predict_proba"):
+    if (not hasattr(estimator, "decision_function") and
+            not hasattr(estimator, "predict_proba")):
         raise ValueError("The base estimator should implement "
                          "decision_function or predict_proba!")
 
 
-def fit_ovr(estimator, X, y):
+def fit_ovr(estimator, X, y, n_jobs=1):
     """Fit a one-vs-the-rest strategy."""
     _check_estimator(estimator)
 
     lb = LabelBinarizer()
     Y = lb.fit_transform(y)
-    estimators = [_fit_binary(estimator, X, Y[:, i])
-                  for i in range(Y.shape[1])]
+
+    estimators = Parallel(n_jobs=n_jobs)(
+        delayed(_fit_binary)(estimator, X, Y[:, i], classes=["not %s" % i, i])
+        for i in range(Y.shape[1]))
     return estimators, lb
 
 
@@ -70,7 +96,35 @@ def predict_ovr(estimators, label_binarizer, X):
     return label_binarizer.inverse_transform(Y.T, threshold=thresh)
 
 
-class OneVsRestClassifier(BaseEstimator, ClassifierMixin):
+def predict_proba_ovr(estimators, X, is_multilabel):
+    """Estimate probabilities using the one-vs-the-rest strategy.
+
+    If multilabel is true, returned matrix will not sum to one.  Estimators
+    must have a predict_proba method."""
+
+    # Y[i,j] gives the probability that sample i has the label j.
+    # In the multi-label case, these are not disjoint.
+    Y = np.array([est.predict_proba(X)[:, 1] for est in estimators]).T
+
+    if not is_multilabel:
+        # Then, probabilities should be normalized to 1.
+        Y /= np.sum(Y, axis=1)[:, np.newaxis]
+    return Y
+
+
+class _ConstantPredictor(BaseEstimator):
+    def fit(self, X, y):
+        self.y_ = y
+        return self
+
+    def predict(self, X):
+        return np.repeat(self.y_, X.shape[0])
+
+    def decision_function(self, X):
+        return np.repeat(self.y_, X.shape[0])
+
+
+class OneVsRestClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
     """One-vs-the-rest (OvR) multiclass/multilabel strategy
 
     Also known as one-vs-all, this strategy consists in fitting one classifier
@@ -88,17 +142,29 @@ class OneVsRestClassifier(BaseEstimator, ClassifierMixin):
     target vector. For multilabel learning, the number of classes must be at
     least three, since otherwise OvR reduces to binary classification.
 
+    In the multilabel learning literature, OvR is also known as the binary
+    relevance method.
+
     Parameters
     ----------
     estimator : estimator object
         An estimator object implementing `fit` and one of `decision_function`
         or `predict_proba`.
 
+    n_jobs : int, optional, default: 1
+
+        The number of jobs to use for the computation. If -1 all CPUs are used.
+        If 1 is given, no parallel computing code is used at all, which is
+        useful for debugging. For n_jobs below -1, (n_cpus + 1 + n_jobs) are
+        used. Thus for n_jobs = -2, all CPUs but one are used.
+
     Attributes
     ----------
     `estimators_` : list of `n_classes` estimators
         Estimators used for predictions.
 
+    `classes_` : array, shape = [`n_classes`]
+        Class labels.
     `label_binarizer_` : LabelBinarizer object
         Object used to transform multiclass labels to binary labels and
         vice-versa.
@@ -106,8 +172,9 @@ class OneVsRestClassifier(BaseEstimator, ClassifierMixin):
         Whether a OneVsRestClassifier is a multilabel classifier.
     """
 
-    def __init__(self, estimator):
+    def __init__(self, estimator, n_jobs=1):
         self.estimator = estimator
+        self.n_jobs = n_jobs
 
     def fit(self, X, y):
         """Fit underlying estimators.
@@ -126,7 +193,8 @@ class OneVsRestClassifier(BaseEstimator, ClassifierMixin):
         -------
         self
         """
-        self.estimators_, self.label_binarizer_ = fit_ovr(self.estimator, X, y)
+        self.estimators_, self.label_binarizer_ = fit_ovr(self.estimator, X, y,
+                                                          n_jobs=self.n_jobs)
         return self
 
     def _check_is_fitted(self):
@@ -150,6 +218,32 @@ class OneVsRestClassifier(BaseEstimator, ClassifierMixin):
 
         return predict_ovr(self.estimators_, self.label_binarizer_, X)
 
+    def predict_proba(self, X):
+        """Probability estimates.
+
+        The returned estimates for all classes are ordered by label of classes.
+
+        Note that in the multilabel case, each sample can have any number of
+        labels. This returns the marginal probability that the given sample has
+        the label in question. For example, it is entirely consistent that two
+        labels both have a 90% probability of applying to a given sample.
+
+        In the single label multiclass case, the rows of the returned matrix
+        sum to 1.
+
+        Parameters
+        ----------
+        X : array-like, shape = [n_samples, n_features]
+
+        Returns
+        -------
+        T : array-like, shape = [n_samples, n_classes]
+            Returns the probability of the sample for each class in the model,
+            where classes are ordered as they are in `self.classes_`.
+        """
+        return predict_proba_ovr(self.estimators_, X,
+                                 is_multilabel=self.multilabel_)
+
     @property
     def multilabel_(self):
         """Whether this is a multilabel classifier"""
@@ -162,6 +256,10 @@ class OneVsRestClassifier(BaseEstimator, ClassifierMixin):
         else:
             return super(OneVsRestClassifier, self).score(X, y)
 
+    @property
+    def classes_(self):
+        return self.label_binarizer_.classes_
+
     @property
     def coef_(self):
         self._check_is_fitted()
@@ -186,15 +284,17 @@ def _fit_ovo_binary(estimator, X, y, i, j):
     y[y == i] = 0
     y[y == j] = 1
     ind = np.arange(X.shape[0])
-    return _fit_binary(estimator, X[ind[cond]], y)
+    return _fit_binary(estimator, X[ind[cond]], y, classes=[i, j])
 
 
-def fit_ovo(estimator, X, y):
+def fit_ovo(estimator, X, y, n_jobs=1):
     """Fit a one-vs-one strategy."""
     classes = np.unique(y)
     n_classes = classes.shape[0]
-    estimators = [_fit_ovo_binary(estimator, X, y, classes[i], classes[j])
-                    for i in range(n_classes) for j in range(i + 1, n_classes)]
+    estimators = Parallel(n_jobs=n_jobs)(
+        delayed(_fit_ovo_binary)(
+            estimator, X, y, classes[i], classes[j])
+        for i in range(n_classes) for j in range(i + 1, n_classes))
 
     return estimators, classes
 
@@ -216,7 +316,7 @@ def predict_ovo(estimators, classes, X):
     return classes[votes.argmax(axis=1)]
 
 
-class OneVsOneClassifier(BaseEstimator, ClassifierMixin):
+class OneVsOneClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
     """One-vs-one multiclass strategy
 
     This strategy consists in fitting one classifier per class pair.
@@ -234,6 +334,13 @@ class OneVsOneClassifier(BaseEstimator, ClassifierMixin):
     estimator : estimator object
         An estimator object implementing `fit` and `predict`.
 
+    n_jobs : int, optional, default: 1
+
+        The number of jobs to use for the computation. If -1 all CPUs are used.
+        If 1 is given, no parallel computing code is used at all, which is
+        useful for debugging. For n_jobs below -1, (n_cpus + 1 + n_jobs) are
+        used. Thus for n_jobs = -2, all CPUs but one are used.
+
     Attributes
     ----------
     `estimators_` : list of `n_classes * (n_classes - 1) / 2` estimators
@@ -243,8 +350,9 @@ class OneVsOneClassifier(BaseEstimator, ClassifierMixin):
         Array containing labels.
     """
 
-    def __init__(self, estimator):
+    def __init__(self, estimator, n_jobs=1):
         self.estimator = estimator
+        self.n_jobs = n_jobs
 
     def fit(self, X, y):
         """Fit underlying estimators.
@@ -261,7 +369,8 @@ class OneVsOneClassifier(BaseEstimator, ClassifierMixin):
         -------
         self
         """
-        self.estimators_, self.classes_ = fit_ovo(self.estimator, X, y)
+        self.estimators_, self.classes_ = fit_ovo(self.estimator, X, y,
+                                                  self.n_jobs)
         return self
 
     def predict(self, X):
@@ -283,7 +392,7 @@ class OneVsOneClassifier(BaseEstimator, ClassifierMixin):
         return predict_ovo(self.estimators_, self.classes_, X)
 
 
-def fit_ecoc(estimator, X, y, code_size=1.5, random_state=None):
+def fit_ecoc(estimator, X, y, code_size=1.5, random_state=None, n_jobs=1):
     """
     Fit an error-correcting output-code strategy.
 
@@ -331,10 +440,12 @@ def fit_ecoc(estimator, X, y, code_size=1.5, random_state=None):
 
     cls_idx = dict((c, i) for i, c in enumerate(classes))
 
-    Y = np.array([code_book[cls_idx[y[i]]] for i in xrange(X.shape[0])])
+    Y = np.array([code_book[cls_idx[y[i]]] for i in xrange(X.shape[0])],
+                 dtype=np.int)
 
-    estimators = [_fit_binary(estimator, X, Y[:, i])
-                  for i in range(Y.shape[1])]
+    estimators = Parallel(n_jobs=n_jobs)(
+        delayed(_fit_binary)(estimator, X, Y[:, i])
+        for i in range(Y.shape[1]))
 
     return estimators, classes, code_book
 
@@ -346,7 +457,7 @@ def predict_ecoc(estimators, classes, code_book, X):
     return classes[pred]
 
 
-class OutputCodeClassifier(BaseEstimator, ClassifierMixin):
+class OutputCodeClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
     """(Error-Correcting) Output-Code multiclass strategy
 
     Output-code based strategies consist in representing each class with a
@@ -374,6 +485,13 @@ class OutputCodeClassifier(BaseEstimator, ClassifierMixin):
         The generator used to initialize the codebook. Defaults to
         numpy.random.
 
+    n_jobs : int, optional, default: 1
+
+        The number of jobs to use for the computation. If -1 all CPUs are used.
+        If 1 is given, no parallel computing code is used at all, which is
+        useful for debugging. For n_jobs below -1, (n_cpus + 1 + n_jobs) are
+        used. Thus for n_jobs = -2, all CPUs but one are used.
+
     Attributes
     ----------
     `estimators_` : list of `int(n_classes * code_size)` estimators
@@ -404,13 +522,14 @@ class OutputCodeClassifier(BaseEstimator, ClassifierMixin):
        2008.
     """
 
-    def __init__(self, estimator, code_size=1.5, random_state=None):
+    def __init__(self, estimator, code_size=1.5, random_state=None, n_jobs=1):
         if (code_size <= 0):
             raise ValueError("code_size should be greater than 0!")
 
         self.estimator = estimator
         self.code_size = code_size
         self.random_state = random_state
+        self.n_jobs = n_jobs
 
     def fit(self, X, y):
         """Fit underlying estimators.
@@ -428,7 +547,8 @@ class OutputCodeClassifier(BaseEstimator, ClassifierMixin):
         self
         """
         self.estimators_, self.classes_, self.code_book_ = \
-            fit_ecoc(self.estimator, X, y, self.code_size, self.random_state)
+            fit_ecoc(self.estimator, X, y, self.code_size, self.random_state,
+                     self.n_jobs)
         return self
 
     def predict(self, X):
