@@ -531,6 +531,7 @@ double Kernel::k_function(const PREFIX(node) *x, const PREFIX(node) *y,
 //
 // solution will be put in \alpha, objective value will be put in obj
 //
+
 class Solver {
 public:
 	Solver() {};
@@ -541,11 +542,12 @@ public:
 		double rho;
                 double *upper_bound;
 		double r;	// for Solver_NU
+                bool solve_timed_out;
 	};
 
 	void Solve(int l, const QMatrix& Q, const double *p_, const schar *y_,
 		   double *alpha_, const double *C_, double eps,
-		   SolutionInfo* si, int shrinking);
+		   SolutionInfo* si, int shrinking, int max_iter);
 protected:
 	int active_size;
 	schar *y;
@@ -645,7 +647,7 @@ void Solver::reconstruct_gradient()
 
 void Solver::Solve(int l, const QMatrix& Q, const double *p_, const schar *y_,
 		   double *alpha_, const double *C_, double eps,
-		   SolutionInfo* si, int shrinking)
+		   SolutionInfo* si, int shrinking, int max_iter)
 {
 	this->l = l;
 	this->Q = &Q;
@@ -656,6 +658,7 @@ void Solver::Solve(int l, const QMatrix& Q, const double *p_, const schar *y_,
         clone(C, C_, l);
 	this->eps = eps;
 	unshrink = false;
+        si->solve_timed_out = false;
 
 	// initialize alpha_status
 	{
@@ -703,6 +706,13 @@ void Solver::Solve(int l, const QMatrix& Q, const double *p_, const schar *y_,
 
 	while(1)
 	{
+                // set max_iter to -1 to disable the mechanism
+                if ((max_iter != -1) && (iter >= max_iter)) {
+                    info("WARN: libsvm Solver reached max_iter");
+                    si->solve_timed_out = true;
+                    break;
+                }
+
 		// show progress and do shrinking
 
 		if(--counter == 0)
@@ -1141,10 +1151,10 @@ public:
 	Solver_NU() {}
 	void Solve(int l, const QMatrix& Q, const double *p, const schar *y,
 		   double *alpha, const double *C_, double eps,
-		   SolutionInfo* si, int shrinking)
+		   SolutionInfo* si, int shrinking, int max_iter)
 	{
 		this->si = si;
-		Solver::Solve(l,Q,p,y,alpha,C_,eps,si,shrinking);
+		Solver::Solve(l,Q,p,y,alpha,C_,eps,si,shrinking,max_iter);
 	}
 private:
 	SolutionInfo *si;
@@ -1595,7 +1605,8 @@ static void solve_c_svc(
 
 	Solver s;
 	s.Solve(l, SVC_Q(*prob,*param,y), minus_ones, y,
-		alpha, C, param->eps, si, param->shrinking);
+		alpha, C, param->eps, si, param->shrinking,
+                param->max_iter);
 
         /*
 	double sum_alpha=0;
@@ -1659,7 +1670,7 @@ static void solve_nu_svc(
 
 	Solver_NU s;
 	s.Solve(l, SVC_Q(*prob,*param,y), zeros, y,
-		alpha, C, param->eps, si,  param->shrinking);
+		alpha, C, param->eps, si,  param->shrinking, param->max_iter);
 	double r = si->r;
 
 	info("C = %f\n",1/r);
@@ -1714,7 +1725,7 @@ static void solve_one_class(
 
 	Solver s;
 	s.Solve(l, ONE_CLASS_Q(*prob,*param), zeros, ones,
-		alpha, C, param->eps, si, param->shrinking);
+		alpha, C, param->eps, si, param->shrinking, param->max_iter);
 
         delete[] C;
 	delete[] zeros;
@@ -1747,7 +1758,7 @@ static void solve_epsilon_svr(
 
 	Solver s;
 	s.Solve(2*l, SVR_Q(*prob,*param), linear_term, y,
-		alpha2, C, param->eps, si, param->shrinking);
+		alpha2, C, param->eps, si, param->shrinking, param->max_iter);
 
 	double sum_alpha = 0;
 	for(i=0;i<l;i++)
@@ -1796,7 +1807,7 @@ static void solve_nu_svr(
 
 	Solver_NU s;
 	s.Solve(2*l, SVR_Q(*prob,*param), linear_term, y,
-		alpha2, C, param->eps, si, param->shrinking);
+		alpha2, C, param->eps, si, param->shrinking, param->max_iter);
 
 	info("epsilon = %f\n",-si->r);
 
@@ -1820,7 +1831,7 @@ struct decision_function
 
 static decision_function svm_train_one(
 	const PREFIX(problem) *prob, const svm_parameter *param,
-	double Cp, double Cn)
+	double Cp, double Cn, int *status)
 {
 	double *alpha = Malloc(double,prob->l);
 	Solver::SolutionInfo si;
@@ -1848,6 +1859,8 @@ static decision_function svm_train_one(
  			break;
 	}
 
+        *status |= si.solve_timed_out;
+
 	info("obj = %f, rho = %f\n",si.obj,si.rho);
 
 	// output SVs
@@ -2072,7 +2085,7 @@ static void multiclass_probability(int k, double **r, double *p)
 // Cross-validation decision values for probability estimates
 static void svm_binary_svc_probability(
 	const PREFIX(problem) *prob, const svm_parameter *param,
-	double Cp, double Cn, double& probA, double& probB)
+	double Cp, double Cn, double& probA, double& probB, int * status)
 {
 	int i;
 	int nr_fold = 5;
@@ -2145,7 +2158,7 @@ static void svm_binary_svc_probability(
 			subparam.weight_label[1]=-1;
 			subparam.weight[0]=Cp;
 			subparam.weight[1]=Cn;
-			struct PREFIX(model) *submodel = PREFIX(train)(&subprob,&subparam);
+			struct PREFIX(model) *submodel = PREFIX(train)(&subprob,&subparam, status);
 			for(j=begin;j<end;j++)
 			{
 #ifdef _DENSE_REP
@@ -2323,7 +2336,8 @@ static void remove_zero_weight(PREFIX(problem) *newprob, const PREFIX(problem) *
 //
 // Interface functions
 //
-PREFIX(model) *PREFIX(train)(const PREFIX(problem) *prob, const svm_parameter *param)
+PREFIX(model) *PREFIX(train)(const PREFIX(problem) *prob, const svm_parameter *param,
+        int *status)
 {
 	PREFIX(problem) newprob;
 	remove_zero_weight(&newprob, prob);
@@ -2352,7 +2366,7 @@ PREFIX(model) *PREFIX(train)(const PREFIX(problem) *prob, const svm_parameter *p
 			model->probA[0] = NAMESPACE::svm_svr_probability(prob,param);
 		}
 
-                NAMESPACE::decision_function f = NAMESPACE::svm_train_one(prob,param,0,0);
+                NAMESPACE::decision_function f = NAMESPACE::svm_train_one(prob,param,0,0, status);
 		model->rho = Malloc(double,1);
 		model->rho[0] = f.rho;
 
@@ -2467,9 +2481,9 @@ PREFIX(model) *PREFIX(train)(const PREFIX(problem) *prob, const svm_parameter *p
 				}
 
 				if(param->probability)
-                                    NAMESPACE::svm_binary_svc_probability(&sub_prob,param,weighted_C[i],weighted_C[j],probA[p],probB[p]);
+                                    NAMESPACE::svm_binary_svc_probability(&sub_prob,param,weighted_C[i],weighted_C[j],probA[p],probB[p], status);
 
-				f[p] = NAMESPACE::svm_train_one(&sub_prob,param,weighted_C[i],weighted_C[j]);
+				f[p] = NAMESPACE::svm_train_one(&sub_prob,param,weighted_C[i],weighted_C[j], status);
 				for(k=0;k<ci;k++)
 					if(!nonzero[si+k] && fabs(f[p].alpha[k]) > 0)
 						nonzero[si+k] = true;
@@ -2703,7 +2717,8 @@ void PREFIX(cross_validation)(const PREFIX(problem) *prob, const svm_parameter *
 			subprob.W[k] = prob->W[perm[j]];
 			++k;
 		}
-		struct PREFIX(model) *submodel = PREFIX(train)(&subprob,param);
+                int dummy_status = 0; // IGNORES TIMEOUT ERRORS
+		struct PREFIX(model) *submodel = PREFIX(train)(&subprob,param, &dummy_status);
 		if(param->probability && 
 		   (param->svm_type == C_SVC || param->svm_type == NU_SVC))
 		{
