@@ -1,22 +1,29 @@
+import warnings
 import numpy as np
 import numpy.linalg as la
 import scipy.sparse as sp
 
-from numpy.testing import assert_almost_equal
-from numpy.testing import assert_array_almost_equal
-from numpy.testing import assert_array_equal
-from numpy.testing import assert_equal
-
-from nose.tools import assert_raises, assert_true, assert_false
+from sklearn.utils.testing import assert_almost_equal
+from sklearn.utils.testing import assert_array_almost_equal
+from sklearn.utils.testing import assert_array_equal
+from sklearn.utils.testing import assert_equal
+from sklearn.utils.testing import assert_raises
+from sklearn.utils.testing import assert_true
+from sklearn.utils.testing import assert_false
 
 from sklearn.utils.sparsefuncs import mean_variance_axis0
 from sklearn.preprocessing import Binarizer
 from sklearn.preprocessing import KernelCenterer
 from sklearn.preprocessing import LabelBinarizer
+from sklearn.preprocessing import OneHotEncoder
+from sklearn.preprocessing import LabelEncoder
 from sklearn.preprocessing import Normalizer
 from sklearn.preprocessing import normalize
-from sklearn.preprocessing import Scaler
+from sklearn.preprocessing import StandardScaler
 from sklearn.preprocessing import scale
+from sklearn.preprocessing import MinMaxScaler
+from sklearn.preprocessing import add_dummy_feature
+from sklearn.preprocessing import balance_weights
 
 from sklearn import datasets
 from sklearn.linear_model.stochastic_gradient import SGDClassifier
@@ -36,7 +43,7 @@ def test_scaler_1d():
     X = rng.randn(5)
     X_orig_copy = X.copy()
 
-    scaler = Scaler()
+    scaler = StandardScaler()
     X_scaled = scaler.fit(X).transform(X, copy=False)
     assert_array_almost_equal(X_scaled.mean(axis=0), 0.0)
     assert_array_almost_equal(X_scaled.std(axis=0), 1.0)
@@ -47,7 +54,7 @@ def test_scaler_1d():
 
     # Test with 1D list
     X = [0., 1., 2, 0.4, 1.]
-    scaler = Scaler()
+    scaler = StandardScaler()
     X_scaled = scaler.fit(X).transform(X, copy=False)
     assert_array_almost_equal(X_scaled.mean(axis=0), 0.0)
     assert_array_almost_equal(X_scaled.std(axis=0), 1.0)
@@ -63,13 +70,13 @@ def test_scaler_2d_arrays():
     X = rng.randn(4, 5)
     X[:, 0] = 0.0  # first feature is always of zero
 
-    scaler = Scaler()
+    scaler = StandardScaler()
     X_scaled = scaler.fit(X).transform(X, copy=True)
     assert_false(np.any(np.isnan(X_scaled)))
 
     assert_array_almost_equal(X_scaled.mean(axis=0), 5 * [0.0])
     assert_array_almost_equal(X_scaled.std(axis=0), [0., 1., 1., 1., 1.])
-    # Check that X has not been copied
+    # Check that X has been copied
     assert_true(X_scaled is not X)
 
     # check inverse transform
@@ -97,7 +104,7 @@ def test_scaler_2d_arrays():
 
     X = rng.randn(4, 5)
     X[:, 0] = 1.0  # first feature is a constant, non zero feature
-    scaler = Scaler()
+    scaler = StandardScaler()
     X_scaled = scaler.fit(X).transform(X, copy=True)
     assert_false(np.any(np.isnan(X_scaled)))
     assert_array_almost_equal(X_scaled.mean(axis=0), 5 * [0.0])
@@ -106,23 +113,90 @@ def test_scaler_2d_arrays():
     assert_true(X_scaled is not X)
 
 
+def test_min_max_scaler_iris():
+    X = iris.data
+    scaler = MinMaxScaler()
+    # default params
+    X_trans = scaler.fit_transform(X)
+    assert_array_equal(X_trans.min(axis=0), 0)
+    assert_array_equal(X_trans.min(axis=0), 0)
+    assert_array_equal(X_trans.max(axis=0), 1)
+    X_trans_inv = scaler.inverse_transform(X_trans)
+    assert_array_almost_equal(X, X_trans_inv)
+
+    # not default params
+    scaler = MinMaxScaler(feature_range=(1, 2))
+    X_trans = scaler.fit_transform(X)
+    assert_array_equal(X_trans.min(axis=0), 1)
+    assert_array_equal(X_trans.max(axis=0), 2)
+    X_trans_inv = scaler.inverse_transform(X_trans)
+    assert_array_almost_equal(X, X_trans_inv)
+
+    # raises on invalid range
+    scaler = MinMaxScaler(feature_range=(2, 1))
+    assert_raises(ValueError, scaler.fit, X)
+
+
+def test_min_max_scaler_zero_variance_features():
+    """Check min max scaler on toy data with zero variance features"""
+    X = [[0.,  1.,  0.5],
+         [0.,  1., -0.1],
+         [0.,  1.,  1.1]]
+
+    X_new = [[+0.,  2.,  0.5],
+             [-1.,  1.,  0.0],
+             [+0.,  1.,  1.5]]
+
+    # default params
+    scaler = MinMaxScaler()
+    X_trans = scaler.fit_transform(X)
+    X_expected_0_1 = [[0.,  0.,  0.5],
+                      [0.,  0.,  0.0],
+                      [0.,  0.,  1.0]]
+    assert_array_almost_equal(X_trans, X_expected_0_1)
+    X_trans_inv = scaler.inverse_transform(X_trans)
+    assert_array_almost_equal(X, X_trans_inv)
+
+    X_trans_new = scaler.transform(X_new)
+    X_expected_0_1_new = [[+0.,  1.,  0.500],
+                          [-1.,  0.,  0.083],
+                          [+0.,  0.,  1.333]]
+    assert_array_almost_equal(X_trans_new, X_expected_0_1_new, decimal=2)
+
+    # not default params
+    scaler = MinMaxScaler(feature_range=(1, 2))
+    X_trans = scaler.fit_transform(X)
+    X_expected_1_2 = [[1.,  1.,  1.5],
+                      [1.,  1.,  1.0],
+                      [1.,  1.,  2.0]]
+    assert_array_almost_equal(X_trans, X_expected_1_2)
+
+
 def test_scaler_without_centering():
     rng = np.random.RandomState(42)
     X = rng.randn(4, 5)
     X[:, 0] = 0.0  # first feature is always of zero
     X_csr = sp.csr_matrix(X)
+    X_csc = sp.csc_matrix(X)
 
-    scaler = Scaler(with_mean=False).fit(X)
+    scaler = StandardScaler(with_mean=False).fit(X)
     X_scaled = scaler.transform(X, copy=True)
     assert_false(np.any(np.isnan(X_scaled)))
 
-    scaler_csr = Scaler(with_mean=False).fit(X_csr)
+    scaler_csr = StandardScaler(with_mean=False).fit(X_csr)
     X_csr_scaled = scaler_csr.transform(X_csr, copy=True)
     assert_false(np.any(np.isnan(X_csr_scaled.data)))
 
+    scaler_csc = StandardScaler(with_mean=False).fit(X_csc)
+    X_csc_scaled = scaler_csr.transform(X_csc, copy=True)
+    assert_false(np.any(np.isnan(X_csc_scaled.data)))
+
     assert_equal(scaler.mean_, scaler_csr.mean_)
     assert_array_almost_equal(scaler.std_, scaler_csr.std_)
 
+    assert_equal(scaler.mean_, scaler_csc.mean_)
+    assert_array_almost_equal(scaler.std_, scaler_csc.std_)
+
     assert_array_almost_equal(
         X_scaled.mean(axis=0), [0., -0.01,  2.24, -0.35, -0.78], 2)
     assert_array_almost_equal(X_scaled.std(axis=0), [0., 1., 1., 1., 1.])
@@ -143,7 +217,28 @@ def test_scaler_without_centering():
     X_csr_scaled_back = scaler_csr.inverse_transform(X_csr_scaled)
     assert_true(X_csr_scaled_back is not X_csr)
     assert_true(X_csr_scaled_back is not X_csr_scaled)
-    assert_array_almost_equal(X_scaled_back, X)
+    assert_array_almost_equal(X_csr_scaled_back.toarray(), X)
+
+    X_csc_scaled_back = scaler_csr.inverse_transform(X_csc_scaled.tocsc())
+    assert_true(X_csc_scaled_back is not X_csc)
+    assert_true(X_csc_scaled_back is not X_csc_scaled)
+    assert_array_almost_equal(X_csc_scaled_back.toarray(), X)
+
+
+def test_scaler_without_copy():
+    """Check that StandardScaler.fit does not change input"""
+    rng = np.random.RandomState(42)
+    X = rng.randn(4, 5)
+    X[:, 0] = 0.0  # first feature is always of zero
+    X_csr = sp.csr_matrix(X)
+
+    X_copy = X.copy()
+    StandardScaler(copy=False).fit(X)
+    assert_array_equal(X, X_copy)
+
+    X_csr_copy = X_csr.copy()
+    StandardScaler(with_mean=False, copy=False).fit(X_csr)
+    assert_array_equal(X_csr.toarray(), X_csr_copy.toarray())
 
 
 def test_scale_sparse_with_mean_raise_exception():
@@ -153,10 +248,10 @@ def test_scale_sparse_with_mean_raise_exception():
 
     # check scaling and fit with direct calls on sparse data
     assert_raises(ValueError, scale, X_csr, with_mean=True)
-    assert_raises(ValueError, Scaler(with_mean=True).fit, X_csr)
+    assert_raises(ValueError, StandardScaler(with_mean=True).fit, X_csr)
 
     # check transform and inverse_transform after a fit on a dense array
-    scaler = Scaler(with_mean=True).fit(X)
+    scaler = StandardScaler(with_mean=True).fit(X)
     assert_raises(ValueError, scaler.transform, X_csr)
 
     X_transformed_csr = sp.csr_matrix(scaler.transform(X))
@@ -175,8 +270,15 @@ def test_scale_function_without_centering():
     X_csr_scaled = scale(X_csr, with_mean=False)
     assert_false(np.any(np.isnan(X_csr_scaled.data)))
 
-    assert_array_almost_equal(
-        X_scaled.mean(axis=0), [0., -0.01,  2.24, -0.35, -0.78], 2)
+    # test csc has same outcome
+    X_csc_scaled = scale(X_csr.tocsc(), with_mean=False)
+    assert_array_almost_equal(X_scaled, X_csc_scaled.toarray())
+
+    # raises value error on axis != 0
+    assert_raises(ValueError, scale, X_csr, with_mean=False, axis=1)
+
+    assert_array_almost_equal(X_scaled.mean(axis=0),
+                              [0., -0.01,  2.24, -0.35, -0.78], 2)
     assert_array_almost_equal(X_scaled.std(axis=0), [0., 1., 1., 1., 1.])
     # Check that X has not been copied
     assert_true(X_scaled is not X)
@@ -186,6 +288,20 @@ def test_scale_function_without_centering():
     assert_array_almost_equal(X_csr_scaled_std, X_scaled.std(axis=0))
 
 
+def test_warning_scaling_integers():
+    """Check warning when scaling integer data"""
+    X = np.array([[1, 2, 0],
+                  [0, 0, 0]], dtype=np.uint8)
+
+    with warnings.catch_warnings(record=True) as w:
+        StandardScaler().fit(X)
+        assert_equal(len(w), 1)
+
+    with warnings.catch_warnings(record=True) as w:
+        MinMaxScaler().fit(X)
+        assert_equal(len(w), 1)
+
+
 def test_normalizer_l1():
     rng = np.random.RandomState(0)
     X_dense = rng.randn(4, 5)
@@ -292,7 +408,7 @@ def test_normalize_errors():
 def test_binarizer():
     X_ = np.array([[1, 0, 5], [2, 3, 0]])
 
-    for init in (np.array, sp.csr_matrix):
+    for init in (np.array, sp.csr_matrix, sp.csc_matrix):
 
         X = init(X_.copy())
 
@@ -300,6 +416,8 @@ def test_binarizer():
         X_bin = toarray(binarizer.transform(X))
         assert_equal(np.sum(X_bin == 0), 4)
         assert_equal(np.sum(X_bin == 1), 2)
+        X_bin = binarizer.transform(X)
+        assert_equal(type(X), type(X_bin))
 
         binarizer = Binarizer(copy=True).fit(X)
         X_bin = toarray(binarizer.transform(X))
@@ -326,14 +444,14 @@ def test_label_binarizer():
     lb = LabelBinarizer()
 
     # two-class case
-    inp = np.array([0, 1, 1, 0])
+    inp = ["neg", "pos", "pos", "neg"]
     expected = np.array([[0, 1, 1, 0]]).T
     got = lb.fit_transform(inp)
     assert_array_equal(expected, got)
     assert_array_equal(lb.inverse_transform(got), inp)
 
     # multi-class case
-    inp = np.array([3, 2, 1, 2, 0])
+    inp = ["spam", "ham", "eggs", "ham", "0"]
     expected = np.array([[0, 0, 0, 1],
                          [0, 0, 1, 0],
                          [0, 1, 0, 0],
@@ -413,6 +531,102 @@ def test_label_binarizer_errors():
     assert_raises(ValueError, LabelBinarizer, neg_label=2, pos_label=2)
 
 
+def test_one_hot_encoder():
+    """Test OneHotEncoder's fit and transform."""
+    X = [[3, 2, 1], [0, 1, 1]]
+    enc = OneHotEncoder()
+    # discover max values automatically
+    X_trans = enc.fit_transform(X).toarray()
+    assert_equal(X_trans.shape, (2, 5))
+    assert_array_equal(enc.active_features_,
+                       np.where([1, 0, 0, 1, 0, 1, 1, 0, 1])[0])
+    assert_array_equal(enc.feature_indices_, [0, 4, 7, 9])
+
+    # check outcome
+    assert_array_equal(X_trans,
+                       [[0., 1., 0., 1., 1.],
+                        [1., 0., 1., 0., 1.]])
+
+    # max value given as 3
+    enc = OneHotEncoder(n_values=4)
+    X_trans = enc.fit_transform(X)
+    assert_equal(X_trans.shape, (2, 4 * 3))
+    assert_array_equal(enc.feature_indices_, [0, 4, 8, 12])
+
+    # max value given per feature
+    enc = OneHotEncoder(n_values=[3, 2, 2])
+    X = [[1, 0, 1], [0, 1, 1]]
+    X_trans = enc.fit_transform(X)
+    assert_equal(X_trans.shape, (2, 3 + 2 + 2))
+    assert_array_equal(enc.n_values_, [3, 2, 2])
+    # check that testing with larger feature works:
+    X = np.array([[2, 0, 1], [0, 1, 1]])
+    enc.transform(X)
+
+    # test that an error is raise when out of bounds:
+    X_too_large = [[0, 2, 1], [0, 1, 1]]
+    assert_raises(ValueError, enc.transform, X_too_large)
+
+    # test that error is raised when wrong number of features
+    assert_raises(ValueError, enc.transform, X[:, :-1])
+    # test that error is raised when wrong number of features in fit
+    # with prespecified n_values
+    assert_raises(ValueError, enc.fit, X[:, :-1])
+    # test exception on wrong init param
+    assert_raises(TypeError, OneHotEncoder(n_values=np.int).fit, X)
+
+    enc = OneHotEncoder()
+    # test negative input to fit
+    assert_raises(ValueError, enc.fit, [[0], [-1]])
+
+    # test negative input to transform
+    enc.fit([[0], [1]])
+    assert_raises(ValueError, enc.transform, [[0], [-1]])
+
+
+def test_label_encoder():
+    """Test LabelEncoder's transform and inverse_transform methods"""
+    le = LabelEncoder()
+    le.fit([1, 1, 4, 5, -1, 0])
+    assert_array_equal(le.classes_, [-1, 0, 1, 4, 5])
+    assert_array_equal(le.transform([0, 1, 4, 4, 5, -1, -1]),
+                       [1, 2, 3, 3, 4, 0, 0])
+    assert_array_equal(le.inverse_transform([1, 2, 3, 3, 4, 0, 0]),
+                       [0, 1, 4, 4, 5, -1, -1])
+    assert_raises(ValueError, le.transform, [0, 6])
+
+
+def test_label_encoder_fit_transform():
+    """Test fit_transform"""
+    le = LabelEncoder()
+    ret = le.fit_transform([1, 1, 4, 5, -1, 0])
+    assert_array_equal(ret, [2, 2, 3, 4, 0, 1])
+
+    le = LabelEncoder()
+    ret = le.fit_transform(["paris", "paris", "tokyo", "amsterdam"])
+    assert_array_equal(ret, [1, 1, 2, 0])
+
+
+def test_label_encoder_string_labels():
+    """Test LabelEncoder's transform and inverse_transform methods with
+    non-numeric labels"""
+    le = LabelEncoder()
+    le.fit(["paris", "paris", "tokyo", "amsterdam"])
+    assert_array_equal(le.classes_, ["amsterdam", "paris", "tokyo"])
+    assert_array_equal(le.transform(["tokyo", "tokyo", "paris"]),
+                       [2, 2, 1])
+    assert_array_equal(le.inverse_transform([2, 2, 1]),
+                       ["tokyo", "tokyo", "paris"])
+    assert_raises(ValueError, le.transform, ["london"])
+
+
+def test_label_encoder_errors():
+    """Check that invalid arguments yield ValueError"""
+    le = LabelEncoder()
+    assert_raises(ValueError, le.transform, [])
+    assert_raises(ValueError, le.inverse_transform, [])
+
+
 def test_label_binarizer_iris():
     lb = LabelBinarizer()
     Y = lb.fit_transform(iris.target)
@@ -433,14 +647,15 @@ def test_label_binarizer_multilabel_unlabeled():
     Y = np.array([[1, 1],
                   [1, 0],
                   [0, 0]])
-    assert_equal(lb.fit_transform(y), Y)
+    assert_array_equal(lb.fit_transform(y), Y)
 
 
 def test_center_kernel():
-    """Test that KernelCenterer is equivalent to Scaler in feature space"""
+    """Test that KernelCenterer is equivalent to StandardScaler
+       in feature space"""
     rng = np.random.RandomState(0)
     X_fit = rng.random_sample((5, 4))
-    scaler = Scaler(with_std=False)
+    scaler = StandardScaler(with_std=False)
     scaler.fit(X_fit)
     X_fit_centered = scaler.transform(X_fit)
     K_fit = np.dot(X_fit, X_fit.T)
@@ -463,7 +678,45 @@ def test_center_kernel():
 def test_fit_transform():
     rng = np.random.RandomState(0)
     X = rng.random_sample((5, 4))
-    for obj in ((Scaler(), Normalizer(), Binarizer())):
+    for obj in ((StandardScaler(), Normalizer(), Binarizer())):
         X_transformed = obj.fit(X).transform(X)
         X_transformed2 = obj.fit_transform(X)
         assert_array_equal(X_transformed, X_transformed2)
+
+
+def test_add_dummy_feature():
+    X = [[1, 0], [0, 1], [0, 1]]
+    X = add_dummy_feature(X)
+    assert_array_equal(X, [[1, 1, 0], [1, 0, 1], [1, 0, 1]])
+
+
+def test_add_dummy_feature_coo():
+    X = sp.coo_matrix([[1, 0], [0, 1], [0, 1]])
+    X = add_dummy_feature(X)
+    assert_true(sp.isspmatrix_coo(X), X)
+    assert_array_equal(X.toarray(), [[1, 1, 0], [1, 0, 1], [1, 0, 1]])
+
+
+def test_add_dummy_feature_csc():
+    X = sp.csc_matrix([[1, 0], [0, 1], [0, 1]])
+    X = add_dummy_feature(X)
+    assert_true(sp.isspmatrix_csc(X), X)
+    assert_array_equal(X.toarray(), [[1, 1, 0], [1, 0, 1], [1, 0, 1]])
+
+
+def test_add_dummy_feature_csr():
+    X = sp.csr_matrix([[1, 0], [0, 1], [0, 1]])
+    X = add_dummy_feature(X)
+    assert_true(sp.isspmatrix_csr(X), X)
+    assert_array_equal(X.toarray(), [[1, 1, 0], [1, 0, 1], [1, 0, 1]])
+
+
+def test_balance_weights():
+    weights = balance_weights([0, 0, 1, 1])
+    assert_array_equal(weights, [1., 1., 1., 1.])
+
+    weights = balance_weights([0, 1, 1, 1, 1])
+    assert_array_equal(weights, [1., 0.25, 0.25, 0.25, 0.25])
+
+    weights = balance_weights([0, 0])
+    assert_array_equal(weights, [1., 1.])
